#!/usr/bin/env tclsh
#
# Copyright (C) 2003-2004 Steven Ness and Navraj S. Pannu, Leiden University
# Copyright (C) 2004-2007 Navraj S. Pannu, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

set debug 0

global tcl_platform
global system
global env
set system(OPSYS) [string toupper $tcl_platform(platform)]
#source [file join $env(CRANK) bin system.tcl]
#source [file join $env(CRANK) bin xml_utils.tcl]
source [file join $env(CCP4I_TOP) src system.tcl]
source [file join $env(CCP4I_TOP) utils xml_utils.tcl]

proc join_all_sftools { nextstep } {
    global XMLParse

# When we have enantiomorphic spacegroups, the spacegroup can
# change.  This is change is transferred through the crank via mtz
# files.  When sftools writes an mtz file, it takes the spacegroup of
# the first mtz file read in.  Hence, look for columns that are created
# on the last crank jobs first.
# ***NSP this is a rather bad hack.

    set script ""
    set mtz 0

    # Check for <phase_columns>
    if { [info exists XMLParse([join "crank soap run step=$nextstep input phase_columns phib" __])] } {
 	set input [string trim $XMLParse([join "crank soap run step=$nextstep input phase_columns phib" __])]
	if { [regexp {([0-9_A-Za-z]*)_PHIB} $input all id] } {
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_F"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_SIGF"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_PHIB"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_FOM"]\n "
	    set sftools_script "$sftools_script quit\n"
	    set command "sftools << \"$sftools_script\""
#           puts $command
	    catch {eval exec $command } output 
#           puts $output
#           puts "<phase_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match phase_columns=($input)"
	}
    }

    # Check for <hl_columns>
    if { [info exists XMLParse([join "crank soap run step=$nextstep input hl_columns hla" __])] } {
	set input [string trim $XMLParse([join "crank soap run step=$nextstep input hl_columns hla" __])]
	if { [regexp {([0-9_A-Za-z]*)_HLA} $input all id] } {
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_HLA"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_HLB"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_HLC"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_HLD"]\n"
	    set sftools_script "$sftools_script quit\n"
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
#	    puts "<hl_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match hl_columns=($input)"
	}
    }

    # Check for <f_columns>
    if { [info exists XMLParse([join "crank soap run step=$nextstep input f_columns f" __])] } {
	set input [string trim $XMLParse([join "crank soap run step=$nextstep input f_columns f" __])]
	if { [regexp {([0-9_A-Za-z]*)_X1_D1_F} $input all id] } {
#	    puts "<f_columns>::appending($id)."
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_X1_D1_F"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_X1_D1_SIGF"]\n"
	    set sftools_script "$sftools_script quit\n"
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
	} elseif { [regexp {([0-9_A-Za-z]*)_F} $input all id] } {
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_F"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_SIGF"]\n"
	    set sftools_script "$sftools_script quit\n"	    
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
#	    puts "<f_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match f_columns=($input)"
	}
    }

    # Check for <i_columns>
    if { [info exists XMLParse([join "crank soap run step=$nextstep input i_columns i" __])] } {
	set input [string trim $XMLParse([join "crank soap run step=$nextstep input i_columns i" __])]
	if { [regexp {([0-9_A-Za-z]*)_I} $input all id] } {
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_I"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_SIGI"]\n"
	    set sftools_script "$sftools_script quit\n"	    
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
#	    puts "<i_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match i_columns=($input)"
	}
    }
	
    # Check for <ea_columns>
     if { [info exists XMLParse([join "crank soap run step=$nextstep input ea_columns ea" __])] } {
	 set input [string trim $XMLParse([join "crank soap run step=$nextstep input ea_columns ea" __])]
	 if { [regexp {([0-9_A-Za-z]*)_EA} $input all id] } {
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_EA"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_SIGEA"]\n"
	    set sftools_script "$sftools_script quit\n"	    
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
#	    puts "<ea_columns>::appending($id)."
	 } else {
	     crank_error "crankutils.tcl::Could not match ea_columns=($input)"
	 }
     } 

    # Check for <fa_columns>
    if { [info exists XMLParse([join "crank soap run step=$nextstep input fa_columns fa" __])] } {
	set input [string trim $XMLParse([join "crank soap run step=$nextstep input fa_columns fa" __])]
	if { [regexp {([0-9_A-Za-z]*)_FA} $input all id] } {
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_FA"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_SIGFA"]\n"
	    set sftools_script "$sftools_script quit\n"
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
#	    puts "<fa_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match fa_columns=($input)"
	}
    }
	
    # Check for <alpha_columns>
    if { [info exists XMLParse([join "crank soap run step=$nextstep input alpha_columns alpha" __])] } {
	set input [string trim $XMLParse([join "crank soap run step=$nextstep input alpha_columns alpha" __])]
	if { [regexp {([0-9_A-Za-z]*)_ALPHA} $input all id] } {
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_ALPHA"]\n"
	    set sftools_script "$sftools_script quit\n"
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
#	    puts "<alpha_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match alpha_columns=($input)"
	}
    }

    # Check for <exp_columns> amplitude columns
    if { [info exists XMLParse([join "crank soap run step=$nextstep input exp_columns crystal=1 data=1 f" __])] } {
	set inputf [string trim $XMLParse([join "crank soap run step=$nextstep input exp_columns crystal=1 data=1 f" __])]
	if { [regexp {([0-9_A-Za-z]*)_X1_D1_F} $inputf all id] } {
	    set i 1
	    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } { 
		set j 1
		while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } { 
		    incr mtz
		    set script "$script read $mtz.mtz\n"
		    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "

		    set id2 [format "%s%s%s%s%s" $id "_X" $i "_D" $j] 

		    set sftools_script "$sftools_script [format "%s%s" $id2 "_F"]"
		    set sftools_script "$sftools_script [format "%s%s" $id2 "_SIGF"]"

		    if { $XMLParse([join "crank parameters crystal=$i data=$j anomalous" __]) == "1" } {
			set sftools_script "$sftools_script [format "%s%s" $id2 "_F+"]"
			set sftools_script "$sftools_script [format "%s%s" $id2 "_SIGF+"]"
			set sftools_script "$sftools_script [format "%s%s" $id2 "_F-"]"
			set sftools_script "$sftools_script [format "%s%s" $id2 "_SIGF-"]\n"
		    } else {
			set sftools_script "$sftools_script \n"
		    }
		    set command "sftools << \"$sftools_script\""
#		    puts $command
		    catch {eval exec $command } output 
#		    puts $output

		    incr j
		}
		incr i
	    }
#	    puts "<exp_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match exp_columns=($inputf)"
	}
    }

    # Check for <exp_columns> intensity columns
    set expi 0

    if { [info exists XMLParse([join "crank soap run step=$nextstep input exp_columns crystal=1 data=1 i" __])] } {
	set inputi [string trim $XMLParse([join "crank soap run step=$nextstep input exp_columns crystal=1 data=1 i" __])]
	if { [regexp {([0-9_A-Za-z]*)_X1_D1_I} $inputi all id] } {
	    set expi 1
	} else {
	    crank_error "crankutils.tcl::Could not match exp_columns=($inputi)"
	}

    } elseif { [info exists XMLParse([join "crank soap run step=$nextstep input exp_columns crystal=1 data=1 i_plus" __])] } {
	set inputi [string trim $XMLParse([join "crank soap run step=$nextstep input exp_columns crystal=1 data=1 i_plus" __])]
	if { [regexp {([0-9_A-Za-z]*)_X1_D1_I+} $inputi all id] } {
	    set expi 1
	} else {
	    crank_error "crankutils.tcl::Could not match exp_columns=($inputi)"
	}
    }

    if { $expi == 1 } {
	set i 1
	while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } { 
	    set j 1
	    while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } { 
		incr mtz
		set script "$script read $mtz.mtz\n"
		set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "

		set id2 [format "%s%s%s%s%s" $id "_X" $i "_D" $j] 

		if { [string compare $XMLParse([join "crank parameters crystal=$i data=$j anomalous" __]) "0"] == 0 } {
		    set sftools_script "$sftools_script [format "%s%s" $id2 "_I"]"
		    set sftools_script "$sftools_script [format "%s%s" $id2 "_SIGI"]\n"
		} else {
		    set sftools_script "$sftools_script [format "%s%s" $id2 "_I+"]"
		    set sftools_script "$sftools_script [format "%s%s" $id2 "_SIGI+"]"
		    set sftools_script "$sftools_script [format "%s%s" $id2 "_I-"]"
		    set sftools_script "$sftools_script [format "%s%s" $id2 "_SIGI-"]\n"
		}
		set command "sftools << \"$sftools_script\""
#		puts $command
		catch {eval exec $command } output 
#		puts $output
#		puts "<exp_columns>::appending($id)."
	 	incr j
	    }
	    incr i
	}
    }
	
    # Check for <phic_columns>
    if { [info exists XMLParse([join "crank soap run step=$nextstep input phic_columns fc" __])] } {
	set input [string trim $XMLParse([join "crank soap run step=$nextstep input phic_columns fc" __])]
	if { [regexp {([0-9_A-Za-z]*)_FC} $input all id] } {
	    incr mtz
	    set script "$script read $mtz.mtz\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script [format "%s%s" $id "_FC"]"
	    set sftools_script "$sftools_script [format "%s%s" $id "_PHIC"]\n"
	    set sftools_script "$sftools_script quit\n"
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
#	    puts "<phic_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match phic_columns=($input)"
	}
    }
    
    # Check for <freer_columns>
    if { [info exists XMLParse([join "crank soap run step=$nextstep input freer_columns freer" __])] } {
	set input [string trim $XMLParse([join "crank soap run step=$nextstep input freer_columns freer" __])]
	if { [regexp {([0-9_A-Za-z]*)_FREER} $input all id] } {
	    incr mtz
	    set freertag [format "%s%s" $id "_FREER"]
	    set script "$script read $mtz.mtz\nselect col $freertag absent\ncalc col $freertag = 1\nselect all\n"
	    set sftools_script "read crank.out.$id.mtz\nwrite $mtz.mtz col "
	    set sftools_script "$sftools_script $freertag\n"
	    set sftools_script "$sftools_script quit\n"
	    set command "sftools << \"$sftools_script\""
#	    puts $command
	    catch {eval exec $command } output 
#	    puts $output
#	    puts "<freer_columns>::appending($id)."
	} else {
	    crank_error "crankutils.tcl::Could not match freer_columns=($input)"
	}
    }

    if { $script != "" } {
	set tag [string trim $XMLParse([join "crank soap run step=$nextstep tag" __])]
	set script "$script write crank.in.$tag.mtz\nquit\n"
	set command "sftools << \"$script\""
#	puts $command
	catch {eval exec $command } output 
#	puts $output
	for { set i 1 } { $i <= $mtz } { incr i } {
	    file delete $i.mtz
	}
    } else {
	puts "no columns to join."
    }
}

proc check_atom { input_pdb output_pdb } {
    global XMLParse
    
    # Some substructure detection programs do not know about the atoms
    # they are trying to find.  This routine will change the atom names
    # to the one's that are looked for.

    if { ![file exists $input_pdb] } {
	crank_error "crankutils.tcl::Input pdb does not exist"
    }

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i native" __])] } {
	if { $XMLParse([join "crank parameters crystal=$i native" __]) == "0"} {
	    if { [info exists XMLParse([join "crank parameters crystal=$i substructure atom_name" __])] } {
		set inputatom [string toupper $XMLParse([join "crank parameters crystal=$i substructure atom_name" __])] 
	    } else {
		crank_error "crankutils.tcl::no substructure atom"
	    }
	    if { [info exists XMLParse([join "crank parameters crystal=$i data=1 bfactor" __])] } {
		set b $XMLParse([join "crank parameters crystal=$i data=1 bfactor" __])  
 	    } else {
		set b 25.0
	    }
	}
	incr i
    }
    set input [open $input_pdb r]
    set indata [split [read $input] "\n"]
    set output [open $output_pdb w]

    foreach line  $indata {
	if { [regexp {HETATM.*} $line junk] || [regexp {ATOM.*} $line junk] } {
	    scan $line "%6s%5d%*c%*c%4s%*c%3s%*11s%8f%8f%8f%6f" label number atom residue x y z occ 
	    puts $output [format "HETATM %4d %2s   %3s   %3d    %8.3f%8.3f%8.3f%6.2f%6.2f" $number $inputatom $inputatom $number $x $y $z $occ $b]
	} else {
	    puts $output $line
	}
    }
    
    close $input
    close $output
}

proc orth2frac { orthx orthy orthz fracxin fracyin fraczin } {
    global XMLParse
    upvar $fracxin fracx
    upvar $fracyin fracy
    upvar $fraczin fracz

    set a     0.000000
    set b     0.000000
    set c     0.000000
    set alpha 0.000000
    set beta  0.000000
    set gamma 0.000000

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i native" __])] } {
	if { $XMLParse([join "crank parameters crystal=$i native" __]) == "0"  } {
    
	    set a $XMLParse([join "crank parameters crystal=$i cell cell_a" __])
	    set b $XMLParse([join "crank parameters crystal=$i cell cell_b" __])
	    set c $XMLParse([join "crank parameters crystal=$i cell cell_c" __])
	    set alpha $XMLParse([join "crank parameters crystal=$i cell cell_alpha" __])
	    set beta  $XMLParse([join "crank parameters crystal=$i cell cell_beta" __])
	    set gamma $XMLParse([join "crank parameters crystal=$i cell cell_gamma" __])
	    break
	}
	incr i
    }

    if { !($a > 0.0 ) } {
	crank_error "crankutils.tcl::orth2frac unit cell edges not determined correctly"
    }

    set degreetorad 0.01745329251994329576924
    set cosalpha [expr cos($alpha*$degreetorad) ]
    set cosbeta  [expr cos($beta*$degreetorad) ]
    set sinbeta  [expr sin($beta*$degreetorad) ]
    set cosgamma [expr cos($gamma*$degreetorad) ]
    set singamma [expr sin($gamma*$degreetorad) ]

    set arg1     [expr ($cosbeta*$cosgamma - $cosalpha)/($sinbeta*$singamma)]

    set arg2     [expr sqrt(1.00000000 - $arg1*$arg1)]

    set or2frac00 [expr 1.00000000/$a]
    set or2frac01 [expr -$cosgamma/($singamma*$a)]
    set or2frac02 [expr -(($cosgamma*$sinbeta*$arg1+$cosbeta*$singamma)/($sinbeta*$arg2*$singamma*$a)) ]

    set or2frac11 [expr 1.000000000/($singamma*$b)]
    set or2frac12 [expr $arg1/($arg2*$singamma*$b)]

    set or2frac22 [expr 1.00000000/($sinbeta*$arg2*$c)]

    set fracx     [expr $orthx*$or2frac00 + $orthy*$or2frac01 + $orthz*$or2frac02]

    set fracy     [expr                     $orthy*$or2frac11 + $orthz*$or2frac12]

    set fracz     [expr                                         $orthz*$or2frac22] 
}

proc frac2orth { fracx fracy fracz orthxin orthyin orthzin  } {
    global XMLParse
    upvar $orthxin orthx
    upvar $orthyin orthy
    upvar $orthzin orthz

    set a     0.000000
    set b     0.000000
    set c     0.000000
    set alpha 0.000000
    set beta  0.000000
    set gamma 0.000000

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } {
	if { [info exists XMLParse([join "crank parameters crystal=$i native" __])] &&
	     $XMLParse([join "crank parameters crystal=$i native" __]) == "0"  } {
    
	    set a $XMLParse([join "crank parameters crystal=$i cell cell_a" __])
	    set b $XMLParse([join "crank parameters crystal=$i cell cell_b" __])
	    set c $XMLParse([join "crank parameters crystal=$i cell cell_c" __])
	    set alpha $XMLParse([join "crank parameters crystal=$i cell cell_alpha" __])
	    set beta  $XMLParse([join "crank parameters crystal=$i cell cell_beta" __])
	    set gamma $XMLParse([join "crank parameters crystal=$i cell cell_gamma" __])
	}
	incr i
    }

    if { !($a > 0.0 ) } {
	crank_error "crankutils.tcl::orth2frac unit cell edges not determined correctly"
    }

    set degreetorad 0.01745329251994329576924
    set cosalpha [expr cos($alpha*$degreetorad) ]
    set cosbeta  [expr cos($beta*$degreetorad) ]
    set sinbeta  [expr sin($beta*$degreetorad) ]
    set cosgamma [expr cos($gamma*$degreetorad) ]
    set singamma [expr sin($gamma*$degreetorad) ]
    set arg1     [expr ($cosbeta*$cosgamma - $cosalpha)/($sinbeta*$singamma)]
    set arg2     [expr sqrt(1.0000000 - $arg1*$arg1)]

    set frac2or00 $a
    set frac2or01 [expr $cosgamma*$b]
    set frac2or02 [expr $cosbeta*$c ]

    set frac2or11 [expr $singamma*$b]
    set frac2or12 [expr -$sinbeta*$c*$arg1]

    set frac2or22 [expr $sinbeta*$c*$arg2]


    set orthx     [expr $fracx*$frac2or00 + $fracy*$frac2or01 + $fracz*$frac2or02]

    set orthy     [expr                     $fracy*$frac2or11 + $fracz*$frac2or12]

    set orthz     [expr                                         $fracz*$frac2or22]
 
}

proc spacegroup_from_mtz { inputmtz name number } {

    upvar $name namein
    upvar $number numberin

# give the spacegroup name and number of an mtz file
    set sftools_script "read $inputmtz \nlist \nquit \ny"
    set command "sftools << \"$sftools_script\""
    catch {eval exec $command } output

    if { ![regexp {.*Space group name    *: *([A-Z])* *([0-9]*) *([0-9]*) *([0-9]*).*} $output full namein n1 n2 n3] } {
	puts "Search string not found"
    }

    set namein "$namein$n1$n2$n3"
    if { ![regexp {.*Space group number  *: *([0-9]*).* } $output full numberin] } {
	puts "Search string not found"
    }
}

proc make_one_mtz_file { mtzout } {

    # store only input columns into one mtz file

    global XMLParse    

    if { [string compare $XMLParse([join "crank parameters input_intensity" __]) "1"] == 0 } {
	set intensity 1
    } else {
	set intensity 0
    }

    # check to see if we just have one big mtz
    set onemtz 0
    set multimtz 0
    set scain 0

    set i 1
    while { [info exists XMLParse([join "crank input_files file=$i type" __]) ] } {
	set type $XMLParse([join "crank input_files file=$i type" __])
 	if { [string compare [string toupper $type] "MTZ"] == 0 } {
	    if { [info exists XMLParse([join "crank input_files file=$i name" __]) ] } {
		set inputmtz $XMLParse([join "crank input_files file=$i name" __])
		if { ![file exists $inputmtz] } {
		    if { [file exists [file join $orig_pwd $mtzin]] } {
			set mtzin [file join $orig_pwd $mtzin]
		    } else {
			crank_error "Could not find inputfile ($inputfile)"
		    }
		} else {
		    set mtzin $inputmtz
		}
		set onemtz 1
		set mtzn $i
	    } elseif { [info exists XMLParse([join "crank input_files file=$i crystal=1 data=1 name" __]) ] } {
		set multimtz 1
		set mtzn $i
	    } else {
		crank_error "Could not find single or multiple mtz files"
	    }
	} elseif { [string compare [string toupper $type] "SCA"] == 0 } {
	    if { [info exists XMLParse([join "crank input_files file=$i crystal=1 data=1 name" __]) ] } {
		set scain $i
	    }
	}
	incr i
    }

    if { ( !($onemtz) && !($multimtz) && !($scain) ) } {
	crank_error "No reflection input files given"
    }

    set mtz 0
    set script ""

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } {
	set j 1
	while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } { 
 	    incr mtz	    
	    set sftoolsfile [open "sftools.script" w]
	    

	    if { $multimtz} {
		set mtzin $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j name" __])
	    } elseif { $scain != 0} {
		# call scalepack2mtz
		crank_error "No support for inputting scafiles at the moment"
	    }
	    
	    if { [string last $mtzin ".mtz"] != [expr [string length $mtzin] - 3 ] } {
	       puts $sftoolsfile "read $mtzin\nmtz\n"
            } else {
	       puts $sftoolsfile "read $mtzin\n"
            }

	    if { [string compare $XMLParse([join "crank parameters crystal=$i data=$j anomalous" __]) "0"] == 0 } { 

		if { $intensity == 1 } {
		    if { $scain } {
			set in_i "I"
			set in_sigi "SIGI"
		    } else {
			set in_i $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j i" __])
			set in_sigi $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j sigi" __])
		    }
		    puts $sftoolsfile "set label col \"$in_i\" \n IN_X${i}_D${j}_I"
		    puts $sftoolsfile "set label col \"$in_sigi\" \n IN_X${i}_D${j}_SIGI"
		    puts $sftoolsfile "purge nodata\ny\nreduce ccp4\n"
		    puts $sftoolsfile "write temp.mtz col IN_X${i}_D${j}_I IN_X${i}_D${j}_SIGI\nquit"
		} else {
 		    set in_f $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j f" __])
		    set in_sigf $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j sigf" __])
		    puts $sftoolsfile "set label col \"$in_f\" \n IN_X${i}_D${j}_F"
		    puts $sftoolsfile "set label col \"$in_sigf\" \n IN_X${i}_D${j}_SIGF"
		    puts $sftoolsfile "purge nodata\ny\nreduce ccp4\n"
		    puts $sftoolsfile "write temp.mtz col IN_X${i}_D${j}_F IN_X${i}_D${j}_SIGF\nquit"
		}
	    } else {
		if { $intensity == 1 } {
		    if { $scain } {
			set in_ip "I+"
			set in_sigip "SIGI+"
			set in_im "I-"
			set in_sigim "SIGI-"
		    } else {
			set in_ip $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j i_plus" __])
			set in_sigip $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j sigi_plus" __])
			set in_im $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j i_minus" __])
			set in_sigim $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j sigi_minus" __])
		    }
		    puts $sftoolsfile "calc J col IN_X${i}_D${j}_I = col \"$in_ip\" col \"$in_im\" + 0.5 *"
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGI = col \"$in_sigip\" 2 ** col \"$in_sigim\" 2 ** +"
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGI = col IN_X${i}_D${j}_SIGI 0.5 **"
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGI = col IN_X${i}_D${j}_SIGI 0.5 *"
		    puts $sftoolsfile "select col IN_X${i}_D${j}_I = absent"
		    puts $sftoolsfile "calc J col IN_X${i}_D${j}_I = col \"$in_ip\""
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGI = col \"$in_sigip\""
		    puts $sftoolsfile "select all"
		    puts $sftoolsfile "select col IN_X${i}_D${j}_I = absent"
		    puts $sftoolsfile "calc J col IN_X${i}_D${j}_I = col  \"$in_im\""
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGI = col \"$in_sigim\""
		    puts $sftoolsfile "select all"
		    puts $sftoolsfile "set label col \"$in_ip\""
		    puts $sftoolsfile "IN_X${i}_D${j}_I+"
		    puts $sftoolsfile "set label col \"$in_sigip\""
		    puts $sftoolsfile "IN_X${i}_D${j}_SIGI+ \n"
		    puts $sftoolsfile "set label col \"$in_im\""
		    puts $sftoolsfile "IN_X${i}_D${j}_I-"
		    puts $sftoolsfile "set label col \"$in_sigim\""
		    puts $sftoolsfile "IN_X${i}_D${j}_SIGI-"
		    puts $sftoolsfile "purge nodata\ny\nreduce ccp4\n"
		    puts $sftoolsfile "write temp.mtz col IN_X${i}_D${j}_I+ IN_X${i}_D${j}_SIGI+ IN_X${i}_D${j}_I- IN_X${i}_D${j}_SIGI-"
		    puts $sftoolsfile "write temp2.mtz col IN_X${i}_D${j}_I IN_X${i}_D${j}_SIGI\nquit"
		} else {
 		    set in_fp $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j f_plus" __])
		    set in_sigfp $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j sigf_plus" __])
 		    set in_fm $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j f_minus" __])
		    set in_sigfm $XMLParse([join "crank input_files file=$mtzn crystal=$i data=$j sigf_minus" __])
		    puts $sftoolsfile "calc F col IN_X${i}_D${j}_F = col \"$in_fp\" col \"$in_fm\" + 0.5 *"
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGF = col \"$in_sigfp\" 2 ** col \"$in_sigfm\" 2 ** +"
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGF = col IN_X${i}_D${j}_SIGF 0.5 **"
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGF = col IN_X${i}_D${j}_SIGF 0.5 *"
		    puts $sftoolsfile "select col IN_X${i}_D${j}_F = absent"
		    puts $sftoolsfile "calc F col IN_X${i}_D${j}_F = col \"$in_fp\""
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGF = col \"$in_sigfp\""
		    puts $sftoolsfile "select all"
		    puts $sftoolsfile "select col IN_X${i}_D${j}_F = absent"
		    puts $sftoolsfile "calc F col IN_X${i}_D${j}_F = col  \"$in_fm\""
		    puts $sftoolsfile "calc Q col IN_X${i}_D${j}_SIGF = col \"$in_sigfm\""
		    puts $sftoolsfile "select all"
		    puts $sftoolsfile "set label col \"$in_fp\" \n IN_X${i}_D${j}_F+"
		    puts $sftoolsfile "set label col \"$in_sigfp\" \n IN_X${i}_D${j}_SIGF+"
		    puts $sftoolsfile "set label col \"$in_fm\" \n IN_X${i}_D${j}_F-"
		    puts $sftoolsfile "set label col \"$in_sigfm\" \n IN_X${i}_D${j}_SIGF-"
		    puts $sftoolsfile "purge nodata\ny\nreduce ccp4\n"
		    puts $sftoolsfile "write temp.mtz col IN_X${i}_D${j}_F+ IN_X${i}_D${j}_SIGF+ IN_X${i}_D${j}_F- IN_X${i}_D${j}_SIGF-"
 		    puts $sftoolsfile "write temp2.mtz col IN_X${i}_D${j}_F IN_X${i}_D${j}_SIGF\nquit"
		}
	    }
	    close $sftoolsfile
	    set command "cat sftools.script | sftools"
	    catch {eval exec $command } output 
	    if { [file exists sftools.script] } {
	        file delete sftools.script
	    }
	    set command "sftools << \"read temp.mtz\nread temp2.mtz\npurge nodata\ny\nwrite $mtz.mtz\nquit\n\" "
	    catch {eval exec $command } output 
	    file delete temp.mtz
	    if { [file exists temp2.mtz] } {
		file delete temp2.mtz
	    }
	    set script "$script read $mtz.mtz\n"
	    incr j
	}
	incr i
    }

    set script "$script write $mtzout\nquit\n"
    set command "sftools << \"$script\""
    catch {eval exec $command } output 
    for { set i 1 } { $i <= $mtz } { incr i } {
	file delete $i.mtz
    }
}

proc spacegroup_enantiomorph { number hand } {

    upvar $hand enant

#  return enantiomorph space group

    set enant $number

    if { $number       == 76 }  {        
	# P41
	set enant 78
    } elseif { $number == 78 }  {        
	# P43
	set enant 76
    } elseif { $number == 91 }  { 
	# P4122
	set enant 95
    } elseif { $number == 95 }  {  
	# P4322
	set enant 91
    } elseif { $number == 92 }  {
	# P41212
	set enant 96
    } elseif { $number == 96 }  {
	# P43212
	set enant 92
    } elseif { $number == 144 } {
	# P31
	set enant 145 
    } elseif { $number == 145 } {
	# P32
	set enant 144 
    } elseif { $number == 151 } {
	# P3112
	set enant 153 
    } elseif { $number == 153 } {
	# P3212
	set enant 151 
    } elseif { $number == 152 } {
	# P3121
	set enant 154 
    } elseif { $number == 154 } {
	# P3221
	set enant 152 
    } elseif { $number == 169 } {
	# P61
	set enant 170 
    } elseif { $number == 170 } { 
        # P65
	set enant 169 
    } elseif { $number == 171 } {
	# P62
	set enant 172 
    } elseif { $number == 172 } {
	# P64
	set enant 171 
    } elseif { $number == 178 } {
	# P6122
	set enant 179 
    } elseif { $number == 179 } {
	# P6522
	set enant 178 
    } elseif { $number == 180 } {
	# P6222
	set enant 181 
    } elseif { $number == 181 } {
	# P6422
	set enant 180 
    } elseif { $number == 212 } { 
	# P4332
	set enant 213 
    } elseif { $number == 213 } {
	 # P4132
	set enant  212 
    }
}

proc arpwarp_asu { number xyz } {

#  return arpwarp asymmetric unit

    set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 1.00000"

    if { $number       == 1      } {
	# P1
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 1.00000"  
    } elseif { $number == 3      } { 
	# P2
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.50000"  
    } elseif { $number == 4      } { 
	# P21
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 5      } { 
	# C2
	set xyzlim "0.00000 0.50000 0.00000 1.00000 0.00000 0.50000"  
    } elseif { $number == 16     } { 
	# P222
	set xyzlim "0.00000 0.50000 0.00000 1.00000 0.00000 0.50000"  
    } elseif { $number == 17     } { 
	# P2221
	set xyzlim "0.00000 0.50000 0.00000 1.00000 0.00000 0.50000"  
    } elseif { $number == 18     } { 
	# P21212
	set xyzlim "0.00000 1.00000 0.00000 0.25000 0.00000 1.00000"  
    } elseif { $number == 19     } { 
	# P212121
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.25000"  
    } elseif { $number == 20     } {
	# C2221
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 21     } { 
	# C222
	set xyzlim "0.00000 0.25000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 22     } {
	# F222
	set xyzlim "0.00000 1.00000 0.00000 0.25000 0.00000 0.25000"  
    } elseif { $number == 23     } { 
	# I222
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 24     } { 
	# I212121
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 75     } { 
	# P4
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 76     } { 
	# P41
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 77     } { 
	# P42
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 78     } { 
	# P43
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 79     } { 
	# I4
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 80     } { 
	# I41
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.25000"  
    } elseif { $number == 89     } { 
	# P422
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 90     } {
	# P4212
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 91     } { 
	# P4122
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.12500"  
    } elseif { $number == 92     } { 
	# P41212
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 93     } { 
	# P4222
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.25000"  
    } elseif { $number == 94     } { 
	# P42212
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 95     } { 
	# P4322
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.12500"  
    } elseif { $number == 96     } { 
	# P43212
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 97     } { 
	# I422
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.25000"  
    } elseif { $number == 98     } { 
	# I4122
	set xyzlim "0.00000 1.00000 0.00000 0.25000 0.00000 0.25000"  
    } elseif { $number == 143    } { 
	# P3
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 1.00000"  
    } elseif { $number == 144    } {
	# P31
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.33334"  
    } elseif { $number == 145    } { 
	# P32
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.33334"  
    } elseif { $number == 146    } {
	# R3
	set xyzlim "0.00000 0.33334 0.00000 0.33334 0.00000 1.00000"  
    } elseif { $number == 149    } {
	# P312
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.50000"  
    } elseif { $number == 150    } {
	# P321
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.50000"  
    } elseif { $number == 151    } { 
	# P3112
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.16667"  
    } elseif { $number == 152    } { 
	# P312
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.16667"  
    } elseif { $number == 153    } { 
	# P3212
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.16667"  
    } elseif { $number == 154    } { 
	# P3221
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.16667"  
    } elseif { $number == 155    } { 
	# R32
	set xyzlim "0.00000 0.33334 0.00000 0.33334 0.00000 0.50000"  
    } elseif { $number == 168    } {
	# P6
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 169    } { 
	# P61
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.16667"  
    } elseif { $number == 170    } { 
	# P65
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.16667"  
    } elseif { $number == 171    } { 
	# P62
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.33334"  
    } elseif { $number == 172    } { 
	# P64
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.33334"  
    } elseif { $number == 173    } { 
	# P63
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.50000"  
    } elseif { $number == 177    } { 
	# P622
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 178    } { 
	# P6122
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.08334"  
    } elseif { $number == 179    } { 
	# P6522
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.08334"  
    } elseif { $number == 180    } { 
	# P6222
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.16667"  
    } elseif { $number == 181    } { 
	# P6422
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.16667"  
    } elseif { $number == 182    } { 
	# P6322
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.25000"  
    } elseif { $number == 195    } { 
	# P23
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 0.50000"  
    } elseif { $number == 196    } { 
	# F23
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 197    } { 
	# I23
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 198    } { 
	# P213
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 199    } { 
	# I213
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 207    } { 
	# P432
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 1.00000"  
    } elseif { $number == 208    } { 
	# P4232
	set xyzlim "0.00000 1.00000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 209    } { 
	# F432
	set xyzlim "0.00000 0.50000 0.00000 0.50000 0.00000 0.50000"  
    } elseif { $number == 210    } { 
	# F4132
	set xyzlim "0.00000 0.50000 0.00000 0.75000 0.00000 0.66667"  
    } elseif { $number == 211    } { 
	# I432
	set xyzlim "0.00000 0.25000 0.00000 0.75000 0.00000 0.66667"  
    } elseif { $number == 212    } { 
	# P4332
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 1.00000"  
    } elseif { $number == 213    } { 
	# P4132
	set xyzlim "0.00000 1.00000 0.00000 1.00000 0.00000 1.00000"  
    } elseif { $number == 214    }  { 
	# I4132
	set xyzlim "0.00000 0.66667 0.00000 0.75000 0.00000 1.00000"  
    }

    uplevel 1 [list set $xyz $xyzlim]
}

#
# Check if all the necessary programs are present, and that
# we have at least a good known working version
#
proc all_necessary_programs { } {
    global XMLParse

    set need_ccp4 1

    #
    # CCP4 version at least 6.0?
    # 
    if { $need_ccp4 } {
	# First, create and run a CCP4 command line.  
	# The "-i flag gives version information"
	set mtz_command "scaleit -i"
	set mtz_script "\nEND\n"
	set command "$mtz_command << \"$mtz_script\""
	catch {eval exec $command } output 
	set err [lindex $::errorCode 0]
	
	# Check to see if the executable is found
	if { [string compare $err "POSIX"] == 0 } {
	    crank_error "crank::crankutils.tcl::Error ccp4 executables not found"
	}

	# Check to see if we have the correct version
	if { [regexp {patch level *([0-9.]*)} $output match version]} {
	    if { $version > 5.9 } {
		puts "Found CCP4 version $version\n"
	    } else { 
		crank_error "crank::crankutils.tcl::Error version CCP4 is not greater than 5.9, current version is $version"
	    }
	} else {
	    crank_error "crank::crankutils.tcl::Error finding version string from scaleit exectuable"
	}
    }

    set need_refmac5D 0
    set need_arpwarp  0
    set need_resolve  0
    set need_buc      0

    for { set i 1 } { [info exists XMLParse([join "crank soap run step=$i name" __])] } { incr i } {
 	set step [string tolower $XMLParse([join "crank soap run step=$i name" __])]

	if { [string compare $step "arpwarp"] == 0 } {
	    set need_arpwarp 1
	    if { $XMLParse([join "crank soap run step=$i arpwarp phase_restrain" __]) == "DIRECT" } {
		set need_refmac5D 1
	    }
	}
  	if { ( ([string compare $step "resolvedm"] == 0) ||
	       ([string compare $step "resolvemb"] == 0) ) } {
	    set need_resolve 1
	}
  	if { ([string compare $step "buccaneer"] == 0) } {
	    set need_buc 1
	}
    }


    if { $need_resolve } {
	set rcommand "resolve_giant"
	set script "END\n"
	set command "$rcommand << \"$script\""
	catch {eval exec $command } output 
	set err [lindex $::errorCode 0]
	# Check to see if the executable is found
 	if { [string compare $err "POSIX"] == 0 } {
	    crank_error "crank::crankutils.tcl::Error resolve_giant executable not found.  Please install RESOLVE or make sure it is in your PATH - see http://www.solve.lanl.gov/ for more info."
	} else {
	    puts "Found resolve_giant binary\n"
	    if { [file exists resolve.ok] } {
		file delete resolve.ok
	    }
	    if { [file exists resolve.plots] } {
 		file delete resolve.plots
	    }
	}
    }

    if { $need_buc } {
	set command "cbuccaneer"
	catch {eval exec $command } output 
	set err [lindex $::errorCode 0]
	# Check to see if the executable is found
 	if { [string compare $err "POSIX"] == 0 } {
	    crank_error "crank::crankutils.tcl::Error cbuccaneer executable not found.  Please install BUCCANEER or make sure it is in your PATH - see http://www.ysbl.york.ac.uk/~cowtan/ for more info."
	} else {
	    puts "Found cbuccaneer binary\n"
	}
    }

    if { $need_arpwarp } {
	# First, create and run an ARP/wARP command line
	set arp_command "arp_warp"
	set arp_script "MODE MIRBUILD\nEND\n"
	set command "$arp_command << \"$arp_script\""
	catch {eval exec $command } output 
	set err [lindex $::errorCode 0]
	# Check to see if the executable is found
 	if { [string compare $err "POSIX"] == 0 } {
	    crank_error "crank::crankutils.tcl::Error arp_warp executable not found.  Please install ARP/wARP or make sure it is in your PATH - see http://www.arp-warp.org/ for more info."
	}

	# Check to see if we have the correct version
	if { [regexp {.*ARP Ver. ([0-9.]*)} $output match version]} {
	    if { $version >= 7.0 } {
		puts "Found ARP/wARP version $version\n"
 	    } else {
		crank_error "crank::crankutils.tcl::Error version 7.0 of ARP/wARP not installed, current version is ($version).  Please install latest version - see http://www.arp-warp.org for more info."
	    }
	} else {
	    crank_error "crank::crankutils.tcl::Error finding version string in arp_warp exectuable"
	}
	if { $need_refmac5D } { 
	    # create and run refmac5D command line
	    set refmac_command "refmac5_sad -i"
	    set refmac_script "\nEND\n"
	    set command "$refmac_command << \"$refmac_script\""
	    catch {eval exec $command } output 
	    set err [lindex $::errorCode 0]

	
	    # Check to see if the executable is found
	    if { [string compare $err "POSIX"] == 0 } {
		    crank_error "crank::crankutils.tcl::Error Refmac version is not greater than or equal to 5.5, current version is $version.\n  To run the SAD function, 5.5 or greater is needed.\n  Please download the correct version from http://www.ysbl.york.ac.uk/~garib/refmac/latest_refmac.html and name the binary refmac5_sad and make sure it is in your path!"
	    }

	    # Check to see if we have the correct version
	    if { [regexp {version *([0-9.]*)} $output match version]} {
		if { $version >= 5.5 } {
		    puts "Found refmac_sad version $version\n"
		} else { 
		    crank_error "crank::crankutils.tcl::Error Refmac version is not greater than or equal to 5.5, current version is $version.\n  To run the SAD function, 5.5 or greater is needed.\n  Please download the correct version from http://www.ysbl.york.ac.uk/~garib/refmac/latest_refmac.html and name the binary refmac5_sad and make sure it is in your path!"
		}
	    } else {
		crank_error "crank::crankutils.tcl::Error finding version string from refmac5 exectuable"
	    }
	}
    }
}

proc crank_run_message { message } {

    puts "<B><FONT COLOR=\"\#330044\"><!--SUMMARY_BEGIN-->"
    puts "crank->Running::$message"
    puts "<!--SUMMARY_END--></FONT></B>"
}

proc crank_error { message } {

    puts "<B><FONT COLOR=\"\#FF0000\"><!--SUMMARY_BEGIN-->"
    puts "<hr>"
    puts "crank->Error:$message"
    puts "Exitting."
    puts "<!--SUMMARY_END--></FONT></B>"
    puts ""
    set exit $exitting
}

#
# runcommand - Execute an outside program
#
# Run a command using an "exec" wrapped with "catch".  If the external
# command fails, catch the error and return a string describing the
# error message, specifically:
#
# SUCCESS     - Command succeeded
# CHILDKILLED - Child process was killed
# CHILDSTATUS - Child exited with a non-zero error status
# CHILDSUSP   - Child was suspended
# POSIX       - One of the kernel calls to launch the command failed
#                (e.g. file could not be found)
#
# To output more debugging information, such as the output of the 
# command, set debug 1
#
#
proc runcommand { command } {
    
    global tcl_platform
    set os $tcl_platform(os)

    puts "--------------------------------------------------------------------------------"
    puts "Crank is now running the program:"
    puts "$command"

    set t0 [clock clicks -millisec ]

    # Run the command
    set rcstatus [catch {eval exec $command } output ]

    set shelx 0


# ***NSP - this is really dumb - i want to see if a shelx* program is running, b/c running
# it in tcl under some operating systems leads to a status 1 when the job is successful...

    set trimmed  [string trimright [string trimleft $command "tclsh "] "-command.tcl"]

    if { ( ($trimmed == "elx") || ($trimmed == "elxe")  ) } { 
	set shelx 1
    }

    if { $rcstatus == 0 } {
	# The command succeeded, and wrote nothing to stderr.
	# $output contains what it wrote to stdout, unless you
	# redirected it.
	puts $output
	set returnval "SUCCESS"	
    } elseif { ( ($rcstatus == 1) && ($os == "Darwin") && ($shelx == 1 ) ) } {
	# shelx run in tcl gives a status of 1 when the job is successful!
 	# puts $output
	set returnval "SUCCESS"
    } elseif { [string equal $::errorCode NONE] } {
	# The command exited with a normal status, but wrote something
	# to stderr, which is included in $output.
 	puts $output
	set returnval "SUCCESS"

    } else {

	puts "Error encountered when running:"
	puts "command:($command)"
	puts "crank::runcommand::ERROR CODE($::errorCode)"
	switch -exact -- [lindex $::errorCode 0] {

	    CHILDKILLED {
                # A child process, whose process ID was $pid,
                # died on a signal named $sigName.  A human-
                # readable message appears in $msg.
                foreach { - pid sigName msg } $::errorCode break
				puts "pid=($pid)"
				puts "sigName=($sigName)"
				puts "msg=($msg)"
				puts "crank::runcommand::STATUS:CHILDKILLED - child killed"
				set returnval "CHILDKILLED"

            }

            CHILDSTATUS {
                # A child process, whose process ID was $pid,
                # exited with a non-zero exit status, $code.
                foreach { - pid code } $::errorCode break
				puts "crank::runcommand::STATUS:CHILDSTATUS - child exited with non-zero exit status"
				set returnval "CHILDSTATUS"

            }

            CHILDSUSP {
                # A child process, whose process ID was $pid,
                # has been suspended because of a signal named
                # $sigName.  A human-readable description of the
                # signal appears in $msg.
                foreach { - pid sigName msg } $::errorCode break
				puts "crank::runcommand::STATUS:CHILDSUSP - child was suspended"
				set returnval "CHILDSUSP"

            }


            POSIX {
				# One of the kernel calls to launch the command
				# failed.  The error code is in $errName, and a
				# human-readable message is in $msg.
		foreach { - errName msg } $::errorCode break
		puts "crank::runcommand::STATUS:POSIX - kernel call failed"
		set returnval "POSIX"
	    }
	}
 	puts $output
  	puts "Exitting"
	set exit $exitting
    }

    if { [string compare $returnval SUCCESS] != 0 } {
	puts "Exitting"
	set exit $exitting
    }

    if { [string first "tclsh " $command] != -1 } {
	set command_just_run [lindex $command 1]
    } else {
	set command_just_run [lindex $command 0]
    }

    puts "crank::command ($command_just_run) run time:[expr ([clock clicks -millisec ]-$t0)/1000.] sec"

    return $returnval
}

proc debug_print_xml_database { } {
    upvar XMLParse xml
	
    set allvars [array get xml]
    for { set i 0 } { $i < [llength $allvars] } { incr i } {
	puts -nonewline "[lindex $allvars $i]\t"
	incr i
	puts "[lindex $allvars $i] "
    }
}

proc XMLElementStartHandler { name attlist } {
    global xmldata
    global XMLParse
    global XMLWork

    set section_id [GetXmlAttributeValue $attlist "id"]

    if { $section_id != "" } {
	set XMLWork(section_id) $section_id
	set XMLWork(section_id_depth) $XMLWork(section_depth)
    } 

    # Add this name to the current "full" name
    set XMLWork(section_name) $name
    if { $section_id == "" } {
	lappend XMLWork(section_full_name) $name
    } else {
	lappend XMLWork(section_full_name) "${name}=${section_id}"
    }
    set XMLWork(section_cat_name) [join $XMLWork(section_full_name) "__"]
    incr XMLWork(section_depth)
}

proc XMLElementEndHandler { name } {
    global xmldata
    global XMLParse
    global XMLWork

    set XMLWork(section_full_name) [lreplace $XMLWork(section_full_name) \
					[expr [llength $XMLWork(section_full_name)] - 1] \
					[expr [llength $XMLWork(section_full_name)] - 1]]
    incr XMLWork(section_depth) -1

    if { $XMLWork(section_depth) < $XMLWork(section_id_depth) } {
	set XMLWork(section_id) ""
	set XMLWork(section_id_depth) 0
    }
}

proc XMLCharacterHandler { data args } {
    global xmldata
    global XMLParse
    global XMLWork

    set data [string trim $data]

    if { $data != "" } {
	set fullvarname "$XMLWork(section_cat_name)"
	set XMLParse($fullvarname) $data
    }
}

proc XMLParsefile { xmlfile } {
    # Global used by the XML parser
    global xmldata
	
    # Data structure to hold actual data
    global XMLParse 
	
    # Working data structure to hold temporary work data
    global XMLWork

    set XMLWork(section_depth) 0
    set XMLWork(section_id) ""
    set XMLWork(section_id_depth) 0

    # Initialise and configure the XML parser
    set parser 0
    set parser [ xml::parser ]
    $parser configure -elementstartcommand XMLElementStartHandler
    $parser configure -elementendcommand XMLElementEndHandler
    $parser configure -characterdatacommand XMLCharacterHandler
    $parser configure -ignorewhitespace true

    # Read and parse the specified file
    if { [ReadFile $xmlfile xmltext] } {
	$parser parse $xmltext
	set xmldata(XML_FILE) $xmlfile
	set xmldata(XML_LOADED) 1
    } else {
	# Couldn't load the data
	set xmldata(XML_LOADED) 0
    }
}

#
# Check to see if $atomname is present in the file
# $CLIBD/atomsf.lib
#
proc GoodAtomName { atomname } {
    global env
	
    set atomname [string tolower $atomname]
    set atomsf "[file join $env(CLIBD) atomsf.lib]"

    set file_id [open $atomsf r]

    set done 0
    while { !$done } {
	gets $file_id line
	if { ! [regexp {AD.*} $line]} {
	    set done 1
	}
    }
	
    lappend atomlist $line
    gets $file_id line
    gets $file_id line
    gets $file_id line
    gets $file_id line

    while { [gets $file_id line] >= 0 } {
	lappend atomlist $line
	gets $file_id line
	gets $file_id line
	gets $file_id line
	gets $file_id line
    }

    set match 0
    for { set i 0 } { $i < [llength $atomlist] } { incr i } {
	if { [regexp {([A-Za-z+0-9-]*) } [lindex $atomlist $i] full atom] } {
	    set atom [string tolower $atom]
	    if { [string compare $atomname $atom] == 0 } {
		set match 1
		break
	    }
	}
    }
	
    if { $match == 1 } {
	return 1
    } else {
	return 0
    }
}

proc doplugins_nocheck { crankplugins } {
    global PluginList
    global XMLParse

    # Make a list of all the index.xml files in each directory 
    # in crank/plugins/*.
    set all_pluginxml [glob -nocomplain [file join $crankplugins * *_index.xml]]
	
    #puts "all_pluginxml($all_pluginxml)"

    # Loop over all plugin index.xml files
    foreach pluginxml $all_pluginxml {
	#puts "pluginxml=($pluginxml)"
	
	# Parse the XML from this file into the global datastructure
	XMLParsefile $pluginxml
	
	# Add the name of the plugin to the global plugin list
	set fileid [open "$pluginxml" r]
	while { [gets $fileid line] >= 0 } {		
	    
	    if { [regexp { *<name>([A-Za-z1-9._]*)</name>} $line all match ] } {
		lappend PluginList $match
	    }
	}
	close $fileid
    }
}


proc reference_crank { } {
    global XMLParse

    puts "Crank:"
    puts "Ness, S.R., de Graaff, R.A.G., Abrahams, J.P. & Pannu, N.S. (2004)"
    puts "CRANK: New Methods for Automated Macromolecular Crystal Structure"
    puts "Solution. Structure 12, 1753-1761.\n"
    puts "Initial number of monomer estimate:"
    puts "Kantardjieff, K.A. & Rupp, B. (2003)  Matthews coefficient probabilities:"
    puts "Improved estimates for unit cell contents of proteins, DNA, and protein-"
    puts "nucleic acid complex crystals.  Protein Science 12, 1865-1871.\n"
}

proc show_parameters { } {
}

proc print_banner { } {		  
    global tcl_platform

    puts "-------------------------------------------------------------------------------"
    puts "                         ___   _ __   __ _   _ __   | | __ "
    puts "                        / __| | '__| / _` | | '_ \\  | |/ / "
    puts "                       | (__  | |   | (_| | | | | | |   <  "
    puts "                        \\___| |_|    \\__,_| |_| |_| |_|\\_\\ "
    puts ""
    puts "                              Crank - Version 1.2.0"
    puts ""
    puts "                           Leiden University (c) - 2004-2007"
    puts ""
    puts "                              LIC, Gorlaeus Laboratories"
    puts "                              Leiden University"
    puts "                              PO Box 9502, 2300 RA Leiden"
    puts "                              The Netherlands"
    puts "    "
    puts "                    http://www.bfsc.leidenuniv.nl/software/crank/"
    puts "\n"
    puts "Please report any bugs or problems to crank@chem.leidenuniv.nl\n"
    puts ""
    #puts "Tcl Version information: [info patchlevel]"
    #parray tcl_platform
    puts ""
}

#
proc print_info_summary { } {
    global XMLParse
    global orig_pwd

    #
    puts "This Crank run is now complete."
    puts ""
    puts "<!--SUMMARY_END--></FONT></B>"
    puts "</pre>"
}

#
proc print_data_summary { } {
}

# Show the reference list for all plugins
proc show_references { } {
    global XMLParse
    
    puts "References:"
    puts "----------"
    reference_crank
   
    for { set i 1 } { [info exists XMLParse(crank__soap__run__step=$i)] } { incr i } {
	set program [string tolower $XMLParse(crank__soap__run__step=$i)]
	
	    
	if { [info exists XMLParse(crank__plugin=${program}__name)] } {
	    ${program}_reference
	} else {
	    crank_error "No name for program ($program)"
	}
    }
}
