#!/usr/bin/env tclsh

# Copyright (C) Navraj Pannu and Steven Ness 2004-2005, Leiden University
# Copyright (C) Navraj Pannu 2006-2007, Leiden University

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
proc OutputAFROcomfile { executable } {
    global XMLParse

    puts "\#!/bin/sh"
    puts " "

    set step $XMLParse(crank__soap__current_step)
    set orig_pwd $XMLParse(crank__soap__orig_pwd)
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    if { [info exists XMLParse(crank__soap__run__standalone)] } {
	set mtz_in "[file join $orig_pwd run workdb crank.in.$tag.mtz]"
    } else {
	set mtz_in "[file join $orig_pwd workdb crank.in.$tag.mtz]"
    }

    set mtz_out "afro.out.mtz"

    puts "$executable HKLIN $mtz_in HKLOUT $mtz_out << END > afro-logfile"

    # determine if we have MAD+native!
    set mad_native 0

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } {
 	if { $XMLParse([join "crank parameters crystal=$i native" __]) == "0" } {
	    set j 1
	    while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } {
		if { $j > 1 } {
		    set mad_native 1
		}
		incr j
	    }
	}
	incr i
    }
    
    #
    # Loop over all crystal/datasets outputting XTAL/DNAME sections for each
    #
    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } { 
	# Start the XTAL section

	if { !( ($mad_native == 1) && ($XMLParse([join "crank parameters crystal=$i native" __]) == "1") ) } {

	    puts "XTAL crystal${i}"
	    if { [info exists XMLParse([join "crank parameters crystal=$i substructure atom_name" __])] } {
		puts "  ATOM $XMLParse([join "crank parameters crystal=$i substructure atom_name" __])"
		puts "    NUMB $XMLParse([join "crank parameters crystal=$i substructure natoms" __])" 
		puts "    BISO $XMLParse([join "crank parameters crystal=$i data=1 bfactor" __])" 
	    }
	    set j 1
	    while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } { 
		# Start the DNAME section
		set type $XMLParse([join "crank parameters crystal=$i data=$j type" __])
		puts "  DNAME ${type}_${i}_${j}"

		# Output the columns for this dataset
		if { [string compare $XMLParse([join "crank parameters crystal=$i data=$j anomalous" __]) "0"] == 0 } { 
		    # Build up labin and labout for non-anomalous data
		    set input_f $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j f" __])
		    set input_sigf $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j sigf" __])
				
		    puts "    COLUMN F=$input_f SF=$input_sigf"
		} else {
		    set input_f_plus $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j f_plus" __])
		    set input_sigf_plus $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j sigf_plus" __])
		    set input_f_minus $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j f_minus" __])
		    set input_sigf_minus $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j sigf_minus" __])
				
		    puts "    COLUMN F+=$input_f_plus SF+=$input_sigf_plus F-=$input_f_minus SF-=$input_sigf_minus"

		    # Output Atom Form Factors
 		    set atomname $XMLParse([join "crank parameters crystal=$i substructure atom_name" __])
		    set fprime $XMLParse([join "crank parameters crystal=$i data=$j fprime" __])
		    set fprimeprime $XMLParse([join "crank parameters crystal=$i data=$j fprimeprime" __])
		    puts "    FORM $atomname FP=$fprime FPP=$fprimeprime"
		}

		if { [info exists XMLParse([join "crank soap run step=$step afro sigf" __])] } {
		    set sigf $XMLParse([join "crank soap run step=$step afro sigf " __])
		}

		if { [info exists XMLParse([join "crank soap run step=$step afro manual_sigmas" __])] } {
		    if { $XMLParse([join "crank soap run step=$step afro manual_sigmas " __])  == 1 } {
			if { [info exists XMLParse([join "crank soap run step=$step afro sano" __])] } {
			    set sano $XMLParse([join "crank soap run step=$step afro sano " __])
			    set sano $sano
			}
		    }
		} else {
		    if { [info exists XMLParse([join "crank soap run step=$step afro sano" __])] } {
			set sano $XMLParse([join "crank soap run step=$step afro sano " __])
			set sano $sano 
		    }
		}

		#     Create the EXCLUDE SIGF line
		#
		if { [info exists sigf] && [info exists sano] } {
		    puts "    EXCLUDE SIGF $sigf SANO $sano SISO $sano"
		} else {
 		    if { [info exists sigf] } {
			puts "    EXCLUDE SIGF $sigf"
		    }
		
		    if { [info exists sano] } {
			puts "    EXCLUDE SANO $sano SISO $sano"
		    }
		}
		incr j
	    }
	}
	incr i

    }

    if { [info exists XMLParse([join "crank soap run step=$step afro manual_reso" __])] } {
	if { $XMLParse([join "crank soap run step=$step afro manual_reso" __]) == "1" } {
	    if { [info exists XMLParse([join "crank soap run step=$step afro hi_res" __])] } {
		puts "HIREs $XMLParse([join "crank soap run step=$step afro hi_res" __])"
	    }
	}
    }

    puts "\nEND"
}

#
#
#
set inputxml [lindex $argv 0]
set executable [lindex $argv 1]
set crankbin [lindex $argv 2]

source [file join $crankbin crankutils.tcl]

if { [file exists $inputxml] == 0 } {
    crank_error "xml2afrocom.tcl::inputxml file does not exist"
}

XMLParsefile $inputxml

#
#
OutputAFROcomfile $executable
