#
# Copyright (C) Navraj S. Pannu and Steven Ness 2004-2005, Leiden University
# Copyright (C) Navraj S. Pannu 2006-2008, Leiden University
#
#
# All rights reserved.
#
# This file may not be copied, reproduced, modified or distributed
# in any way.
#

proc arpwarp_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse

    crank_run_message "$step-arpwarp"

    source [file join $crankbin crankutils.tcl]

    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    set inputmtz [file join [pwd] workdb crank.in.$tag.mtz]
    
    if { $XMLParse([join "crank parameters residues" __]) < 1 } {
	crank_error "No protein residues in macromolecule"
    }

    set phase_restrain ""

    if { [info exists XMLParse([join "crank soap run step=$step arpwarp phase_restrain" __])] } {
       set phase_restrain $XMLParse([join "crank soap run step=$step arpwarp phase_restrain" __])
    }

    if { $phase_restrain == "DIRECT" } {
	set substructure_tag $XMLParse([join "crank soap run step=$step input substructure" __])
	set inputsubstructure [file join [pwd] workdb crank.out.$substructure_tag.substructure.pdb]
    }

    file mkdir $step-arpwarp
    cd $step-arpwarp

    #
    # Setup ARP/wARP defaults
    #
    set workdir [file join [pwd] run]
    file mkdir $workdir

    set heavyin ""

    if { $phase_restrain == "DIRECT" } {
        file copy $inputsubstructure heavy.pdb	
	# Set the heavyin variable to the name of the heavy.pdb file we just created.
	set heavyin [file join [pwd] heavy.pdb]
    }

    if { [info exists XMLParse([join "crank parameters sequence" __])] } {
	set seq [open sequence.pir w+]
	puts $seq ">\n"
	regsub -all {[^a-zA-Z]} $XMLParse([join "crank parameters sequence" __]) "" sequence
	for {set i 0 } { $i < [string length $sequence] } { incr i 60 } {
	    if { [expr $i + 59] <  [string length $sequence] } {
 		puts $seq "[string range $sequence $i [expr $i + 59] ]" 
	    } else {
 		puts $seq "[string range $sequence $i  end]" 
	    }
	}
	flush $seq
	close $seq
    }

    # Now generate the ARP/wARP .par input file
    runcommand [concat [file join $crankplugins arpwarp xml2arpwarp_par.tcl] $inputxml $inputmtz $crankbin $heavyin >& input.par]

    # Set dummy ccp4i def file
     set dum [open dummy.def w+]
     puts $dum "#CCP4I VERSION DUMMY\n#CCP4I PROJECT PROJECT" 
     flush $dum
     close $dum

    # Run run arpwarp
    set script [open arp_refmac-command.com w+]
    puts $script "[file join $crankbin run_arpwarp_refmac5D] < input.par"
    flush $script
    close $script
    # runcommand [concat chmod ugo+x ./arp_refmac-command.com]
    runcommand [concat sh arp_refmac-command.com]

    if { [file exists 1_arp_warp.log] } {
       file copy 1_arp_warp.log arpwarp-logfile
    } else {
       crank_error "ARP/wARP + REFMAC log file not created"
    }

    # Copy output to stdout
    runcommand [concat cat arpwarp-logfile]

    if { [file exists dummy.def] } {
       file delete dummy.def
    }

    # Rename ARP/wARP output column names.
    runcommand [concat [file join $crankplugins arpwarp crank-rename-arpwarp-output.tcl] $inputxml 1_warpNtrace.mtz rename.out.mtz $crankbin]

    if { [file exists rename.out.mtz] } {
       file copy rename.out.mtz [file join .. workdb crank.out.$tag.mtz]
       file delete rename.out.mtz
    } else {
       crank_error "ARP/wARP + REFMAC mtz file not created"
    }

    # Copy the final ARP/wARP PDB model to the work directory
    if { [file exists 1_warpNtrace.pdb] } {
        file copy 1_warpNtrace.pdb [file join .. workdb crank.out.$tag.pdb]
    } else {
       crank_error "ARP/wARP + REFMAC pdb not created"
    }

    # Copy logfile to main log directory
    file copy arpwarp-logfile [file join .. log $step-arpwarp-logfile]

    if { [file exists CCP4_DATABASE] } {
       file delete CCP4_DATABASE
    }
    
    cd ..
}

proc arpwarp_reference { } {
    global XMLParse

    puts "ARP/wARP:"
    puts "Perrakis, A., Morris, R.M. & Lamzin, V.S. (1999) Automated protein"
    puts "model building combined with iterative structure refinement."
    puts "Nature Struct. Biol. 6, 458-463)"
    puts ""

    # General Refmac Reference
    puts "Refmac:"
    puts "Murshudov, G.N., Vagin, A.A. & Dodson, E.J. (1997) Refinement of"
    puts "macromolecular structures by the maximum-likelihood method."
    puts "Acta Cryst. D53, 240-255.\n"

    for { set i 1 } { [info exists XMLParse([join "crank soap run step=$i name" __])] } { incr i } {
	set step [string tolower $XMLParse([join "crank soap run step=$i name" __])]

	if { [string compare $step "arpwarp"] == 0 } {
	    if { ($XMLParse([join "crank soap run step=$i arpwarp phase_restrain" __]) == "SADH") ||
                 ($XMLParse([join "crank soap run step=$i arpwarp phase_restrain" __]) == "SAD") } {
		# SAD function
		puts "SAD likelihood function:"
		puts "Skubak, P., Murshudov, G.N. & Pannu, N.S. (2004)"
		puts "Direct incorporation of experimental phase information in model refinement"
		puts "structure refinement.  Acta Cryst. D60, 2196-2201.\n"
		break
            } elseif { $XMLParse([join "crank soap run step=$i arpwarp phase_restrain" __]) == "MLHL" } {
		# MLHL function
		puts "MLHL likelihood function:"
		puts "Pannu, N.S., Murshudov, G.N., Dodson, E.J. & Read R.J. (1998)"
		puts "Incorporation of prior phase information strengthens maximum-likelihood"
		puts "structure refinement.  Acta Cryst. D54, 1285-1294.\n"
		break
	    }
        }
    }
}

