#
# Copyright (C) Navraj S. Pannu and Steven Ness 2004-2005, Leiden University
# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc bp3_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    global env

    #
    # Run BP3 to refine the coordinates and generate phase estimates
    #
    crank_run_message "$step-bp3"
    
    file mkdir $step-bp3
    cd $step-bp3
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    # Set the location of the executable to the crank bp3 plugin directory
    if { $inccp4 } {
       set executable [file join $env(CBIN) $executable]
    } else {
       set executable [file join $crankbin  $executable]
    }
    set run 1
    # Generate the BP3 command file
    set substructure_tag $XMLParse([join "crank soap run step=$step input substructure" __])
    runcommand [concat [file join $crankplugins bp3 xml2bp3com.tcl] $inputxml [file join .. workdb crank.out.$substructure_tag.substructure.pdb] $run $executable 0 $crankbin >& bp3-command$run.com]
    runcommand [concat [file join $crankplugins bp3 xml2bp3tcl.tcl] $inputxml [file join .. workdb crank.out.$substructure_tag.substructure.pdb] $run $executable 0 $crankbin >& bp3-command$run.tcl]


    runcommand [concat tclsh bp3-command$run.tcl]
    
    runcommand [concat cat bp3-logfile$run]

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } {
	set j 1
        while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } {
		incr j
	}
	incr i
    }

   if { (1 == 2) && ($i == 2) && ($j == 2) } { 
    	# calculate difference map for SAD
    	runcommand [concat [file join $crankplugins bp3 gradient.tcl]  >& gradient.com]

    	runcommand [concat chmod ugo+x ./gradient.com]
    	runcommand [concat ./gradient.com |& tee gradient-logfile]

    	# calculate map for SAD 
	set mtz_in ../workdb/crank.in.$tag.mtz
    	set sftools_script "read $mtz_in\nread bp3.out1.mtz\n calc col DANO = col 3 col 5 -  \n set type col DANO\nd \n write dano.mtz col DANO PHIB\nquit\n"

	set command "sftools << \"$sftools_script\""
#       puts $command
        catch {eval exec $command } output
#       puts $output

    	runcommand [concat [file join $crankplugins bp3 dano.tcl]  >& dano.com]

    	runcommand [concat chmod ugo+x ./dano.com]
    	runcommand [concat ./dano.com |& tee dano-logfile]

    	# Create new substructure xml if new sites have been found
  	#  runcommand [concat [file join $crankplugins bp3 newsites.tcl] ] 
    
    	if { [file exists more.xml] } {
		incr $run
		runcommand [concat [file join $crankplugins bp3 xml2bp3com.tcl] $input.xml more.xml $run $executable >& bp3-command2.com]
		runcommand [concat chmod ugo+x ./bp3-command2.com]
		runcommand [concat ./bp3-command2.com |& tee bp3-logfile2]
    	}
    }
	
    # Rename output columns to the names specified in the XML
    runcommand [concat [file join $crankplugins bp3 crank-rename-bp3-output.tcl] $inputxml bp3.out${run}.mtz rename.out.mtz $crankbin]
    file copy rename.out.mtz [file join .. workdb crank.out.$tag.mtz]
    file delete rename.out.mtz

    # For other hand calculation - if it was run
    if { [file exists bp3.out${run}.mtz-oh] } {
	runcommand [concat [file join $crankplugins bp3 crank-rename-bp3-output.tcl] $inputxml bp3.out${run}.mtz-oh rename.out.mtz $crankbin]
	file copy rename.out.mtz [file join .. workdb crank.out.$tag.oh.mtz]
	file delete rename.out.mtz
    }

    if { [file exists heavy.xml] } {
#file copy heavy.xml [file join .. workdb crank.out.$tag.substructure.xml]
        if { [file exists heavy1.pdb] } {
	   file copy heavy1.pdb [file join .. workdb crank.out.$tag.substructure.pdb]
	} elseif { [file exists heavy2.pdb] } {
	   file copy heavy2.pdb [file join .. workdb crank.out.$tag.substructure.pdb]
        }
    }
    
    if { [file exists heavy-oh.xml] } {
#file copy heavy-oh.xml [file join .. workdb crank.out.$tag.oh.substructure.xml]
        if { [file exists heavy-oh1.pdb] } {
	   file copy heavy-oh1.pdb [file join .. workdb crank.out.$tag.oh.substructure.pdb]
	} elseif { [file exists heavy-oh2.pdb] } {
	   file copy heavy-oh2.pdb [file join .. workdb crank.out.$tag.oh.substructure.pdb]
        }
    }

    # Copy logfile to main log directory
    file copy bp3-logfile$run [file join .. log $step-bp3-logfile$run]
    if { [file exists bp3-logfile2] } {
        file copy bp3-logfile2 [file join .. log $step-bp3-logfile2]
    }

    set stop_fom $XMLParse([join "crank soap run step=$step bp3 stop_fom" __])
    set logfile [open "bp3-logfile$run" r]
    set output [read $logfile]

    if { ![regexp { .*The overall FOM is* *([0-9.]*).*} $output junk fom] } {
        puts "Search string not found"
    }
    
    close $logfile
    
    if { $fom < $stop_fom } {
	crank_error "The obtained FOM is less than the minimum required FOM"
    }
    
    cd ..
}

proc bp3_reference { } {
    global XMLParse
    
    set dataset 0
    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } {
	set j 1
	while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } {
	    set dataset [expr $dataset + 1]
            incr j
	}
        incr i
    }

    puts "Bp3:"

    if { $dataset == 1 } { 
	puts "Pannu, N.S. & Read, R.J. (2004) The application of multivariate"
	puts "statistical techniques improves single-wavelength anomalous"
	puts "diffraction phasing. Acta Cryst. D60, 22-27.\n"
    } else {
	puts "Pannu, N.S., McCoy A.J., and Read, R.J. (2003) Application of the"
	puts "complex multivariate normal distribution to crystallographic methods"
	puts "with insights into multiple isomorphous replacement phasing. Acta"
	puts "Cryst. D59, 1801-1808.\n"
    }
}
