#!/usr/bin/env tclsh

# Copyright (C) Navraj S. Pannu and Steven Ness 2004-2005, Leiden University
# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
proc OutputBP3comfile { executable run substructurepdb crunch2 } {
    global XMLParse
	
    set step $XMLParse(crank__soap__current_step)
    set orig_pwd $XMLParse(crank__soap__orig_pwd)
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    puts "\#!/bin/sh"
    puts " "

    set mtz_in [file join .. workdb crank.in.$tag.mtz]

    set mtz_out "bp3.out${run}.mtz"

    puts "$executable HKLIN $mtz_in HKLOUT $mtz_out << END > bp3-logfile${run}"
    puts " "

    # determine if we have MAD+native!
    set mad_native 0

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } {
	if { $XMLParse([join "crank parameters crystal=$i native" __]) == "0" } {
	    set j 1
	    while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } {
		if {$j > 1} {
		    set mad_native 1
		}
		incr j
	    }
	}
	incr i
    }

    # Loop over all crystal/dataset pairs
    #
    # Loop over all crystal/datasets outputting XTAL/DNAME sections for each
    #
    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } { 

	if { !( ($mad_native == 1) && ($XMLParse([join "crank parameters crystal=$i native" __]) == "1") ) } {

	    # Start the XTAL section

	    puts "XTAL crystal${i}"

	    if { [info exists XMLParse([join "crank parameters crystal=$i native" __])] &&
		 $XMLParse([join "crank parameters crystal=$i native" __]) == "0"} { 

		if { [info exists XMLParse([join "crank parameters crystal=$i substructure coordinate_format" __]) ] } {
		    set format $XMLParse([join "crank parameters crystal=$i substructure coordinate_format" __])
		    set j 1

		    while { [info exists XMLParse([join "crank parameters crystal=$i substructure atom=$j name" __]) ] } {
			set x    $XMLParse([join "crank parameters crystal=$i substructure atom=$j x" __])
			set y    $XMLParse([join "crank parameters crystal=$i substructure atom=$j y" __])
			set z    $XMLParse([join "crank parameters crystal=$i substructure atom=$j z" __])
			# Atom name
			puts "   atom $XMLParse([join "crank parameters crystal=$i substructure atom=$j name" __])"
			if { ($format == "Fractional") } {
			    puts -nonewline "    xyz "
			    puts -nonewline "$x "
			    puts -nonewline "$y "
			    puts -nonewline "$z "
			} else {
			    orth2frac $x $y $z fracx fracy fracz
			    puts -nonewline "    xyz "
			    puts -nonewline "$fracx "
			    puts -nonewline "$fracy "
			    puts -nonewline "$fracz "
			}
			if {  [info exists XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_x" __])] &&
			      [info exists XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_y" __])] &&
			      [info exists XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_z" __])] } {
			    if { $XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_x" __]) ||
				 $XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_y" __]) ||
				 $XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_z" __]) } {
				puts -nonewline "NOREF "
				if { $XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_x" __]) } {
				    puts -nonewline "X "
				}
				if { $XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_y" __]) } {
				    puts -nonewline "Y "
				}
				if { $XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_z" __]) } {
				    puts -nonewline "Z "
				}
			    }
			}
			puts ""
			puts -nonewline [format "    occu %10.3f" $XMLParse([join "crank parameters crystal=$i substructure atom=$j occ" __])]

 			if {  [info exists XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_occ" __])] } {
			    if { $XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_occ" __]) } {
				puts -nonewline "NOREF "
			    }
			}
			puts ""
			puts -nonewline "    biso $XMLParse([join "crank parameters crystal=$i substructure atom=$j bfac" __])"
  			if {  [info exists XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_bfac" __])] } {

			    if { $XMLParse([join "crank parameters crystal=$i substructure atom=$j noref_bfac" __]) } {
				puts -nonewline "NOREF "
			    } 
			}
			puts ""
			incr j
		    }
		} else {
		    puts "  MODL $substructurepdb"
		}
	    }

	    set j 1
	    while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } { 
		# Start the DNAME section
		set type $XMLParse([join "crank parameters crystal=$i data=$j type" __])
		puts "  DNAME ${type}_${i}_${j}"

		# Output the columns for this dataset
		if { $XMLParse([join "crank parameters crystal=$i data=$j anomalous" __]) == 0 } { 
		    # Build up labin and labout for non-anomalous data
		    set input_f $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j f" __])
		    set input_sigf $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j sigf" __])
		
		    puts "    COLUMN F=$input_f SF=$input_sigf"
		} else {
		    set input_f_plus $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j f_plus" __])
		    set input_sigf_plus $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j sigf_plus" __])
		    set input_f_minus $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j f_minus" __])
		    set input_sigf_minus $XMLParse([join "crank soap run step=$step input exp_columns crystal=$i data=$j sigf_minus" __])
				
		    # Output COLUMN identifier
		    puts "    COLUMN F+=$input_f_plus SF+=$input_sigf_plus F-=$input_f_minus SF-=$input_sigf_minus"
		
		    # Output Atom Form Factors
		    set atomname $XMLParse([join "crank parameters crystal=$i substructure atom_name" __])
		    set fprime $XMLParse([join "crank parameters crystal=$i data=$j fprime" __])
		    set fprimeprime $XMLParse([join "crank parameters crystal=$i data=$j fprimeprime" __])
		    puts "    FORM $atomname FP=$fprime FPP=$fprimeprime"
				
		}
		incr j
	    }
	}
	incr i
    }

	
    #
    # BP3 Options
    #

    if { [info exists XMLParse([join "crank soap run step=$step bp3 hand" __])] } {
	if { $XMLParse([join "crank soap run step=$step bp3 hand" __]) != "0" } {
	    puts "hand"
	}
    }

    if { [info exists XMLParse([join "crank soap run step=$step bp3 noref_xyz" __])] } {
	if { $XMLParse([join "crank soap run step=$step bp3 noref_xyz" __]) == "1" } {
	    puts "NOREf XYZ"
	}
    }


    if { [info exists XMLParse([join "crank soap run step=$step bp3 noref_bfac" __])] } {
	if { $XMLParse([join "crank soap run step=$step bp3 noref_bfac" __]) == "1" } {
	    puts "NOREf BFAC"
	}
    }

    if { [info exists XMLParse([join "crank soap run step=$step bp3 noref_occ" __])] } {
	if { $XMLParse([join "crank soap run step=$step bp3 noref_occ" __]) == "1" } {
	    puts "NOREf OCCU"
	}
    }

    if { [info exists XMLParse([join "crank soap run step=$step bp3 ncycles" __])] } {
	if { $XMLParse([join "crank soap run step=$step bp3 ncycles" __]) != "0" } {
	    set cycle $XMLParse([join "crank soap run step=$step bp3 ncycles" __])
	    puts "cycle $cycle"
	}
    }

    if { [info exists XMLParse([join "crank soap run step=$step bp3 phase" __])] } {
	if { $XMLParse([join "crank soap run step=$step bp3 phase" __]) != "0" } {
	    puts "phase"
	}
    }

    if { $crunch2 == 1 } {
	puts "check"
    }

    puts "END"
}

set inputxml [lindex $argv 0]
set substructurepdb [lindex $argv 1]
set run [lindex $argv 2]
set executable [lindex $argv 3]
set crunch2 [lindex $argv 4]
set crankbin [lindex $argv 5]

# Set CRANK path
source [file join $crankbin crankutils.tcl]

#
# Check to make sure files exist
#
if { [file exists $inputxml] == 0 } {
    crank_error "xml2bp3com.tcl::inputxml file does not exist"
}

if { [file exists $substructurepdb] == 0 } {
    crank_error "xml2bp3com.tcl::substructurepdb file does not exist"
}

XMLParsefile $inputxml

OutputBP3comfile $executable $run $substructurepdb $crunch2
