#!/usr/bin/env tclsh
#
# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
#
proc OutputBuccaneercomfile { ref ex } {
    global XMLParse
    global env

    set step $XMLParse(crank__soap__current_step)
    set orig_pwd $XMLParse(crank__soap__orig_pwd)
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    set mtz_in "[file join $orig_pwd workdb crank.in.$tag.mtz]"

    set c $XMLParse([join "crank parameters ref_crystal" __])
    set d $XMLParse([join "crank parameters ref_dataset" __])

    set in_fp $XMLParse([join "crank soap run step=$step input exp_columns crystal=$c data=$d f" __])
    set in_sigfp $XMLParse([join "crank soap run step=$step input exp_columns crystal=$c data=$d sigf" __])

    # Sequence file

    set in_seq ""
    if { [file exists input.seq ] } {
	set in_seq [file join $orig_pwd $step-buccaneer input.seq]
	set reliable 0.95
	if { [info exists XMLParse([join "crank soap run step=$step buccaneer sequence_match" __])] } {
	    if { $XMLParse([join "crank soap run step=$step buccaneer sequence_match" __]) == "definite" } {
		set reliable 0.99
	    } elseif { $XMLParse([join "crank soap run step=$step buccaneer sequence_match" __]) == "possible" } {
 		set reliable 0.5
	    }
	}
    }	

    # HL columns

    if { [info exists XMLParse([join "crank soap run step=$step input hl_columns hla" __])] &&
	  [string compare $XMLParse([join "crank soap run step=$step input hl_columns hla" __]) "NONE"] != 0 } {
	set in_hla $XMLParse([join "crank soap run step=$step input hl_columns hla" __])
	set in_hlb $XMLParse([join "crank soap run step=$step input hl_columns hlb" __])
	set in_hlc $XMLParse([join "crank soap run step=$step input hl_columns hlc" __])
	set in_hld $XMLParse([join "crank soap run step=$step input hl_columns hld" __])
     }

    # Free R column

    if { [info exists XMLParse([join "crank soap run step=$step input freer_columns freer" __])] } {
       set in_freer $XMLParse([join "crank soap run step=$step input freer_columns freer" __])
    }  else {
       set in_freer ""
    }

    if { [info exists XMLParse([join "crank soap run step=$step buccaneer nousefreer" __])] } {
	if { $XMLParse([join "crank soap run step=$step buccaneer nousefreer" __]) } {
	    set in_freer ""
	}
    } 

    # options

    set fragments 20
    if { [info exists XMLParse([join "crank soap run step=$step buccaneer fragments" __])] } {
	if { $XMLParse([join "crank soap run step=$step buccaneer fragments" __]) > 0 } {
	    set fragments $XMLParse([join "crank soap run step=$step buccaneer fragments" __])
	}
    }
    
    set numcycles 3

    if { [info exists XMLParse([join "crank soap run step=$step buccaneer num_cycles" __])] } {
	if { $XMLParse([join "crank soap run step=$step buccaneer num_cycles" __]) } {
	    set numcycles $XMLParse([join "crank soap run step=$step buccaneer num_cycles" __])
	}
    }

    set fastbuild 0

    if { [info exists XMLParse([join "crank soap run step=$step buccaneer fast_build" __])] } {
	if { $XMLParse([join "crank soap run step=$step buccaneer fast_build" __]) } {
	    set fastbuild 1
	}
    }

    puts "\#!/bin/sh" 
    puts " "

    if { $fastbuild } {

	puts "$ex -stdin << end-buccaneer > buccaneer-logfile"
	puts "mtzin-ref [file join $ref reference-1ajr.mtz]"
	puts "pdbin-ref [file join $ref reference-1ajr.pdb]"
	puts "colin-ref-fo /*/*/\[FP.F_sigF.F,FP.F_sigF.sigF\] "
	puts "colin-ref-hl /*/*/\[FC.ABCD.A,FC.ABCD.B,FC.ABCD.C,FC.ABCD.D\] "
	puts "mtzin-wrk $mtz_in"
	puts "colin-wrk-fo /*/*/\[$in_fp,$in_sigfp\] "
	puts "colin-wrk-hl /*/*/\[$in_hla,$in_hlb,$in_hlc,$in_hld\] "
	if { $in_freer != "" } {
	    puts "colin-wrk-free /*/*/\[$in_freer\] "
	}
	if { $in_seq != "" } {
	    puts "seqin-wrk $in_seq\nsequence\nsequence-reliability $reliable"
	}
	puts "find\ngrow\njoin\ncorrect\nprune\nrebuild "
	puts "fragments-per-100-residues $fragments "
	puts "end-buccaneer"
    } else {
	puts "python -u [file join $env(CBIN) ccp4_pipeline_simple] < buccaneer-script > buccaneer-logfile"
	set scr [open "buccaneer-script" w+ ]
	puts $scr "%loop-cyc 3"
	puts $scr "%copy-pre $mtz_in temporary_refmac.mtz"
	puts $scr "%prgm-loop buccaneer cbuccaneer %exit"
	puts $scr "%prgm-loop refmac refmac5 %exit"
	puts $scr "%copy-post temporary_refmac.pdb buccaneer.out.pdb"
	puts $scr "buccaneer %arg -stdin"
#	puts "buccaneer title [No title given]"
	puts $scr "buccaneer pdbin-ref [file join $ref reference-1ajr.pdb]"
	puts $scr "buccaneer mtzin-ref [file join $ref reference-1ajr.mtz]"
	puts $scr "buccaneer colin-ref-fo /*/*/\[FP.F_sigF.F,FP.F_sigF.sigF\]"
	puts $scr "buccaneer colin-ref-hl /*/*/\[FC.ABCD.A,FC.ABCD.B,FC.ABCD.C,FC.ABCD.D\]"
	puts $scr "buccaneer colin-wrk-fo /*/*/\[$in_fp,$in_sigfp\] "
	puts $scr "buccaneer:0 colin-wrk-hl /*/*/\[$in_hla,$in_hlb,$in_hlc,$in_hld\] "
	puts $scr "buccaneer mtzin-wrk $mtz_in"
	if { $in_freer != "" } {
	    puts $scr "buccaneer colin-wrk-free /*/*/\[$in_freer\] "
	}
	if { $in_seq != "" } {
	    puts $scr "buccaneer seqin-wrk $in_seq\nsequence\nsequence-reliability $reliable"
	}
	puts $scr "buccaneer:1-* colin-wrk-phifom /*/*/\[PHCOMB,FOM\]"
	puts $scr "buccaneer:1-* pdbin-wrk temporary_refmac.pdb"
	puts $scr "buccaneer pdbout-wrk temporary_buccaneer.pdb"
	puts $scr "buccaneer find"
	puts $scr "buccaneer grow"
	puts $scr "buccaneer join"
	puts $scr "buccaneer link"
	puts $scr "buccaneer sequence"
	puts $scr "buccaneer correct"
	puts $scr "buccaneer filter"
	puts $scr "buccaneer ncsbuild"
	puts $scr "buccaneer prune"
	puts $scr "buccaneer rebuild"
	puts $scr "buccaneer:0 cycles 3"
	puts $scr "buccaneer:0 sequence-reliability 0.95"
	puts $scr "buccaneer:1-* cycles 1"
	puts $scr "buccaneer:1-* correlation-mode"
	puts $scr "buccaneer:1-* sequence-reliability 0.95"
	puts $scr "buccaneer new-residue-name UNK"
	puts $scr "buccaneer resolution 2.0"
	puts $scr "refmac %arg HKLIN $mtz_in"
	puts $scr "refmac %arg XYZIN temporary_buccaneer.pdb"
	puts $scr "refmac %arg HKLOUT temporary_refmac.mtz"
	puts $scr "refmac %arg XYZOUT temporary_refmac.pdb"
#	puts "refmac title [No title given]"
	puts $scr "refmac weight MATRIX 0.1"
	puts $scr "refmac labin FP=$in_fp SIGFP=$in_sigfp -"
	if { $in_freer != "" } {
	    puts $scr "refmac FREE=$in_freer -"
	} 
	puts $scr "refmac HLA=$in_hla  HLB=$in_hlb HLC=$in_hlc HLD=$in_hld"
	puts $scr "refmac "
	puts $scr "refmac make check NONE"
	puts $scr "refmac make hydrogen YES hout NO peptide NO cispeptide YES ssbridge YES symmetry YES sugar YES connectivity NO link NO"
	puts $scr "refmac refi type REST PHASE SCBL 1.0 BBLU 0.0 resi MLKF meth CGMAT bref ISOT"
	puts $scr "refmac ncyc 10"
	puts $scr "refmac scal type SIMP LSSC ANISO EXPE"
	puts $scr "refmac solvent YES VDWProb 1.4 IONProb 0.8 RSHRink 0.8"
	puts $scr "refmac monitor MEDIUM torsion 10.0 distance 10.0 angle 10.0 plane 10.0 chiral 10.0 bfactor 10.0 bsphere 10.0 rbond 10.0 ncsr 10.0"
	puts $scr "refmac labout FC=FC FWT=FWT PHIC=PHIC PHWT=PHWT DELFWT=DELFWT PHDELWT=PHDELWT FOM=FOM PHCOMB=PHCOMB"
	puts $scr "refmac PNAME buccaneer"
	puts $scr "refmac DNAME buccaneer"
	puts $scr "refmac RSIZE 80"
	puts $scr "refmac END"
	flush $scr
	close $scr
    }
}

#
set inputxml [lindex $argv 0]
set crankbin [lindex $argv 1]
set ref      [lindex $argv 2]
set ex       [lindex $argv 3]

if { [file exists $inputxml] == 0 } {
    crank_error "xml2buccaneercom.tcl::inputxml file does not exist"
}

source [file join $crankbin crankutils.tcl]

XMLParsefile $inputxml

OutputBuccaneercomfile $ref $ex
