#
# Copyright (C) Navraj S. Pannu, Steven Ness, and RAG de Graaff, 2004-2005 Leiden University
# Copyright (C) Navraj S. Pannu and RAG de Graaff, 2006-2008 Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc crunch2_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    global env

    #
    # Run CRUNCH2
    #
    crank_run_message "$step-crunch2"

    file mkdir $step-crunch2
    cd $step-crunch2
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    set fa $XMLParse([join "crank soap run step=$step input fa_columns fa" __])
    set sigfa $XMLParse([join "crank soap run step=$step input fa_columns sigfa" __])
    set ea $XMLParse([join "crank soap run step=$step input ea_columns ea" __])
    set sigea $XMLParse([join "crank soap run step=$step input ea_columns sigea" __])

    runcommand [concat [file join $crankplugins crunch2 mtz2drear.tcl] [file join .. workdb crank.in.$tag.mtz] crunch2.drear $fa $sigfa $ea $sigea]

    if { $XMLParse([join "crank soap run step=$step crunch2 patt_search" __]) == 1 }  {

	if { $inccp4 } {
	    set pmfexecutable [file join $env(CBIN) pmf]
        } else {
	    set pmfexecutable [file join $crankbin  pmf]
        }

	# Run PMF
	runcommand [concat [file join $crankplugins crunch2 xml2pmfcom.tcl] $inputxml $pmfexecutable $crankbin > pmf-command.com]
	runcommand [concat [file join $crankplugins crunch2 xml2pmftcl.tcl] $inputxml $pmfexecutable $crankbin > pmf-command.tcl]

	runcommand [concat tclsh pmf-command.tcl]

#       set i 2
#	while { [info exists XMLParse([join "crank input_files otherdata=$i type" __])] } {
#    if { $XMLParse([join "crank input_files otherdata=$i type" __]) == "SUBSTRUCTURE" } {
#	set pdb [open $XMLParse([join "crank input_files otherdata=$i filename" __]) r]
#	set indata [split [read $pdb] "\n"]
#	set num_atoms 0
#	set patterson [open patterson.xyz a+]
#		foreach line  $indata {
#	    if { [regexp {HETATM.*} $line junk] || [regexp {ATOM.*} $line junk] } {
#		incr num_atoms
#	    }
#	}
#	puts $patterson [format "%5d" $num_atoms]
#	set fracx 0.0000
#	set fracy 0.0000
#	set fracz 0.0000

#		foreach line $indata {
#	    if { [regexp {HETATM.*} $line junk] || [regexp {ATOM.*} $line junk] } {
#		scan $line "%6s%5d%*c%*c%4s%*c%3s%*11s%8f%8f%8f%6f" label number atom residue x y z occ 
#		orth2frac $x $y $z fracx fracy fracz
#		puts $patterson [format "%3.1f      %10.3f%10.3f%10.3f" 15.0 $fracx $fracy $fracz]
#	    }
#	}
#	close $patterson
#	close $pdb
#    }
#    incr i
#}
	set total_zeros 0
	if { [file exists patterson.xyz] } {
	    set coordinate_fileid [open patterson.xyz r]
	    for { set i 0 } { $i < 5 } { incr i } {
		gets $coordinate_fileid line
		if { [string compare [string trim $line " "] "0"] == 0 } {
		    incr total_zeros
		}
	    }
	    close $coordinate_fileid
	    if { $total_zeros > 3 } {
		puts "crank::pmf did not find any patterson-compliant solutions"
		file delete patterson.xyz
	    }
	}
    }

    if { $inccp4 } {
        set executable [file join $env(CBIN) crunch2]
    } else {
        set executable [file join $crankbin  crunch2]
    }

   set icol 0
    if { [file exists patterson.xyz] } {
	set icol 1
    }

    # Set atom name as the first atom name we encounter while looping over all crystals
    set i 1
    while { ![info exists XMLParse([join "crank parameters crystal=$i substructure atom_name" __])] } { 
	incr i
    }
    set atom_name [string toupper $XMLParse([join "crank parameters crystal=$i substructure atom_name" __])]

    set score_threshold $XMLParse([join "crank soap run step=$step crunch2 score_threshold" __])
    
    set deviation $XMLParse([join "crank soap run step=$step crunch2 deviation" __])

    set max_trials $XMLParse([join "crank soap run step=$step crunch2 ntrials" __])
    
    set before_bp3 $XMLParse([join "crank soap run step=$step crunch2 bp3_verify" __])

    array set score {}
    array set previousindex {}
    set testind  0
    set minscore 2.0
    set maxscore 0
    set maxindex 0
    set iter 1
    
    while { (![file exists done] && ($iter <= $max_trials)) } {

	runcommand [concat [file join $crankplugins crunch2 xml2crunch2com.tcl] $inputxml $executable $icol $iter $before_bp3 $max_trials $crankbin > crunch2-command$iter.com]
	runcommand [concat [file join $crankplugins crunch2 xml2crunch2tcl.tcl] $inputxml $executable $icol $iter $before_bp3 $max_trials $crankbin > crunch2-command$iter.tcl]

	runcommand [concat tclsh crunch2-command$iter.tcl]

	if { ![file exists done] && [file exists trial1a.XYZ] } { 
	    set bp3_luzz 0.70
	    
	    set alltrials [glob trial*a.XYZ]
	    set nindex 0
	    foreach trial $alltrials {
		incr nindex
		set index [string trimleft [string trimright $trial "a.XYZ"] "trial"]
		set score($index) [exec tail -1 $trial]
		if { $score($index) > $maxscore } {
		    set maxscore $score($index)
		    set maxindex $index
		}
		if { $score($index) < $minscore } {
		    set minscore $score($index)
		}
	    }

	    if { ( ( $maxscore > $score_threshold) || ( ( $maxscore > [expr $deviation * $minscore] ) && ($maxscore > 0.3) ) ) } {
		set dum [open "done" w+ ]
		puts $dum "Crunch2 parameters indicates solution"
		flush $dum
		close $dum
		break
	    }

	    if { [file exists trial${max_trials}a.XYZ] } {
		set dum [open "done" w+ ]
		puts $dum "Maximum trials completed"
		flush $dum
		close $dum
		break
	    }		

	    set mod 0
	    if { ($iter > 1) } {
		set mod  [expr $iter % $before_bp3]
	    }

	    set alreadytested 0

	    if {$testind > 0} {
		for { set i 1 } { $i <= $testind } { incr i } {
		    if { $previousindex($i) == $maxindex } {
			set alreadytested 1
			break
		    }
		}
	    }

	    if { ($before_bp3 > 0) && ($alreadytested == 0) && ($mod == 0) && ( ($maxscore > 0.21) || ($maxscore > [expr $minscore * 1.05]) ) && ($maxindex > [expr $iter-$before_bp3-1] ) } {
		incr testind
		set previousindex($testind) $maxindex 
		check_atom trial${maxindex}a.pdb substructure$iter.pdb
                if { $inccp4 } {
                    set bp3_executable [file join $env(CBIN) bp3]
                } else {
                    set bp3_executable [file join $crankbin  bp3]    
                }
		runcommand [concat [file join $crankplugins bp3 xml2bp3com.tcl] $inputxml substructure$iter.pdb $iter $bp3_executable 1 $crankbin > bp3-command$iter.com]
		runcommand [concat [file join $crankplugins bp3 xml2bp3tcl.tcl] $inputxml substructure$iter.pdb $iter $bp3_executable 1 $crankbin > bp3-command$iter.tcl]

		runcommand [concat tclsh bp3-command$iter.tcl]
		#runcommand [concat chmod ugo+x ./bp3-command$iter.com]
		#runcommand [concat ./bp3-command$iter.com]
		set logfile [open "bp3-logfile$iter" r]
		set data [read $logfile]
		set data [split $data "\n"]

		set luzzati 0
		set luzzatisd -1
		foreach line $data {
		    if { [regexp {average anomalous Luzzati error is *([0-9.]*).*} $line junk luzz] } {
			set luzzati $luzz
		    }
		    if { [regexp {deviation of the anomalous Luzzati error is *([0-9.]*).*} $line junk luzzsd] } {
			set luzzatisd $luzzsd
		    }
		}
		if { ($luzzati > $bp3_luzz) } {
		    set dum [open "done" w+ ]
		    puts $dum "Bp3 luzzati parameter indicates solution"
		    flush $dum
		    close $dum
		    break
		}
		if { [file exists bp3.out$iter.mtz] } {
		    file delete bp3.out$iter.mtz
		}
	    }
	}
	incr iter
    }

    #
    if [file exists trial1a.XYZ] {

        set alltrials [glob trial*a.XYZ]
	foreach trial $alltrials {
	   set score($index) [exec tail -1 $trial]
	   if { $score($index) > $maxscore } {
	       set maxscore $score($index)
	       set maxindex $index 
	   }
        }
	if { ($maxscore < 0.1 ) } {
	   crank_error "CRUNCH2 FOM is lower than 0.1 - this probably means a solution was not found"
	}

	check_atom trial${maxindex}a.pdb crank.out.$tag.substructure.pdb
 	file copy crank.out.$tag.substructure.pdb [file join .. workdb crank.out.$tag.substructure.pdb]
 	file delete crank.out.$tag.substructure.pdb 

    } else {
       crank_error "No crunch2 trials have been produced"
    }

    runcommand [concat cat crunch2-logfile]

    # Copy logfile to main log directory
    file copy pmf-logfile [file join .. log $step-pmf-logfile]
    file copy crunch2-logfile [file join .. log $step-crunch2-logfile]

    cd ..
}

proc crunch2_reference { } {
    global XMLParse
    
    puts "Crunch2:"
    puts "de Graaff, R.A.G., Hilge, M., van der Plas, J.L. & Abrahams, J.P. (2001)"
    puts "Matrix methods for solving protein substructures of chlorine and"
    puts "sulfur from anomalous data. Acta Cryst. D57, 1857-1862.\n"
}
