#
# Copyright (C) Navraj S. Pannu and Steven Ness 2004-2005, Leiden University
# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc dm_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    #
    # Run DM for density modification
    #
    crank_run_message "Running $step-dm"
    
    file mkdir $step-dm
    cd $step-dm
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]
    set ncycle 0
    set enant 1

    if { [info exists XMLParse([join "crank update solvent_content" __])] } {
	set solv $XMLParse(crank__update__solvent_content)
    } else {
	set solv $XMLParse(crank__parameters__solvent_content)
    }
    
    if { $XMLParse([join "crank soap run step=$step dm determine_hand" __]) == "1" } {

	file mkdir hand1
	cd hand1
	runcommand [concat [file join $crankplugins dm xml2dmcom.tcl] $inputxml $enant $crankbin $solv >& dm-command.com]
	runcommand [concat [file join $crankplugins dm xml2dmtcl.tcl] $inputxml $enant $crankbin $solv >& dm-command.tcl]

        runcommand [concat tclsh dm-command.tcl]	
	#runcommand [concat chmod ugo+x ./dm-command.com]
	#runcommand [concat ./dm-command.com]
	cd ..

	file mkdir hand2
	cd hand2
	set enant 2
	set temptag [string trim $XMLParse([join "crank soap run step=$step input exp_columns crystal=1 data=1 f" __])]
	set ftag [string trimright $temptag "_X1_D1_F" ]
	set temptag [string trim $XMLParse([join "crank soap run step=$step input phase_columns f" __])]
	set bp3tag [string trim $temptag _F]
	set sftools_script "read [file join .. .. workdb crank.out.$bp3tag.oh.mtz]\n"
	set sftools_script "$sftools_script read [file join .. .. workdb crank.out.$ftag.mtz]\n"
	set sftools_script "$sftools_script write [file join .. .. workdb crank.in.$tag.oh.mtz]\nquit\n"
	set command "sftools << \"$sftools_script\""
	puts $command
	catch {eval exec $command } output

	runcommand [concat [file join $crankplugins dm xml2dmcom.tcl] $inputxml $enant $crankbin $solv >& dm-command.com]
	runcommand [concat [file join $crankplugins dm xml2dmtcl.tcl] $inputxml $enant $crankbin $solv >& dm-command.tcl]
	
	runcommand [concat tclsh dm-command.tcl]
	#runcommand [concat chmod ugo+x ./dm-command.com]
	#runcommand [concat ./dm-command.com]
	cd ..

	# determine correct hand
	set tol $XMLParse([join "crank soap run step=$step dm margin" __])
	# Select the correct enantiomorph from dm log file
	runcommand [concat [file join $crankplugins dm dmhand.tcl] $inputxml $crankbin $tol |& tee dmhand-logfile]

	set logfile1 [open "dmhand-logfile" r]
        set data [read $logfile1]
        set data [split $data "\n"]

 	foreach line $data {
	    if { $line       == "First hand selected" } {
		set enant 1
		break
 	    } elseif { $line == "Second hand selected" } {
		set enant 2
		break
	    }
	}

	close $logfile1

	file copy dmhand-logfile [file join .. log $step-dmhand-logfile]
    }

    if { $XMLParse([join "crank soap run step=$step dm density_modify" __]) == "1" } {

  	set solv $XMLParse(crank__parameters__solvent_content)

	if { [file exists dmoptimize-logfile] } {
	    set logfile1 [open "dmoptimize-logfile" r]
	    set data [read $logfile1]
	    set data [split $data "\n"]
	    foreach line $data {
		if { [regexp {Choosing solvent content* **([0-9.]*).*} $line junk solvent ] } {
		    set solv $solvent
		    break
		}
	    }
	    close $logfile1
	} 
	
	runcommand [concat [file join $crankplugins dm xml2dmcom.tcl] $inputxml $enant $crankbin $solv >& dm-command.com]
	runcommand [concat [file join $crankplugins dm xml2dmtcl.tcl] $inputxml $enant $crankbin $solv >& dm-command.tcl]

        runcommand [concat tclsh dm-command.tcl]

#  	runcommand [concat chmod ugo+x ./dm-command.com]
#       runcommand [concat ./dm-command.com]

	# Rename output columns to the names specified in the XML
	if { [file exists dm.out.mtz] } {
	    runcommand [concat [file join $crankplugins dm crank-rename-dm-output.tcl] $inputxml $crankbin dm.out.mtz rename.out.mtz]
	    file copy rename.out.mtz [file join .. workdb crank.out.$tag.mtz]
	    file delete rename.out.mtz
	} else {
	    crank_error "crank::rundm:: dm.out.mtz file not present.  Exitting."
	}

	file copy dm-logfile [file join .. log $step-dm-logfile]
    }

    cd ..
}

proc dm_reference { } {
    global XMLParse
    
    puts "DM:"
    puts "Cowtan, K. (1994) Joint CCP4 and ESF-EACBM Newsletter on Protein"
    puts "Crystallography. 31, 34-38.\n"
} 
