# Copyright (C) 2006-2007 Leiden University
#
# Authors: Navraj S. Pannu and Steven Ness 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc prep_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    #
    #
    crank_run_message "$step-prep"

    file mkdir $step-prep
    cd $step-prep
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    file copy [file join .. workdb crank.out.IN.mtz] extracted.mtz

    if { [string compare $XMLParse([join "crank parameters input_intensity" __]) "1"] == 0 } {
	# Run truncate on all crystal/datasets to convert I+/I- to F+/F-
	run_all_truncate

	# Combine all resultant datasets together and rename where necessary
	rename_input_columns $step

    } else {
	if { [info exists XMLParse([join "crank soap run step=$step prep rescale_f" __])] } {
	    if { $XMLParse([join "crank soap run step=$step prep rescale_f" __]) } {
		run_apply_scale $crankbin $inccp4 
	    } else {
	        file copy extracted.mtz output.mtz
	    }
	}
    }

    file delete extracted.mtz

    set mtzout "output.mtz"

    # FreeR factor
    # Output FREER column name
    set out_freer_column "${step}_PREP_FREER"

    set inrfree 0
    if { [info exists XMLParse([join "crank soap run step=$step prep input_rfree" __])] } {
        if { $XMLParse([join "crank soap run step=$step prep input_rfree" __]) && 
            [info exists XMLParse([join "crank soap run step=$step prep rfree_label" __])] } {
	    set inrfree 1
        }
    }	

    if { $inrfree } {
        
	# Input FREER column name
	set in_freer_column $XMLParse([join "crank soap run step=$step prep rfree_label" __])

	set sftools_script "read $mtzin\n set label col $in_freer_column \n $out_freer_column \n"
	set sftools_script "$sftools_script write rfree.mtz col $out_freer_column \n quit \n"
	set command "sftools << $sftools_script"
	catch {eval exec $command } output 
	set sftools_script "read $mtzout\n read rfree.mtz\n write $mtzout\ny \n quit \n"
	set command "sftools << $sftools_script"
	catch {eval exec $command } output 
	file delete rfree.mtz

    } else {

	# Output FREER FRACTION

	if { [info exists XMLParse([join "crank soap run step=$step prep rfree_fraction" __])] } {	
	    set freer_fraction $XMLParse([join "crank soap run step=$step prep rfree_fraction" __])
	} else {
	    set freer_fraction 0.05
	}

	set sftools_script "read $mtzout\n calc I col $out_freer_column = rfree($freer_fraction) \n"
	set sftools_script "$sftools_script write $mtzout\ny \n quit\n"
	set command "sftools << \"$sftools_script\""
        catch {eval exec $command } output 
    }

    # Run scaling
    runcommand [concat [file join $crankplugins prep xml2scaleitcom.tcl] $inputxml $crankbin > scaleit-command.com]
    runcommand [concat [file join $crankplugins prep xml2scaleittcl.tcl] $inputxml $crankbin > scaleit-command.tcl]

    runcommand [concat tclsh scaleit-command.tcl]
    #runcommand [concat chmod ugo+x ./scaleit-command.com]
    #runcommand [concat ./scaleit-command.com]

    file delete output.mtz

    # Rename output columns to the names specified in the XML
    runcommand [concat [file join $crankplugins prep crank-rename-prep-output.tcl] $inputxml $crankbin $step scaleit.out.mtz rename.out.mtz]
    file copy rename.out.mtz [file join .. workdb crank.out.$tag.mtz]
    file delete rename.out.mtz

    runcommand [concat cat scaleit-logfile]

    # Copy logfile to main log directory
    file copy scaleit-logfile [file join .. log $step-prep-scaleit-logfile]

    cd ..
}

proc prep_reference { } {
    global XMLParse

    puts "Truncate/Scaleit:"
    puts "CCP4 (1994) The CCP4 Suite: Programs for protein crystallography."
    puts "Acta Cryst. D50, 760-763.\n"
}

proc run_truncate { i j } {
    global XMLParse

    # Setup variables
    set mtzin "extracted.mtz"
    set mtzout "truncate_${i}_${j}.mtz"

    set anomalous $XMLParse([join "crank parameters crystal=$i data=$j anomalous" __])
    set nmonomers $XMLParse(crank__parameters__nmonomers)
    set nnucleotides $XMLParse(crank__parameters__nucleotides) 
    set nresidues $XMLParse(crank__parameters__residues) 
    set nhydro [expr ($nresidues*8 + $nnucleotides*12)*$nmonomers]
    set ncarb [expr ($nresidues*4.869 + $nnucleotides*9.683)*$nmonomers]
    set nnitr [expr ($nresidues*1.351 + $nnucleotides*3.911)*$nmonomers]
    set noxyg [expr ($nresidues*1.492 + $nnucleotides*6.424)*$nmonomers]
    set nsulf [expr $nresidues*0.051*$nmonomers]
    set nphos [expr $nnucleotides*$nmonomers]
    set type $XMLParse([join "crank parameters crystal=$i data=$j type" __])

    set in_i "IN_X${i}_D${j}_I"
    set in_sigi "IN_X${i}_D${j}_SIGI"

    if { $anomalous == 1 } { 
	set in_ip "IN_X${i}_D${j}_I+"
	set in_sigip "IN_X${i}_D${j}_SIGI+"
	set in_im "IN_X${i}_D${j}_I-"
	set in_sigim "IN_X${i}_D${j}_SIGI-"
    }

    set in_f "IN_X${i}_D${j}_F"
    set in_sigf "IN_X${i}_D${j}_SIGF"
 
    if { $anomalous == 1 } { 
	set in_fp "IN_X${i}_D${j}_F+"
	set in_sigfp "IN_X${i}_D${j}_SIGF+"
	set in_fm "IN_X${i}_D${j}_F-"
	set in_sigfm "IN_X${i}_D${j}_SIGF-"
    }

    # Create command line
    set command "HKLIN $mtzin HKLOUT $mtzout"

    # Create script
    if { $anomalous == 0 } {
	set labin "labin IMEAN=$in_i SIGIMEAN=$in_sigi"
	set labout "labout IMEAN=$in_i SIGIMEAN=$in_sigi F=$in_f SIGF=$in_sigf"
    }  else {
	set labin "labin IMEAN=$in_i SIGIMEAN=$in_sigi I(+)=$in_ip SIGI(+)=$in_sigip I(-)=$in_im SIGI(-)=$in_sigim"
	set labout "labout I(+)=$in_ip SIGI(+)=$in_sigip I(-)=$in_im SIGI(-)=$in_sigim - \n"
 	set labout "$labout F=$in_f SIGF=$in_sigf - \n"
 	set labout "$labout F(+)=$in_fp SIGF(+)=$in_sigfp F(-)=$in_fm SIGF(-)=$in_sigfm"

    }

    set script "\ntitle test convert scalepack to mtz"
    set script "$script\ntruncate YES"
    if { $anomalous == 0 } {
	set script "$script\nanomalous NO"
    } else {
	set script "$script\nanomalous YES"
    }
    set script "$script\ncontents H $nhydro C $ncarb N $nnitr O $noxyg S $nsulf P $nphos"
    set script "$script\nplot OFF"
    set script "$script\nfalloff yes"
    set script "$script\ndname $j"
    set script "$script\nrsize 80"
    set script "$script\n$labin"
    set script "$script\n$labout"
    set script "$script\nEND\n"
	
    set run_command "truncate $command << \"$script\""
    puts $run_command
    catch {eval exec $run_command } output 
    puts $output
    set logfile [open "truncate_${i}_${j}-logfile" w]
    puts $logfile $output
    file copy truncate_${i}_${j}-logfile [file join .. log 1-prep-truncate_${i}_${j}-logfile]
    close $logfile

    # remove dano, sigdano and isym columns
    set sftools_script "read truncate_${i}_${j}.mtz\ndelete col DANO\ndelete col SIGDANO\ndelete col ISYM\n"
    set sftools_script "$sftools_script write truncate_${i}_${j}.mtz\ny\nquit\n"
    set run_command "sftools << \"$sftools_script\""
    catch {eval exec $run_command } output 
}

proc run_apply_scale { crankbin inccp4 } {
    global XMLParse
    global env

    if { $inccp4 } {
	set gcx_ex [file join $env(CBIN) gcx]
    } else {
	set gcx_ex [file join $crankbin  gcx]
    }

    set script "\nSCALE\n"

    if { [info exists XMLParse([join "crank parameters seqin filename" __]) ] } {
        set seq_in $XMLParse([join "crank parameters seqin filename" __])
        set command "$gcx_ex HKLIN extracted.mtz HKLOUT output.mtz SEQIN $seq_in "
    } else {
        if { [info exists XMLParse([join "crank parameters residues" __]) ] } {
            set script "$script NRES $XMLParse(crank__parameters__residues)\n"
        } 

        if { [info exists XMLParse([join "crank parameters nucleotides" __]) ] } {
            set script "$script NNUC $XMLParse(crank__parameters__nucleotides)\n"
        } 
        set command "$gcx_ex HKLIN extracted.mtz HKLOUT output.mtz "
    }

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } { 
	set script "$script XTAL CRYSTAL$i\n" 
	if { [info exists XMLParse([join "crank parameters crystal=$i substructure atom_name" __])] } {
	    set script "$script   ATOM $XMLParse([join "crank parameters crystal=$i substructure atom_name" __])\n"
	    set script "$script   NUMB [expr $XMLParse([join "crank parameters crystal=$i substructure natoms" __])/$XMLParse([join "crank parameters nmonomers" __])]\n"
	}
	set j 1
	while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } { 
	    set script "$script   DNAME DATASET$j\n"
	    if { $XMLParse([join "crank parameters crystal=$i data=$j anomalous" __]) == 1 } {
		set script "$script     COLU F=IN_X${i}_D${j}_F SF=IN_X${i}_D${j}_SIGF F+=IN_X${i}_D${j}_F+ SF+=IN_X${i}_D${j}_SIGF+ F-=IN_X${i}_D${j}_F- SF-=IN_X${i}_D${j}_SIGF-\n"
	    } else {
		set script "$script     COLU F=IN_X${i}_D${j}_F SF=IN_X${i}_D${j}_SIGF\n"
	    }
	    if { [info exists XMLParse([join "crank parameters crystal=$i substructure atom_name" __])] && 
		 [info exists XMLParse([join "crank parameters crystal=$i data=$j fprime" __])] && 
		 [info exists XMLParse([join "crank parameters crystal=$i data=$j fprimeprime" __])] } {
		set script "$script     FORM $XMLParse([join "crank parameters crystal=$i substructure atom_name" __]) FP=$XMLParse([join "crank parameters crystal=$i data=$j fprime" __]) FPP= $XMLParse([join "crank parameters crystal=$i data=$j fprimeprime" __])\n"
	    } 
	    incr j
	}
	incr i
    }
	
    set run_command "$command << \"$script\""
    puts $run_command
    catch {eval exec $run_command } output 
#    puts $output
    set logfile [open "gcx-logfile" w]
    puts $logfile $output
    close $logfile
}

#
# Run truncate on all crystals/datasets separately to convert the
# input intensities (I's) to amplitudes (F's)
#
proc run_all_truncate { } {
    global XMLParse
    
    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } { 
	set j 1
	while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } { 
	    # Run truncate on this crystal/dataset
	    run_truncate $i $j
	    incr j
	}
	incr i
    }
}

proc rename_input_columns { step } {
    global XMLParse

    set sftools_script ""

    set i 1
    while { [info exists XMLParse([join "crank parameters crystal=$i data=1 type" __])] } { 
	set j 1
	while { [info exists XMLParse([join "crank parameters crystal=$i data=$j type" __])] } { 
	    set sftools_script "$sftools_script read truncate_${i}_${j}.mtz\n"
	    incr j
	}
	incr i
    }

    set mtzout "output.mtz"
    set sftools_script "$sftools_script write $mtzout\n\quit\n"
    set command "sftools << \"$sftools_script\""
#   puts $command
    catch {eval exec $command } output 
#   puts $output
}
