#
# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc resolvedm_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    #
    # Run RESOLVE for density modification
    #
    crank_run_message "Running $step-resolvedm"
    
    file mkdir $step-resolvedm
    cd $step-resolvedm
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    if { [info exists XMLParse([join "crank update solvent_content" __])] } {
	set solv $XMLParse(crank__update__solvent_content)
    } else {
	set solv $XMLParse(crank__parameters__solvent_content)
    }
    
    if { [info exists XMLParse([join "crank parameters sequence" __])] } {
	set seq [open seq.dat w+]
	puts $seq "$XMLParse([join "crank parameters sequence" __])"
	flush $seq
	close $seq
    }

    set substructure_tag $XMLParse([join "crank soap run step=$step input substructure" __])

    set sfile [file join .. workdb crank.out.$substructure_tag.substructure.pdb] 
	
    runcommand [concat [file join $crankplugins resolvedm xml2resolvedmcom.tcl] $inputxml $sfile $crankbin $solv >& resolvedm-command.com]
    runcommand [concat [file join $crankplugins resolvedm xml2resolvedmtcl.tcl] $inputxml $sfile $crankbin $solv >& resolvedm-command.tcl]

    runcommand [concat tclsh resolvedm-command.tcl]

    # Rename output columns to the names specified in the XML
    if { [file exists resolvedm.out.mtz] } {
        runcommand [concat [file join $crankplugins resolvedm crank-rename-resolvedm-output.tcl] $inputxml $crankbin resolvedm.out.mtz rename.out.mtz]
	file copy rename.out.mtz [file join .. workdb crank.out.$tag.mtz]
	file delete rename.out.mtz
    } else {
        crank_error "crank::runresolvedm:: resolvedm.out.mtz file not present.  Exitting."
    }

    if { [file exists resolvedm-logfile] } {
	file copy resolvedm-logfile [file join .. log $step-resolvedm-logfile]
	runcommand [concat cat resolvedm-logfile]
    } else {
        crank_error "crank::runresolvedm:: resolvedm-logfile file not present.  Exitting."
    }

    cd ..
}

proc resolvedm_reference { } {
    global XMLParse
    
    puts "RESOLVEDM:"
    puts "Terwilliger, T.C. (2000) Maximum likelihood density modification."
    puts "Acta Cryst. D56, 965-972.\n"
} 
