#
# Copyright (C) Navraj S. Pannu 2006-2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc resolvemb_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    #
    # Run RESOLVE for model building
    #
    crank_run_message "Running $step-resolvemb"
    
    file mkdir $step-resolvemb
    cd $step-resolvemb
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    if { [info exists XMLParse([join "crank update solvent_content" __])] } {
	set solv $XMLParse(crank__update__solvent_content)
    } else {
	set solv $XMLParse(crank__parameters__solvent_content)
    }

    set substructure_tag $XMLParse([join "crank soap run step=$step input substructure" __])
    
    set sfile [file join .. workdb crank.out.$substructure_tag.substructure.pdb]
    
    if { [info exists XMLParse([join "crank parameters sequence" __])] } {
	set seq [open seq.dat w+]
	regsub -all {[^a-zA-Z]} $XMLParse([join "crank parameters sequence" __]) "" sequence
 	for {set i 0 } { $i < [string length $sequence] } { incr i 60 } {
	    if { [expr $i + 59] <  [string length $sequence] } {
		puts $seq "[string range $sequence $i [expr $i + 59] ]" 
 	    } else {
		puts $seq "[string range $sequence $i  end]" 
	    }
	}	
	flush $seq
	close $seq
    }
	
    runcommand [concat [file join $crankplugins resolvemb xml2resolvembcom.tcl] $inputxml $sfile $crankbin $solv >& resolvemb-command.com]
    runcommand [concat [file join $crankplugins resolvemb xml2resolvembtcl.tcl] $inputxml $sfile $crankbin $solv >& resolvemb-command.tcl]

    runcommand [concat tclsh resolvemb-command.tcl]

    # Rename output columns to the names specified in the XML
    if { [file exists resolvemb.out.mtz] } {
        runcommand [concat [file join $crankplugins resolvemb crank-rename-resolvemb-output.tcl] $inputxml $crankbin resolvemb.out.mtz rename.out.mtz]
	file copy rename.out.mtz [file join .. workdb crank.out.$tag.mtz]
	file delete rename.out.mtz
     } else {
        crank_error "crank::runresolvemb:: resolvemb.out.mtz file not present.  Exitting."
     }

    if { [file exists resolvemb-logfile] } {
	file copy resolvemb-logfile [file join .. log $step-resolvemb-logfile]
	runcommand [concat cat resolvemb-logfile]
    } else {
        crank_error "resolvemb-logfile not created"
    }

     if { [file exists resolve.pdb] } {
        file copy resolve.pdb [file join .. workdb crank.out.$tag.pdb]
     } else {
        crank_error "resolve.pdb not created"
     }

     cd ..
}

proc resolvemb_reference { } {
    global XMLParse
    
    puts "RESOLVEMB:"
    puts "Terwilliger, T.C. (2003) Automated main-chain model building by"
    puts "template matching and iterative fragment extension."
    puts "Acta Cryst. D59, 38-44.\n"
} 
