# Copyright (C) 2003-2006 Leiden University
#
# Authors: Navraj S. Pannu and Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
proc shelxe_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    global env

    if { $inccp4 } {
	set gcx_ex [file join $env(CBIN) gcx] 
	set mtz2hklcommand [file join $env(CBIN) bp3]
    } else {
	set gcx_ex [file join $crankbin  gcx] 
	set mtz2hklcommand [file join $crankbin  bp3]
    }
	
    #
    # Run Shelxe
    #
    crank_run_message "$step-shelxe"

    file mkdir $step-shelxe
    cd $step-shelxe
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    set inputmtz [file join .. workdb crank.in.$tag.mtz]

    # Convert the FA, SIGFA and ALPHA columns into a crank_fa.hkl file.run/1-shelxc/crank_fa.hkl
    set fa $XMLParse([join "crank soap run step=$step input fa_columns fa" __])
    set sigfa $XMLParse([join "crank soap run step=$step input fa_columns sigfa" __])
    set alpha $XMLParse([join "crank soap run step=$step input alpha_columns alpha" __])
    runcommand [concat [file join $crankplugins shelxe mtz2hkl_fa.tcl] $inputmtz crank_fa.hkl $fa $sigfa $alpha]

    # Run SHELXC to generate SHELXE command file 
    runcommand [concat [file join $crankplugins shelxe xml2shelxccom.tcl] $inputxml $crankbin >& shelxc-command.com]
    runcommand [concat [file join $crankplugins shelxe xml2shelxctcl.tcl] $inputxml $crankbin >& shelxc-command.tcl]

    runcommand [concat tclsh shelxc-command.tcl]

    # Delete the crank.hkl file that was just created, since 
    # it could mislead SHELXE
    file delete crank.hkl
    file delete crank_fa.hkl

    # Convert the FA, SIGFA and ALPHA columns into a crank_fa.hkl file.
    if { [info exists XMLParse([join "crank soap run step=$step input fa_columns fa" __])] } {
	set fa $XMLParse([join "crank soap run step=$step input fa_columns fa" __])
	set sigfa $XMLParse([join "crank soap run step=$step input fa_columns sigfa" __])
	set alpha $XMLParse([join "crank soap run step=$step input alpha_columns alpha" __])
	runcommand [concat [file join $crankplugins shelxe mtz2hkl_fa.tcl] $inputmtz crank_fa.hkl $fa $sigfa $alpha]
    }
    
    if { [string compare $XMLParse([join "crank parameters input_intensity" __]) "1"] == 0 } {
	# Convert the I and SIGI columns into a crank.hkl file
	set i $XMLParse([join "crank soap run step=$step input i_columns i" __])
	set sigi $XMLParse([join "crank soap run step=$step input i_columns sigi" __])
	runcommand [concat [file join $crankplugins shelxe mtz2hkl.tcl] $mtz2hklcommand $inputmtz crank $i $sigi 1]
    } else {
	# Convert the F and SIGF columns into a crank.hkl file
	set f $XMLParse([join "crank soap run step=$step input f_columns f" __])
	set sigf $XMLParse([join "crank soap run step=$step input f_columns sigf" __])
	runcommand [concat [file join $crankplugins shelxe mtz2hkl.tcl] $mtz2hklcommand $inputmtz crank $f $sigf 0]
    }

    # Convert the coordinates to .res format
    set substructure $XMLParse([join "crank soap run step=$step input substructure" __])
    set substructurepdb "[file join .. workdb crank.out.$substructure.substructure.pdb]"
    runcommand [concat [file join $crankplugins shelxe pdb2shelxres.tcl] $crankbin $inputxml $substructurepdb crank_fa.ins > crank_fa.res]

    #
    # Setup shelxe command line parameters
    #

    # Solvent content
    set solvent $XMLParse(crank__parameters__solvent_content)

    set generateheavy ""
    if { [info exists XMLParse([join "crank soap run step=$step shelxe generate_heavy" __])] } {
	if { $XMLParse([join "crank soap run step=$step shelxe generate_heavy" __]) == "1" } {
	    set generateheavy "-b"
	} 
    }

    # Set heavy flag if there is no native data present
    set heavy "-h"
    set i 1 
    while { [info exists XMLParse([join "crank parameters crystal=$i native" __])] } {
	if { $XMLParse([join "crank parameters crystal=$i native" __]) == "1"  } {
	    set heavy ""
	    break
	}
	incr i
    }

    set amplitude ""
    if { [string compare $XMLParse([join "crank parameters input_intensity" __]) "1"] == 0 } {
	set amplitude ""
    } else {
	set amplitude "-f"
    }	
    
    set freelunch ""

    set invert ""

    if { $XMLParse([join "crank soap run step=$step shelxe determine_hand" __]) == "1" } {
	# first shelxe to determine hand

 	if { [info exists XMLParse([join "crank soap run step=$step shelxe hand_cycles" __])] } {
	    set handcycles $XMLParse([join "crank soap run step=$step shelxe hand_cycles" __])
	} else {
	    set handcycles 5
	}

	file mkdir hand1
	cd hand1

	set com [open "shelxe-command.com" w+]
	puts $com "/bin/cp -f [file join .. crank_fa.hkl] crank_fa.hkl"
 	puts $com "/bin/cp -f [file join .. crank.hkl] crank.hkl"
	puts $com "/bin/cp -f [file join .. crank_fa.res] crank_fa.res"
	puts $com "/bin/cp -f [file join .. crank_fa.ins] crank_fa.ins"
	puts $com "shelxe crank crank_fa -s$solvent $heavy -m$handcycles $generateheavy $invert $amplitude >> shelxe-logfile"
	puts $com "/bin/rm -f crank_fa.hkl crank.hkl crank_fa.res crank_fa.ins"
	close $com

	set tcl [open "shelxe-command.tcl" w+]
	puts $tcl "\#!usr/bin/env tclsh\n"
	puts $tcl "file copy [file join .. crank_fa.hkl] crank_fa.hkl"
	puts $tcl "file copy [file join .. crank.hkl] crank.hkl"
	puts $tcl "file copy [file join .. crank_fa.res] crank_fa.res"
	puts $tcl "file copy [file join .. crank_fa.ins] crank_fa.ins"
	puts $tcl "set command \"shelxe crank crank_fa -s$solvent $heavy -m$handcycles $generateheavy $invert $amplitude >> shelxe-logfile\""
 	puts $tcl "eval exec \$command"
 	puts $tcl "file delete crank_fa.hkl crank.hkl crank_fa.res crank_fa.ins"
	close $tcl	
	runcommand [concat tclsh shelxe-command.tcl]

	cd ..

	# first shelxe to determine hand

	file mkdir hand2
	cd hand2
 	set invert "-i"

	set com [open "shelxe-command.com" w+]
	puts $com "/bin/cp -f [file join .. crank_fa.hkl] crank_fa.hkl"
 	puts $com "/bin/cp -f [file join .. crank.hkl] crank.hkl"
	puts $com "/bin/cp -f [file join .. crank_fa.res] crank_fa.res"
	puts $com "/bin/cp -f [file join .. crank_fa.ins] crank_fa.ins"
	puts $com "shelxe crank crank_fa -s$solvent $heavy -m$handcycles $generateheavy $invert $amplitude >> shelxe-logfile"
	puts $com "/bin/rm -f crank_fa.hkl crank.hkl crank_fa.res crank_fa.ins"
	close $com

	set tcl [open "shelxe-command.tcl" w+]
	puts $tcl "\#!usr/bin/env tclsh\n"
	puts $tcl "file copy [file join .. crank_fa.hkl] crank_fa.hkl "
	puts $tcl "file copy [file join .. crank.hkl]    crank.hkl"
	puts $tcl "file copy [file join .. crank_fa.res] crank_fa.res "
	puts $tcl "file copy [file join .. crank_fa.ins] crank_fa.ins "
	puts $tcl "set command \"shelxe crank crank_fa -s$solvent $heavy -m$handcycles $generateheavy $invert $amplitude >> shelxe-logfile\""
	puts $tcl "eval exec \$command"
 	puts $tcl "file delete crank_fa.hkl crank.hkl crank_fa.res crank_fa.ins"
	close $tcl	
  	runcommand [concat tclsh shelxe-command.tcl]

	cd ..	

	# determine which hand is correct
	runcommand [concat [file join $crankplugins shelxe shelxehand.tcl] $inputxml $crankbin |& tee shelxehand-logfile]
    }

    set invert ""
    if { [file exists shelxehand-logfile] } { 
	set logfile [open "shelxehand-logfile" r]
	set data [read $logfile]
	set data [split $data "\n"]
 	foreach line $data {
	    if { $line       == "First hand selected" } {
		set invert ""
		break
	    } elseif { $line == "Second hand selected" } {
		set invert "-i"
		break
	    }
	}
	file copy shelxehand-logfile [file join .. log $step-shelxehand-logfile]
    }

    if { $XMLParse([join "crank soap run step=$step shelxe optimize_solvent" __]) == "1" } {

 	if { [info exists XMLParse([join "crank soap run step=$step shelxe opt_cycles" __])] } {
	    set optcycles $XMLParse([join "crank soap run step=$step shelxe opt_cycles" __])
	} else {
	    set optcycles 5
	}

	set initmonomers $XMLParse([join "crank parameters nmonomers" __])

	if { [info exists XMLParse([join "crank parameters seqin filename" __]) ] } {
	    set seq_in $XMLParse([join "crank parameters seqin filename" __])
	    if { [string compare $XMLParse([join "crank parameters input_intensity" __]) "1"] == 0 } {
		# Convert the I and SIGI columns into a crank.hkl file
		set i $XMLParse([join "crank soap run step=$step input i_columns i" __])
		set sigi $XMLParse([join "crank soap run step=$step input i_columns sigi" __])
		set command "$gcx_ex HKLIN $inputmtz SEQIN $seq_in << \"NOUT\n XTAL DER1\n DNAME TMP\n COLU I=$i SI=$sigi\" "

	    } else {
		# Convert the F and SIGF columns into a crank.hkl file
		set f $XMLParse([join "crank soap run step=$step input f_columns f" __])
		set sigf $XMLParse([join "crank soap run step=$step input f_columns sigf" __])
		set command "$gcx_ex HKLIN $inputmtz SEQIN $seq_in << \"NOUT\n XTAL DER1\n DNAME TMP\n COLU F=$f SF=$sigf\" "
	    }

	} else {
	    set nresidues $XMLParse(crank__parameters__residues)

	    set nnucleotides $XMLParse(crank__parameters__nucleotides)

	    if { [string compare $XMLParse([join "crank parameters input_intensity" __]) "1"] == 0 } {
		# Convert the I and SIGI columns into a crank.hkl file
		set i $XMLParse([join "crank soap run step=$step input i_columns i" __])
		set sigi $XMLParse([join "crank soap run step=$step input i_columns sigi" __])
		set command "$gcx_ex HKLIN $inputmtz << \"NRES $nresidues\n NNUC $nnucleotides\n NOUT\n XTAL DER1\n DNAME TMP\n COLU I=$i SI=$sigi\" "

	    } else {
		# Convert the F and SIGF columns into a crank.hkl file
		set f $XMLParse([join "crank soap run step=$step input f_columns f" __])
		set sigf $XMLParse([join "crank soap run step=$step input f_columns sigf" __])
		set command "$gcx_ex HKLIN $inputmtz << \"NRES $nresidues\n NNUC $nnucleotides\n NOUT\n XTAL DER1\n DNAME TMP\n COLU F=$f SF=$sigf\" "
	    }

	}

        catch {eval exec $command } output	
	set data $output
	set data [split $data "\n"]
	array set listmatt {}
	array set listsolv {}
	array set listprob {}
	set nummon 1

        foreach line $data {
            if       { [regexp {    *([0-9]*)         *([-0-9.]*)             *([-0-9.]*)            *([0-1].*).*} $line junk number matthews sol prob ] } {
                if { $number != "" } {
                    set listmatt($number) $matthews
                    set listsolv($number) $sol
                    set listprob($number) $prob
                    set nummon $number
                }
            } elseif { [regexp {   *([0-9]*)         *([-0-9.]*)             *([-0-9.]*)            *([0-1].*).*} $line junk number matthews sol prob ] } {
                if { $number != "" } {
                    set listmatt($number) $matthews
                    set listsolv($number) $sol
                    set listprob($number) $prob
                    set nummon $number
                }
            }
        }

#	foreach line $data {
#	    regsub -all {\s+} $line " " line 
#	    if       { [regexp { *([0-9]*) *([-0-9.]*) *([-0-9.]*) *([0-1].*).*} $line junk number matthews sol prob ] } {
#		if { $number != "" } {
#		    set listmatt($number) $matthews
#		    set listsolv($number) $sol
#		    set listprob($number) $prob
#		    set nummon $number
#		}
#	    } 
#	}

	set minprob 0.01
	if { $nummon > 1 } {
	    foreach {key solv} [array get listsolv] {
		if { ($listprob($key) > $minprob) && ($solv > 0.1) && ($solv < 0.9) } {
		    file mkdir shelxe-$solv
		    cd shelxe-$solv
		    set com [open "shelxe-command.com" w+]
		    puts $com "/bin/cp -f [file join .. crank_fa.hkl] crank_fa.hkl"
		    puts $com "/bin/cp -f [file join .. crank.hkl] crank.hkl"
		    puts $com "/bin/cp -f [file join .. crank_fa.res] crank_fa.res"
		    puts $com "/bin/cp -f [file join .. crank_fa.ins] crank_fa.ins"
		    puts $com "shelxe crank crank_fa -s$solv $heavy -m$optcycles $generateheavy $invert $amplitude >> shelxe-logfile" 
		    puts $com "/bin/rm -f crank_fa.hkl crank.hkl crank_fa.res crank_fa.ins"
		    close $com
		    #runcommand [concat chmod ugo+x ./shelxe-command.com]
		    #runcommand [concat ./shelxe-command.com]		
		    set tcl [open "shelxe-command.tcl" w+]
		    puts $tcl "\#!usr/bin/env tclsh\n"
		    puts $tcl "file copy [file join .. crank_fa.hkl] crank_fa.hkl "
		    puts $tcl "file copy [file join .. crank.hkl]    crank.hkl    "
		    puts $tcl "file copy [file join .. crank_fa.res] crank_fa.res "
		    puts $tcl "file copy [file join .. crank_fa.ins] crank_fa.ins "
		    puts $tcl "set command \"shelxe crank crank_fa -s$solv $heavy -m$optcycles $generateheavy $invert $amplitude >> shelxe-logfile\""
		    puts $tcl "eval exec \$command"
		    puts $tcl "file delete crank_fa.hkl crank.hkl crank_fa.res crank_fa.ins"
		    close $tcl	
		    runcommand [concat tclsh shelxe-command.tcl]

		    cd ..
		}
	    }

	    set allsolvent [glob shelxe-0.*]
	
	    array set contrast {}
	    array set content {}
	    set run 0
	    set num1 0
            set optlog [open "shelxeoptimize-logfile" w+]
	    foreach solvent $allsolvent { 
		set number 0    
		set logfile [open "[file join $solvent shelxe-logfile]" r]
		set content($run) [string trimleft $solvent "shelxe-"]
	    
		set data [read $logfile]
		set data [split $data "\n"]
		foreach line $data {
		    if { [regexp {<wt> = *([0-9.]*).*, Contrast = *([-0-9.]*).*, Connect. = *([-0-9.]*).* for dens.mod. cycle *([0-9])*} $line junk wt cont connect n] } {
			set contrast($run,$number) $cont
			incr number
		    }
		} 
		close $logfile
		set num1 [expr $handcycles - 1]
		puts $optlog "Solvent content $content($run): Contrast from the final cycle is $contrast($run,$num1)"
		incr run
	    }
	
	    set maxcontrast 0
	    set optimal 0
	    for { set j 0 } { $j < $run } { incr j } {
		 if { $contrast($j,$num1) > $maxcontrast } {
		     set maxcontrast $contrast($j,$num1) 
		     set optimal $j 
		 }
	    }

	    set monomers $XMLParse(crank__parameters__nmonomers)
	    puts $optlog "\nChoosing solvent content $content($optimal)\n"
	    foreach {key solv} [array get listsolv] {
		if {$listsolv($key) == $content($optimal) } {
		    set monomers $key
		    puts $optlog "Corresponding to $key monomers"
		    break
		}
	    }
	    close $optlog

	    if { $monomers != $XMLParse(crank__parameters__nmonomers) } {
		set input [open "../xml/input.xml" a+]
		puts $input "<crank><update>"
		puts $input "<nmonomers>$monomers</nmonomers>"
		puts $input "<solvent_content>$content($optimal)</solvent_content>"
		puts $input "</update></crank>"
		close $input
	    }
	}
	if { [file exists shelxeoptimize-logfile] } {
	   runcommand [concat cat shelxeoptimize-logfile] 
	}
    }

    if { $XMLParse([join "crank soap run step=$step shelxe density_modify" __]) == "1" } {

	set solv $XMLParse(crank__parameters__solvent_content)

	if { [file exists shelxeoptimize-logfile] } {
	    set logfile1 [open "shelxeoptimize-logfile" r]
	    set data [read $logfile1]
	    set data [split $data "\n"]
	    foreach line $data {
		if { [regexp {Choosing solvent content* *([0-9.]*).*} $line junk solvent ] } {
			    set solv $solvent
			    break
			}
		    }
		    close $logfile1
	} 

	if { [info exists XMLParse([join "crank soap run step=$step shelxe free_lunch" __])] } {
	    if { $XMLParse([join "crank soap run step=$step shelxe free_lunch" __]) == "1" } {
		set reso $XMLParse([join "crank soap run step=$step shelxe free_lunch_reso" __])
		set freelunch "-e$reso"
	    } 
	}
	# Number of cycles of density modification to run
	if { [info exists XMLParse([join "crank soap run step=$step shelxe ncycles" __])] } {
	    set cycles $XMLParse([join "crank soap run step=$step shelxe ncycles" __])
	} else {
	    set cycles 100
	}

	set com [open "shelxe-command.com" w+]
	puts $com "shelxe crank crank_fa -s$solv $heavy -m$cycles $generateheavy $invert $freelunch $amplitude >> shelxe-logfile" 
	close $com
	# runcommand [concat chmod ugo+x ./shelxe-command.com]
	# runcommand [concat ./shelxe-command.com]
	set tcl [open "shelxe-command.tcl" w+]
	puts $tcl "\#!usr/bin/env tclsh\n"
	puts $tcl "set command \"shelxe crank crank_fa -s$solv $heavy -m$cycles $generateheavy $invert $freelunch $amplitude\""
	puts $tcl "set logfile \[open \"shelxe-logfile\" w+\]"
	puts $tcl "catch \{eval exec \$command\} output"
	puts $tcl "puts \$logfile \$output"
	puts $tcl "close \$logfile"
	close $tcl	
	runcommand [concat tclsh shelxe-command.tcl]
	runcommand [concat cat shelxe-logfile]

	# Convert the SHELXE output to XML
	# Convert and rename output columns

	if { $invert == "" } {
	    set phs_file_name "crank.phs"
	    #	set pha_file_name "./crank.pha"
	    set hat_file_name "crank.hat"
	} else {
	    set phs_file_name "crank_i.phs"
	    #	set pha_file_name "./crank_i.pha"
	    set hat_file_name "crank_i.hat"
	}

	if { ![file exists $phs_file_name] } {
	    crank_error "crank_shelxe.tcl::file ($phs_file_name) does not exist."
	}

	#    if { ![file exists $pha_file_name] } {
	#	crank_error "crank_shelxe.tcl::file ($pha_file_name) does not exist."
	#    }

	runcommand [concat [file join $crankplugins shelxe crank-rename-shelxe-output.tcl] $inputxml $crankbin $phs_file_name rename.out.mtz $invert]
	file copy rename.out.mtz [file join .. workdb crank.out.$tag.mtz]
	file delete rename.out.mtz
	# Copy logfile to main log directory
	file copy shelxe-logfile [file join .. log $step-shelxe-logfile]
    } elseif { $XMLParse([join "crank soap run step=$step shelxe determine_hand" __]) == "1" } {
	if { $invert == "" } {
	    set hat_file_name "[file join hand1 crank.hat]"
	} else {
	    set hat_file_name "[file join hand2 crank_i.hat]"
	}
    }


    if { ![file exists $hat_file_name] } {
	file copy $substructurepdb [file join .. workdb crank.out.$tag.substructure.pdb]
    } else {
	# Revised coordinates
	runcommand [concat [file join $crankplugins shelxe shelxhat2pdb.tcl] $inputxml $crankbin $hat_file_name  > [file join .. workdb crank.out.$tag.substructure.pdb]]
    }
	
    cd ..
}

proc shelxe_reference { } {

	puts "SHELXE:"
	puts "Sheldrick, G.M. (2002) Macromolecular phasing with "
	puts "SHELXE. Z. Kristallogr. 217, 644-650.\n"

}
