#!/usr/bin/env tclsh
#
# Copyright (C) Navraj S. Pannu 2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc pdb2shelxres { pdbfile insfile } {
    global XMLParse

    set pdb [open $pdbfile r]
    set ins [open $insfile r]
    set done 0
    while { [gets $ins line] >= 0 && !$done } {
	if { [regexp {UNIT} $line full path] } {
	    set done 1
	}
	puts $line
    }
    close $ins

    set fracx 0.0000
    set fracy 0.0000
    set fracz 0.0000

    set i 1
    foreach line [split [read $pdb] "\n"] {
	if { [regexp {HETATM.*} $line junk] || [regexp {ATOM.*} $line junk] } {
	    scan $line "%6s%5d%*c%*c%4s%*c%3s%*11s%8f%8f%8f%6f" label number atom residue x y z occ 
	    orth2frac $x $y $z fracx fracy fracz
	    puts [format "%1s%03i%4i%10.6f%10.6f%10.6f%9.4f%5.1f" "P" $i 1 $fracx $fracy $fracz $occ 0.2]
 	    incr i
	} 
    }
    puts "HKLF 3"
    puts "END "
}


set crankbin [lindex $argv 0]
set inputxml [lindex $argv 1]
set pdbfile  [lindex $argv 2]
set insfile  [lindex $argv 3]

source [file join $crankbin crankutils.tcl]

XMLParsefile $inputxml

pdb2shelxres $pdbfile $insfile
