#!/usr/bin/env tclsh

# Copyright (C) Navraj S. Pannu 2006, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

proc DetermineHand { step tag } {
    global XMLParse
 
    set logfile1 [open "hand1/shelxe-logfile" r]

    if { [info exists XMLParse([join "crank soap run step=$step shelxe hand ncycles" __])] } {
	set shelxecycle $XMLParse([join "crank soap run step=$step shelxe hand ncycles" __])
    } else {
	set shelxecycle 5
    }

    puts "Looking at the shelxe logfile for the first hand\n"

    set data [read $logfile1]
    set data [split $data "\n"]

    set 1contrast 0

# <wt> = 0.300, Contrast = 0.049, Connect. = 0.582 for dens.mod. cycle 1
# <wt> = 0.300, Contrast = 0.073, Connect. = 0.660 for dens.mod. cycle 2
# <wt> = 0.300, Contrast = 0.115, Connect. = 0.724 for dens.mod. cycle 3
# <wt> = 0.300, Contrast = 0.138, Connect. = 0.752 for dens.mod. cycle 4
# <wt> = 0.300, Contrast = 0.159, Connect. = 0.769 for dens.mod. cycle 5

    foreach line $data {
	if { [regexp {<wt> = *([0-9.]*).*, Contrast = *([-0-9.]*).*, Connect. = *([-0-9.]*).* for dens.mod. cycle *([0-9])*} $line junk wt contrast connect number] } {
	    if { $number == $shelxecycle } {
		puts "Contrast in the last cycle is  $contrast\n"
		set 1contrast $contrast
	    }
	}
    }

    close $logfile1

    puts "Looking at the shelxe logfile for the second hand\n"

    set logfile2 [open "hand2/shelxe-logfile" r]

    set data [read $logfile2]
    set data [split $data "\n"]

    set 2contrast 0

    foreach line $data {
	if { [regexp {<wt> = *([0-9.]*).*, Contrast = *([-0-9.]*).*, Connect. = *([-0-9.]*).* for dens.mod. cycle *([0-9])*} $line junk wt contrast connect number] } {
	    if { $number == $shelxecycle } {
		puts "Contrast in the last cycle is  $contrast\n"
		set 2contrast $contrast
	    }
	}
    }

    close $logfile2

    if { $1contrast > $2contrast } {
	puts "First hand selected\n"
    } elseif { $2contrast > $1contrast } {
	puts "Second hand selected\n"
    }
}

set inputxml [lindex $argv 0]
set crankbin [lindex $argv 1]

if { [file exists $inputxml] == 0 } {
    crank_error "shelxe_hand.tcl::inputxml file does not exist"
}

source [file join $crankbin crankutils.tcl]

XMLParsefile $inputxml

set step $XMLParse(crank__soap__current_step)
set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

DetermineHand $step $tag