#!/usr/bin/env tclsh

# Copyright (C) Navraj S. Pannu 2007, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

global XMLParse

set inputxml [lindex $argv 0]
set crankbin [lindex $argv 1]
set hat [open [lindex $argv 2] r]

source [file join $crankbin crankutils.tcl]

XMLParsefile $inputxml

while { [gets $hat line] >= 0} {
    if { [regexp {SFAC (.*) *}  $line full element] } {
	break
    }
    if { [regexp {UNIT.*}  $line full] } {
	break
    }
}

set atomnum 1

set i 1
set orthx 0.0000
set orthy 0.0000
set orthz 0.0000
set residue "HAT"


while { [info exists XMLParse([join "crank parameters crystal=$i native" __])] } {
    if { $XMLParse([join "crank parameters crystal=$i native" __]) == "0"} {
	if { [info exists XMLParse([join "crank parameters crystal=$i substructure atom_name" __])] } {
	    set inputatom [string toupper $XMLParse([join "crank parameters crystal=$i substructure atom_name" __])]  
	} else {
	    crank_error "crankutils.tcl::no substructure atom"
	}
	if { [info exists XMLParse([join "crank parameters crystal=$i data=1 bfactor" __])] } {
	    set b $XMLParse([join "crank parameters crystal=$i data=1 bfactor" __])  
	} else {
	    set b 25.0
	}
    }
    incr i
}

set i 1

while { [gets $hat line] >= 0} {
    regsub -all {\s+} $line " " line
    regexp {([A-Z0-9a-z]*) *([0-9]*) *([-0-9.]*) *([-0-9.]*) *([-0-9.]*) *([-0-9.]*) *} \
	$line full atomname num x y z occ;
    if { ($x != "") && ($occ > 0) } {
	frac2orth $x $y $z orthx orthy orthz
	puts [format "HETATM %4d %2s   %3s   %3d    %8.3f%8.3f%8.3f%6.2f%6.2f" $i $inputatom $inputatom $i $orthx $orthy $orthz $occ $b]
	incr i
    }
}

puts "END"
