#
# Copyright (C) Navraj S. Pannu and Jan Pieter Abrahams 2006,  Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc solomon_run { code crankbin executable crankplugins inputxml inccp4 step } {
    global XMLParse
    
    crank_run_message "Running $step-solomon"
    
    file mkdir $step-solomon
    cd $step-solomon
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]
    set enant 1

    set nomtz 1
    if { $XMLParse([join "crank soap run step=$step solomon keep_mtz" __]) == "1" } {
       set nomtz 0
    }

    set nomap 1
    if { $XMLParse([join "crank soap run step=$step solomon keep_map" __]) == "1" } {
       set nomap 0
    }

    if { $XMLParse([join "crank soap run step=$step solomon determine_hand" __]) == "1" } {
	
  	runcommand [concat [file join $crankplugins solomon solomonhand.tcl] $inputxml $crankbin $crankplugins $nomtz $nomap |& tee solomonhand-logfile]	

	set logfile1 [open "solomonhand-logfile" r]
	set data [read $logfile1]
	set data [split $data "\n"]
 	
	if { [info exists XMLParse([join "crank soap run step=$step solomon hand_cycles" __])] } {
	    set ncycles $XMLParse([join "crank soap run step=$step solomon hand_cycles" __])
	} else {
	    set ncycles 2
	}

	foreach line $data {
	    if { $line       == "First hand selected" } {
		set enant 1
		set solomonmtz "[file join hand1 solomon_cycle$ncycles.mtz]"
		break
	    } elseif { $line == "Second hand selected" } {
		set enant 2
		set solomonmtz "[file join hand2 solomon_cycle$ncycles.mtz]"
		break
	    }
	}
	close $logfile1
	file copy solomonhand-logfile [file join .. log $step-solomonhand-logfile]
    } else {
	set inputtag [string trim $XMLParse([join "crank soap run step=$step input substructure" __])]
	file copy [file join .. workdb crank.out.$inputtag.substructure.pdb] [file join .. workdb crank.out.$tag.substructure.pdb]
    }

    if { $XMLParse([join "crank soap run step=$step solomon optimize_solvent" __]) == "1" } {
	runcommand [concat [file join $crankplugins solomon solomonoptimize.tcl] $inputxml $crankbin $crankplugins $inccp4 $enant $nomtz $nomap |& tee solomonoptimize-logfile]

	if { [file exists solomonoptimize-logfile] } {
	    if { [info exists XMLParse([join "crank soap run step=$step solomon opt_cycles" __])] } {
		set ncycles $XMLParse([join "crank soap run step=$step solomon opt_cycles" __])                                     
	    } else {
		set ncycles 2
	    }

	    set logfile1 [open "solomonoptimize-logfile" r]
	    set data [read $logfile1]
	    set data [split $data "\n"]
	    foreach line $data {
		if { [regexp {Choosing solvent content* *([0-9.]*).*} $line junk solvent ] } {
		    set solomonmtz [file join solomon-$solvent solomon_cycle$ncycles.mtz]
		    break
		}
	    }
	    close $logfile1
	} 

	file copy solomonoptimize-logfile [file join .. log $step-solomonoptimize-logfile]
    }

    if { $XMLParse([join "crank soap run step=$step solomon density_modify" __]) == "1" } {

	set phasingphases 0

	if { [info exists XMLParse([join "crank soap run step=$step solomon ncycles " __])] } {
	    set ncycles $XMLParse([join "crank soap run step=$step solomon ncycles" __])
	} else {
	    set ncycles 10
	}    

	set solv $XMLParse(crank__parameters__solvent_content)

	if { [file exists solomonoptimize-logfile] } {
	    set logfile1 [open "solomonoptimize-logfile" r]
	    set data [read $logfile1]
	    set data [split $data "\n"]
	    foreach line $data {
		if { [regexp {Choosing solvent content* *([0-9.]*).*} $line junk solvent ] } {
		    set solv $solvent
		    break
		}
	    }
	    close $logfile1
	} 

	runcommand [concat [file join $crankplugins solomon xml2solomoncom.tcl] $inputxml $crankbin $ncycles $solv $enant $nomtz $nomap 1 >& solomon-command.com]

	#runcommand [concat chmod ugo+x ./solomon-command.com]
	#runcommand [concat ./solomon-command.com]

	runcommand [concat [file join $crankplugins solomon xml2solomontcl.tcl] $inputxml $crankbin $ncycles $solv $enant $nomtz $nomap 1 >& solomon-command.tcl]
	runcommand [concat tclsh solomon-command.tcl]

	set solomonmtz "solomon_cycle$ncycles.mtz"

        runcommand [concat cat solomon-logfile]
    } else {
	set phasingphases 1
	set phasingcoefficients 1
    }
	

    if { [info exists XMLParse([join "crank soap run step=$step solomon hlphasing_coefficients " __])] } {
	set phasingcoefficients $XMLParse([join "crank soap run step=$step solomon hlphasing_coefficients" __])
    } else {
	set phasingcoefficients 1
    }

    # Rename output columns to the names specified in the XML
    if { [file exists $solomonmtz] } {
	set sftools_script "read $solomonmtz\n"

	if { $phasingcoefficients == 1 } {
	    set temptag [string trim $XMLParse([join "crank soap run step=$step input phase_columns f" __])]
	    set phasingtag [string trim $temptag _F]
	    if { $phasingphases == 1 } {
		set phib _PHIB
		set fom _FOM
 		set sftools_script "$sftools_script delete col PHISOL FOMSOL\n"
		set sftools_script "$sftools_script set label col [format "%s%s" $phasingtag $phib] \n PHISOL  \n"
		set sftools_script "$sftools_script set label col [format "%s%s" $phasingtag $fom] \n FOMSOL  \n"
	    }
	    set hla _HLA
	    set hlb _HLB
	    set hlc _HLC
	    set hld _HLD
	    set sftools_script "$sftools_script set label col [format "%s%s" $phasingtag $hla] \n HLA  \n"
	    set sftools_script "$sftools_script set label col [format "%s%s" $phasingtag $hlb] \n HLB  \n"
	    set sftools_script "$sftools_script set label col [format "%s%s" $phasingtag $hlc] \n HLC  \n"
	    set sftools_script "$sftools_script set label col [format "%s%s" $phasingtag $hld] \n HLD  \n"
	    set sftools_script "$sftools_script write temp.mtz\nquit\n"
	} else {
	    set sftools_script "$sftools_script hlconv col PHISOL FOMSOL\n"
	    set sftools_script "$sftools_script set label col PA\nHLA\n"
	    set sftools_script "$sftools_script set label col PB\nHLB\n"
	    set sftools_script "$sftools_script calc A col HLC = 0\n"
	    set sftools_script "$sftools_script calc A col HLD = 0\n"
	    set sftools_script "$sftools_script write temp.mtz\nquit\n"
	}
	set command "sftools << \"$sftools_script\""

#        puts $command
	catch {eval exec $command } output
#        puts $output	
	runcommand [concat [file join $crankplugins solomon crank-rename-solomon-output.tcl] $inputxml $crankbin temp.mtz rename.out.mtz]
	file copy rename.out.mtz [file join .. workdb crank.out.$tag.mtz]
	file delete temp.mtz
	file delete rename.out.mtz
    } else {
	crank_error "crank::runsolomon:: $solomonmtz not present.  Exitting."
    }

    cd ..
}

proc solomon_reference { } {
    global XMLParse
    
    puts "Solomon:"
    puts "Abrahams, J.P. and Leslie, A.G.W. (1996) Methods used in the"
    puts "structure determination of bovine mitochondrial F1 ATPase."
    puts "Acta Cryst. D52, 30-42.\n"
} 
