#!/usr/bin/env tclsh
#
# Copyright (C) Navraj S. Pannu 2006, Leiden University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

proc DetermineHand { step tag inputxml crankbin crankplugins nomtz nomap } {
    global XMLParse

    # run a few cycles on the first hand
    file mkdir hand1
    cd hand1
    set enant 1

    if { [info exists XMLParse([join "crank soap run step=$step solomon hand_cycles" __])] } {
	set ncycles $XMLParse([join "crank soap run step=$step solomon hand_cycles" __])
    } else {
	set ncycles 2
    }    

    set solv $XMLParse([join "crank parameters solvent_content" __])
    runcommand [concat [file join $crankplugins solomon xml2solomoncom.tcl] $inputxml $crankbin $ncycles $solv $enant $nomtz $nomap 0 >& solomon-command.com]
    runcommand [concat [file join $crankplugins solomon xml2solomontcl.tcl] $inputxml $crankbin $ncycles $solv $enant $nomtz $nomap 0 >& solomon-command.tcl]

    #    runcommand [concat chmod ugo+x ./solomon-command.com]
    #    runcommand [concat ./solomon-command.com]

    runcommand [concat tclsh solomon-command.tcl]

    cd ..

    # run a few cycles on the second hand
	
    file mkdir hand2
    cd hand2
    set enant 2
    set temptag [string trim $XMLParse([join "crank soap run step=$step input exp_columns crystal=1 data=1 f" __])]
    set ftag [string trimright $temptag "_X1_D1_F" ]
    set temptag [string trim $XMLParse([join "crank soap run step=$step input phase_columns f" __])]
    set bp3tag [string trim $temptag _F]
    set sftools_script "read [file join .. .. workdb crank.out.$bp3tag.oh.mtz]\n"
    set sftools_script "$sftools_script read [file join .. .. workdb crank.out.$ftag.mtz]\n"
    set sftools_script "$sftools_script write [file join .. .. workdb crank.in.$tag.oh.mtz]\nquit\n"
    set command "sftools << \"$sftools_script\""
#   puts $command
    catch {eval exec $command } output
#   puts $output
    runcommand [concat [file join $crankplugins solomon xml2solomoncom.tcl] $inputxml $crankbin $ncycles $solv $enant $nomtz $nomap 0 >& solomon-command.com]
    runcommand [concat [file join $crankplugins solomon xml2solomontcl.tcl] $inputxml $crankbin $ncycles $solv $enant $nomtz $nomap 0 >& solomon-command.tcl]
#    runcommand [concat chmod ugo+x ./solomon-command.com]
#    runcommand [concat ./solomon-command.com]
    runcommand [concat tclsh solomon-command.tcl]
    cd ..

    # determine hand
    set tol $XMLParse([join "crank soap run step=$step solomon margin" __])
    # Select the correct enantiomorph from dm log file

    
    set logfile1 [open "[file join hand1 solomon-logfile]" r]

    puts "Looking at the solomon logfile for the first hand\n"

    set data [read $logfile1]
    set data [split $data "\n"]

    set number 0
    array set contrast1 {}
    array set cor1 {} 
    array set fom1 {}

    foreach line $data {
  	if { [regexp       {Inverse contrast is* *([0-9.]*).*} $line junk con] } {
 	    if { $con < 0 } {
		puts "Error contrast is negative\n"
	    } else { 
		set contrast1($number) [expr 1.0/$con ] 
	    }
	}
	if { [regexp       { Overall MEAN FOM is* *([0-9.]*).*} $line junk fomcycle] } {
	    set fom1($number) $fomcycle
	}
	if { [regexp       { Overall MEAN W is* *([0-9.]*).*} $line junk fomcycle] } {
	    set fom1($number) $fomcycle
	}
	if { [regexp       {.*([0-9]*) Reflections is *([0-9.]*).*} $line junk refs correl] } {
	    set cor1($number) $correl 
	    incr number
	}
    }

    set num1 [expr $number - 1]

    puts [format "Mean FOM: %.3f  Correlation coefficient on |E|^2 %.3f" $fom1($num1) $cor1($num1) ]
    puts [format "Contrast: %.3f Correlation*Contrast: %.3f " $contrast1($num1) [expr $cor1($num1) * $contrast1($num1)] ]
    puts "\n"

    close $logfile1

    set logfile2 [open "[file join hand2 solomon-logfile]" r]

    puts "Looking at the solomon logfile for the second hand\n"

    set data [read $logfile2]
    set data [split $data "\n"]

    set number 0
    array set contrast2 {}
    array set cor2 {} 
    array set fom2 {}

    foreach line $data {
  	if { [regexp       {Inverse contrast is* *([0-9.]*).*} $line junk con] } {
 	    if { $con < 0 } {
		puts "Error contrast is negative\n"
	    } else {
		set contrast2($number) [expr 1.0/$con ] 
	    }
	}
	if { [regexp       { Overall MEAN FOM is* *([0-9.]*).*} $line junk fomcycle] } {
	    set fom2($number) $fomcycle
	}
	if { [regexp       { Overall MEAN W is* *([0-9.]*).*} $line junk fomcycle] } {
	    set fom2($number) $fomcycle
	}
	if { [regexp       {.*([0-9]*) Reflections is *([0-9.]*).*} $line junk refs correl] } {
	    set cor2($number) $correl 
	    incr number
	}
    }

    set num1 [expr $number - 1]

    puts [format "Mean FOM: %.3f  Correlation coefficient on |E|^2 %.3f" $fom2($num1) $cor2($num1) ]
    puts [format "Contrast: %.3f Correlation*Contrast: %.3f " $contrast2($num1) [expr $cor2($num1) * $contrast2($num1)] ]
    puts "\n"

    close $logfile2

    set enantin 0
   
    if {  [expr $contrast1($num1)*$cor1($num1) - $contrast2($num1)*$cor2($num1) ] > $tol  } {
	set enantin 1
    } elseif { [expr $contrast2($num1)*$cor2($num1) - $contrast1($num1)*$cor1($num1) ] > $tol  } {
	set enantin 2
    }

    if { ($enantin == 0) } {
 	if { [expr $contrast1(0)*$cor1(0) - $contrast2(0)*$cor2(0) ] > $tol  } {
	    set enantin 1
	} elseif { [expr $contrast2(0)*$cor2(0) - $contrast1(0)*$cor1(0) ] > $tol } {
	    set enantin 2
	}
    }

    set temptag [string trim $XMLParse([join "crank soap run step=$step input phase_columns f" __])]
    set bp3tag [string trim $temptag _F]

    if {       ($enantin == 1) } {
	puts "First hand selected\n"
	file copy [file join .. workdb crank.out.$bp3tag.substructure.pdb] [file join .. workdb crank.out.$tag.substructure.pdb]
    } elseif { ($enantin == 2) } {
	puts "Second hand selected\n"
	file copy [file join .. workdb crank.out.$bp3tag.oh.substructure.pdb] [file join .. workdb crank.out.$tag.substructure.pdb]
    } else {
 	crank_error "solomon_hand.tcl::Correct hand could not be determined\n"
    }
}

set inputxml     [lindex $argv 0]
set crankbin     [lindex $argv 1]
set crankplugins [lindex $argv 2]
set nomtz        [lindex $argv 3]
set nomap        [lindex $argv 4]

if { [file exists $inputxml] == 0 } {
    crank_error "solomonhand.tcl::inputxml file does not exist"
}

source [file join $crankbin crankutils.tcl]

XMLParsefile $inputxml

set step $XMLParse(crank__soap__current_step)
set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

DetermineHand $step $tag $inputxml $crankbin $crankplugins $nomtz $nomap
