#!/usr/bin/env tclsh

# Copyright (C) 2006 Leiden University
#
# Author: Navraj S. Pannu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

proc DetermineSolvent { step tag inputxml crankbin crankplugins inccp4 enant nomtz nomap } {
    global XMLParse
    global env

    if { $inccp4 }  {
	set gcx_ex [file join $env(CBIN) gcx]
    } else {
	set gcx_ex [file join $crankbin  gcx]
    }

    set orig_pwd $XMLParse(crank__soap__orig_pwd)

    if { $enant == "1" } {
	set mtz_in "[file join $orig_pwd workdb crank.in.$tag.mtz]"
    } else {
	set mtz_in "[file join $orig_pwd workdb crank.in.$tag.oh.mtz]"
    }

    set fp $XMLParse([join "crank soap run step=$step input exp_columns crystal=1 data=1 f" __])
    set sigfp $XMLParse([join "crank soap run step=$step input exp_columns crystal=1 data=1 sigf" __])

    set initmonomers $XMLParse([join "crank parameters nmonomers" __])

    if { [info exists XMLParse([join "crank parameters seqin filename" __]) ] } {
	set seq_in $XMLParse([join "crank parameters seqin filename" __])
	set command "$gcx_ex HKLIN $mtz_in SEQIN $seq_in << \"NOUT\n XTAL DER1\n DNAME TMP\n COLU F=$fp SF=$sigfp\" "
    } else {
	
	if { [info exists XMLParse([join "crank parameters residues" __]) ] } {
	    set nresidues $XMLParse(crank__parameters__residues)
	} else {
	    set nresidues 0
	}

 	if { [info exists XMLParse([join "crank parameters nucleotides" __]) ] } {
	    set nnucleotides $XMLParse(crank__parameters__nucleotides)
	} else {
 	    set nnucleotides 0
	}

	set command "$gcx_ex HKLIN $mtz_in << \"NRES $nresidues\n NNUC $nnucleotides\n NOUT\n XTAL DER1\n DNAME TMP\n COLU F=$fp SF=$sigfp\" "
    }
    
    catch {eval exec $command } output

    set data $output
    set data [split $data "\n"]
    array set listmatt {}
    array set listsolv {}
    array set listprob {}
    set num 0

    foreach line $data {
	if {       [regexp {    *([0-9]*)         *([-0-9.]*)             *([-0-9.]*)            *([0-1].*).*} $line junk number matthews sol prob ] } {
	    if { $number != "" } {
		set listmatt($number) $matthews
		set listsolv($number) $sol
		set listprob($number) $prob
		set num $number
	    }
	} elseif { [regexp {   *([0-9]*)         *([-0-9.]*)             *([-0-9.]*)            *([0-1].*).*} $line junk number matthews sol prob ] } {
	    if { $number != "" } {
		set listmatt($number) $matthews
		set listsolv($number) $sol
		set listprob($number) $prob
		set num $number
	    }
	}
    }

    if { [info exists XMLParse([join "crank soap run step=$step solomon opt_cycles" __])] } {
	set ncycles $XMLParse([join "crank soap run step=$step solomon opt_cycles" __])
    } else {
	set ncycles 2
    }

    set minprob 0.01

    if { $num > 1 } {
	foreach {key solv} [array get listsolv] {
	    if { ( ($listprob($key) > $minprob) && ($solv > 0.1) && ($solv < 0.9) ) } {
		file mkdir solomon-$solv
		cd solomon-$solv
		runcommand [concat [file join $crankplugins solomon xml2solomoncom.tcl] $inputxml $crankbin $ncycles $solv $enant $nomtz $nomap 1 >& solomon-command.com]
		runcommand [concat [file join $crankplugins solomon xml2solomontcl.tcl] $inputxml $crankbin $ncycles $solv $enant $nomtz $nomap 1 >& solomon-command.tcl]
#		runcommand [concat chmod ugo+x ./solomon-command.com]
#		runcommand [concat ./solomon-command.com]
		runcommand [concat tclsh solomon-command.tcl]
		cd ..
	    }
	}
    
	set allsolvent [glob solomon-0.*]

	array set contrast {}
	array set cor {} 
	array set fom {}
	array set content {}
	set run 0
	set number 0    
	set num1 0

	foreach solvent $allsolvent { 
	    set logfile [open "[file join $solvent solomon-logfile]" r]
	    set content($run) [string trimleft $solvent "solomon-"] 
	    puts "Statistics from the final cycle with solvent content set to $content($run)"

	    set data [read $logfile]
	    set data [split $data "\n"]

	    set number 0
	    foreach line $data {
		if { [regexp       {Inverse contrast is* *([0-9.]*).*} $line junk con] } {
		    set contrast($run,$number) [expr 1.0/$con ] 
		}
		if { [regexp       { Overall MEAN FOM is* *([0-9.]*).*} $line junk fomcycle] } {
		    set fom($run,$number) $fomcycle
		}
		if { [regexp       { Overall MEAN W is* *([0-9.]*).*} $line junk fomcycle] } {
		    set fom($run,$number) $fomcycle
		}
		if { [regexp       {.*([0-9]*) Reflections is *([0-9.]*).*} $line junk refs correl] } {
		    set cor($run,$number) $correl 
		    incr number
		}
	    }

	    close $logfile
	    set num1 [expr $number -1]

	    puts [format "Mean FOM: %.3f  Correlation coefficient on |E|^2 %.3f" $fom($run,$num1) $cor($run,$num1) ]
	    puts [format "Contrast: %.3f Correlation*Contrast: %.3f " $contrast($run,$num1) [expr $cor($run,$num1) * $contrast($run,$num1)] ]
	    puts "\n"
	    incr run
	}

	# Find the solvent content with the greatest correlation * contrast
	set maxcorrel 0
	set optimal 0
	for { set j 0 } { $j < $run } { incr j } {
	    if { [expr $contrast($j,$num1) * $cor($j,$num1) ] > $maxcorrel } {
		set maxcorrel [expr $contrast($j,$num1) * $cor($j,$num1) ]
		set optimal $j 
	    }
	}

	set monomers $XMLParse(crank__parameters__nmonomers)
	puts "Choosing solvent content $content($optimal)"

	foreach {key solv} [array get listsolv] {
	    if {$listsolv($key) == $content($optimal) } {
		set monomers $key
		puts  "Corresponding to $key monomers"
		break
	    }
	}
    
	if { $monomers != $XMLParse(crank__parameters__nmonomers) } {
	    set input [open "[file join .. xml input.xml]" a+]
	    puts $input "<crank><update>"
	    puts $input "<nmonomers>$monomers</nmonomers>"
	    puts $input "<solvent_content>$content($optimal)</solvent_content>"
	    puts $input "</update></crank>"
	    close $input
	}

    }
}

set inputxml     [lindex $argv 0]
set crankbin     [lindex $argv 1]
set crankplugins [lindex $argv 2]
set version      [lindex $argv 3]
set enant        [lindex $argv 4]
set nomtz        [lindex $argv 5]
set nomap        [lindex $argv 6]


if { [file exists $inputxml] == 0 } {
    crank_error "solomonoptimize.tcl::inputxml file does not exist"
}

source [file join $crankbin crankutils.tcl]

XMLParsefile $inputxml

set step $XMLParse(crank__soap__current_step)
set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

DetermineSolvent $step $tag $inputxml $crankbin $crankplugins $version $enant $nomtz $nomap
