#!/usr/bin/env tclsh
#
# Copyright (C) Navraj S. Pannu and Jan Pieter Abrahams 2006, Leiden University
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc OutputSolomoncomfile { ncycles solv enant nomtz nomap flattened } {
    global XMLParse

    puts "\#!/bin/sh"
    puts " "
    puts "/bin/rm -f solomon-logfile"
    set step $XMLParse(crank__soap__current_step)
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    puts "solvdens=0"

    set flip [expr ($solv - 1.0)/$solv]
    puts "flip=$flip"

    puts "EndCycle=$ncycles"

    puts "LastCycle=0"
    puts "CurrentCycle=\$LastCycle"
   
    if { $enant == "1" } {
      set mtz_in "[file join [pwd] workdb crank.in.$tag.mtz]"
    } else {
      set mtz_in "[file join [pwd] workdb crank.in.$tag.oh.mtz]"
    } 

    set c $XMLParse([join "crank parameters ref_crystal" __])
    set d $XMLParse([join "crank parameters ref_dataset" __])

    set in_fp $XMLParse([join "crank soap run step=$step input exp_columns crystal=$c data=$d f" __])
    set in_sigfp $XMLParse([join "crank soap run step=$step input exp_columns crystal=$c data=$d sigf" __])

    # Phase columns
    set in_phib $XMLParse([join "crank soap run step=$step input phase_columns phib" __])
    set in_fom $XMLParse([join "crank soap run step=$step input phase_columns fom" __])

    # HL columns
    if { [info exists XMLParse([join "crank soap run step=$step input hl_columns hla" __])] &&
	  [string compare $XMLParse([join "crank soap run step=$step input hl_columns hla" __]) "NONE"] != 0 } {
	set in_hla $XMLParse([join "crank soap run step=$step input hl_columns hla" __])
	set in_hlb $XMLParse([join "crank soap run step=$step input hl_columns hlb" __])
	set in_hlc $XMLParse([join "crank soap run step=$step input hl_columns hlc" __])
	set in_hld $XMLParse([join "crank soap run step=$step input hl_columns hld" __])
    }

#    puts "\#"
#    puts "\# A) Use CAD to resort the data to P1"
#    puts "cad HKLIN1 $mtz_in HKLOUT P1.mtz << end-cad > cad.log"
#    puts "OUTLIM SPACEGROUP P1"
#    puts "TITLE  extended to P1 cell"
#    puts "LABIN FILE 1 E1=$in_fp E2=$in_sigfp E3=$in_phib E4=$in_fom E5=$in_hla E6=$in_hlb E7=$in_hlc E8=$in_hld"
#    puts "CTYPE FILE 1 E1=F E2=Q E3=P E4=W E5=A E6=A E7=A E8=A"
#    puts "end"
#    puts "end-cad"

    puts "\#"
    puts "\#  Calculate initial map with best phases and FOM weighted"
    puts "\#  amplitudes."
    puts "\#"
    puts "cfft -stdin << end-fft > cfft.log"
    puts "mtzin $mtz_in"
    puts "colin-fo /*/*/\[$in_fp,$in_sigfp\]"
    puts "colin-hl /*/*/\[$in_hla,$in_hlb,$in_hlc,$in_hld\]"
    puts "mapout solomon_cycle0.map"
    puts "end-fft"

 #   puts "fft HKLIN P1.mtz MAPOUT solomon_cycle0.map << end-fft > fft.log"
 #   puts "LABIN F1=$in_fp SIG1=$in_sigfp PHI=$in_phib W=$in_fom"
 #   puts "FFTSPACEGROUP 1"
 #   puts "AXIS Z X Y"
 #   puts "end"
 #   puts "end-fft"
    puts "\#"
    puts "\#"
    puts "while test \$CurrentCycle != \$EndCycle "
    puts "do"
    puts "\#"
    puts "CurrentCycle=`expr \$LastCycle + 1`"
    puts "\#"
    if { $flattened == "1" } { 
	puts "if test \$CurrentCycle -eq \$EndCycle"
	puts "then\n   flip=0"
	puts "fi"
    }

    if { $nomtz } {
	 puts "/bin/rm -f solomon_cycle\$LastCycle.mtz"
    }

    puts "\#"
    puts "\# 1) Modify the density with Solomon"
    puts "\#"
    puts "\#"
    puts "solomon MAPIN solomon_cycle\$LastCycle.map MAPOUT junk.map << end-solomon >> solomon.log"
    puts "slvfrc $solv"
    puts "mulsolv auto \$flip"
    puts "slvdens \$solvdens"

    set radius $XMLParse([join "crank soap run step=$step solomon radius" __])

    puts "radius $radius"
    
    set trunc_low $XMLParse([join "crank soap run step=$step solomon trunc_low" __])
    set trunc_high $XMLParse([join "crank soap run step=$step solomon trunc_high" __])
    
    puts "trunc $trunc_low $trunc_high"
    puts "extrude"
    puts "mapout"
    puts "end-solomon"
    puts "\#"
    puts "solvdens=`grep \"solvdens:\" solomon.log|tail -1|awk '{print \$3}'`"
    puts "echo Cycle \$CurrentCycle >> solomon-logfile"
    puts "contrast=`grep \"ratio:\" solomon.log|tail -1|awk '{print \$2}'`"
    puts "echo \" Inverse contrast is\" \$contrast >> solomon-logfile"
    if { $nomap } {
	puts "/bin/rm -f solomon_cycle\$LastCycle.map"
    }
    puts "\#"
    puts "\# 2) generate sfs and phases from this modified map"
    puts "\#"
#    puts "sfall HKLIN P1.mtz MAPIN junk.map HKLOUT junk.mtz << end-sfall >>  sfall.log"
#    puts "TITLE SFs from density modification"
#    puts "MODE SFCALC MAPIN HKLIN"
#    puts "LABIN FP=$in_fp SIGFP=$in_sigfp F0=$in_phib F1=$in_fom - "
#    puts "F2=$in_hla F3=$in_hlb F4=$in_hlc F5=$in_hld"
#    puts "SFSG 1"
#    puts "BINS 40"
#    puts "LABO FC=FCmod PHIC=PHICmod"
#    puts "end"
#    puts "end-sfall"
    
    puts "cinvfft -stdin << end-cinvfft > cinvfft.log"
    puts "mtzin $mtz_in"
    puts "mapin junk.map"
    puts "mtzout junk.mtz"
    puts "colout mod"
    puts "end-cinvfft"
    puts "\#"
    # puts "grep \"Overall Reliability index is\" sfall.log| tail -1 >> solomon-logfile"
    puts "\#"
    puts "\# 3) Combine phases"
    puts "\#"
    puts "sigmaa HKLIN junk.mtz HKLOUT solomon_cycle\$CurrentCycle.mtz << end-sigmaa >> sigmaa.log"
    puts "LABI FP=$in_fp SIGFP=$in_sigfp HLA=$in_hla HLB=$in_hlb HLC=$in_hlc HLD=$in_hld -"
    puts "     PHIBP=$in_phib WP=$in_fom FC=mod.F_phi.F PHIC=mod.F_phi.phi"
    puts "LABO PHCMB=PHISOL WCMB=FOMSOL FWT=FWT PHFWT=PHFWT" 
# -"
#    puts "     HLAC=HLASOL HLBC=HLBSOL HLCC=HLCSOL HLDC=HLDSOL"
    puts "RANGES 10 1000"
    puts "ERROR"
    puts "COMBINE PART 1"
    puts "END"
    puts "end-sigmaa"
    puts "\#"
    puts "grep \"Overall MEAN FOM is \" sigmaa.log| tail -1  >> solomon-logfile"
    puts "grep \"Overall MEAN W is \" sigmaa.log| tail -1  >> solomon-logfile"
    puts "grep \"Overall Correlation on \" sigmaa.log| tail -1 >> solomon-logfile"
    puts "echo >> solomon-logfile"
    puts "\# 4) recalculate map."
    puts "\#"
#    puts "fft HKLIN solomon_cycle\$CurrentCycle.mtz MAPOUT solomon_cycle\$CurrentCycle.map << end-fft >> fft.log"
#    puts "TITLE after \$CurrentCycle averaging cycles"
#    puts "LABI F1=FWT  SIG1=$in_sigfp PHI=PHFWT"
#    puts "FFTSPACEGROUP 1"
#    puts "AXIS Z X Y"
#    puts "end"
#    puts "end-fft"
    puts "cfft -stdin << end-fft > cfft.log"
    puts "mtzin solomon_cycle\$CurrentCycle.mtz"
    puts "colin-fc /*/*/\[FWT,PHFWT\]"
    puts "mapout solomon_cycle\$CurrentCycle.map"
    puts "end-fft"

#    puts "\#"
#    puts "cad HKLIN1 solomon_cycle\$CurrentCycle.mtz HKLOUT temp.mtz << end-cad > cad.log"
#    puts "OUTLIM SPACEGROUP $spgr"
#    puts "TITLE  SPACEGROUP change from P1 to $spgr"
#    puts "LABIN FILE 1 E1=$in_fp E2=$in_sigfp E3=$in_phib E4=$in_fom E5=$in_hla E6=$in_hlb E7=$in_hlc E8=$in_hld -"
#    puts "E9=PHISOL E10=FOMSOL E11=FWT E12=PHFWT"
#    puts "CTYPE FILE 1 E1=F E2=Q E3=P E4=W E5=A E6=A E7=A E8=A E9=P E10=W E11=F E12=P"
#    puts "end"
#    puts "end-cad"
#    puts "\#"
#    puts "/bin/rm -f solomon_cycle\$CurrentCycle.mtz"
#    puts "mv temp.mtz solomon_cycle\$CurrentCycle.mtz"
    puts "\#"
    puts "LastCycle=`expr \$LastCycle + 1`"
    puts "done"
    puts "/bin/rm -f junk.map junk.mtz"
}

set inputxml [lindex $argv 0]
set crankbin [lindex $argv 1] 
set cycles   [lindex $argv 2]
set solv     [lindex $argv 3]
set enant    [lindex $argv 4]
set nomtz    [lindex $argv 5]
set nomap    [lindex $argv 6]
set flatten  [lindex $argv 7]

if { [file exists $inputxml] == 0 } {
    crank_error "xml2solomoncom.tcl::inputxml file does not exist"
}

source [file join $crankbin crankutils.tcl]

XMLParsefile $inputxml 

OutputSolomoncomfile $cycles $solv $enant $nomtz $nomap $flatten
