#!/usr/bin/env tclsh
#
# Copyright (C) Navraj S. Pannu and Jan Pieter Abrahams 2006, Leiden University
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

proc OutputSolomontclfile { ncycles solv enant nomtz nomap flattened } {
    global XMLParse

    set step $XMLParse(crank__soap__current_step)
    set orig_pwd $XMLParse(crank__soap__orig_pwd)
    set tag [string trim $XMLParse([join "crank soap run step=$step tag" __])]

    puts "\#!/usr/bin/env tclsh\n"

    puts "if \{ \[file exists solomon-logfile \] \} \{"
    puts "    file delete solomon-logfile"
    puts "\}\n"

    puts "set solomonlogfile \[open \"solomon-logfile\" w+\]\n"

    puts "set solvdens 0"

    set flip [expr ($solv - 1.0)/$solv]
    puts "set flip $flip"

    puts "set EndCycle $ncycles"

    puts "set LastCycle 0"
    puts "set CurrentCycle \$LastCycle\n"

     if { $enant == "1" } {
 	 if { [info exists XMLParse(crank__soap__run__standalone)] } {
	     set mtz_in "[file join $orig_pwd run workdb crank.in.$tag.mtz]"
	 } else {
	     set mtz_in "[file join $orig_pwd workdb crank.in.$tag.mtz]"
	 }
     } else {
 	 if { [info exists XMLParse(crank__soap__run__standalone)] } {
	     set mtz_in "[file join $orig_pwd run workdb crank.in.$tag.oh.mtz]"
	 } else {
	     set mtz_in "[file join $orig_pwd workdb crank.in.$tag.oh.mtz]"
	 }
     } 

    set c $XMLParse([join "crank parameters ref_crystal" __])
    set d $XMLParse([join "crank parameters ref_dataset" __])

    set in_fp $XMLParse([join "crank soap run step=$step input exp_columns crystal=$c data=$d f" __])
    set in_sigfp $XMLParse([join "crank soap run step=$step input exp_columns crystal=$c data=$d sigf" __])

    # Phase columns
    set in_phib $XMLParse([join "crank soap run step=$step input phase_columns phib" __])
    set in_fom $XMLParse([join "crank soap run step=$step input phase_columns fom" __])

    # HL columns
    if { [info exists XMLParse([join "crank soap run step=$step input hl_columns hla" __])] &&
	  [string compare $XMLParse([join "crank soap run step=$step input hl_columns hla" __]) "NONE"] != 0 } {
	set in_hla $XMLParse([join "crank soap run step=$step input hl_columns hla" __])
	set in_hlb $XMLParse([join "crank soap run step=$step input hl_columns hlb" __])
	set in_hlc $XMLParse([join "crank soap run step=$step input hl_columns hlc" __])
	set in_hld $XMLParse([join "crank soap run step=$step input hl_columns hld" __])
    }
    
    puts -nonewline "set script \"start -mtzin $mtz_in "
    puts -nonewline "-colin-fo /*/*/\\\\\\\[$in_fp,$in_sigfp\\\\\\\] "
    puts -nonewline "-colin-hl /*/*/\\\\\\\[$in_hla,$in_hlb,$in_hlc,$in_hld\\\\\\\] "
    puts            "-mapout solomon_cycle0.map\""

    puts "set logfile \[open \"cfft.log\" a+\]"
    puts "set command \"cfft <<\$script\""
    puts "catch \{eval exec \$command\} output"
    puts "puts \$logfile \$output"
    puts "close \$logfile\n\n"
    puts "while \{ \$CurrentCycle != \$EndCycle \} \{ \n"
    puts "set CurrentCycle \[expr \$LastCycle + 1\]"
    if { $flattened == "1" } { 
	puts "if \{ \$CurrentCycle == \$EndCycle \} \{"
	puts "    set flip 0"
	puts "\}"
    }
    if { $nomtz } {
	puts "if \{ \[file exists solomon_cycle\$LastCycle.mtz \] \} \{"
	 puts "    file delete solomon_cycle\$LastCycle.mtz"
	puts "\}\n"
    }
    puts "\#"
    puts "\# 1 Modify the density with Solomon"
    puts "\#"
    puts "set script \"slvfrc $solv"
    puts "mulsolv auto \$flip"
    puts "slvdens \$solvdens"
    set radius $XMLParse([join "crank soap run step=$step solomon radius" __])
    puts "radius $radius"
    set trunc_low $XMLParse([join "crank soap run step=$step solomon trunc_low" __])
    set trunc_high $XMLParse([join "crank soap run step=$step solomon trunc_high" __])
    puts "trunc $trunc_low $trunc_high"
    puts "extrude"
    puts "mapout"
    puts "END\"\n"

    puts "set logfile \[open \"solomon.log\" a+\]"
    puts "set command \"solomon MAPIN solomon_cycle\$LastCycle.map MAPOUT junk.map << \\\"\$script\\\"\""
    puts "catch \{eval exec \$command\} output"
    puts "puts \$logfile \$output"
    puts "close \$logfile\n"
    puts "set logfile \[open \"solomon.log\" r\]"
    puts "set data \[read \$logfile\]"
    puts "set data \[split \$data \"\\n\"\]"
    puts "foreach line \$data \{"
    puts "   if \{ \[regexp \{new solvdens* *(\[-0-9.\]*).*\} \$line junk sdens \] \} \{"
    puts "   \}"
    puts "   if \{ \[regexp \{  ratio:*                           *(\[-0-9.\]*).*\} \$line junk con \] \} \{"
    puts "   \}"
    puts "\}"
    puts "close \$logfile\n"
    puts "set solvdens \$sdens"
    puts "puts \$solomonlogfile \"Cycle \$CurrentCycle\""
    puts "puts \$solomonlogfile \" Inverse contrast is \$con\""
    puts ""
    if { $nomap } {
	puts "file delete solomon_cycle\$LastCycle.map"
    }
    puts "\#"
    puts "\# 2 Generate structure factors from the modified map"
    puts "\#"
    puts ""
    puts -nonewline "set script \"start -mtzin $mtz_in "
    puts -nonewline "-mapin junk.map -mtzout junk.mtz -colout mod\""
    puts ""
    puts "set logfile \[open \"cinvfft.log\" a+\]"
    puts "set command \"cinvfft <<\$script\""
    puts "catch \{eval exec \$command\} output"
    puts "puts \$logfile \$output"
    puts "close \$logfile"
    puts ""
    puts "\#"
    puts "\# 3 Combine phases"
    puts "\#"
    puts ""
    puts "set script \"LABI FP=$in_fp SIGFP=$in_sigfp HLA=$in_hla HLB=$in_hlb HLC=$in_hlc HLD=$in_hld -"
    puts "PHIBP=$in_phib WP=$in_fom FC=mod.F_phi.F PHIC=mod.F_phi.phi"
    puts "LABO PHCMB=PHISOL WCMB=FOMSOL FWT=FWT PHFWT=PHFWT" 
    puts "RANGES 10 1000"
    puts "ERROR"
    puts "COMBINE PART 1"
    puts "END\"\n"

    puts "set logfile \[open \"sigmaa.log\" a+\]"
    puts "set command \"sigmaa HKLIN junk.mtz HKLOUT solomon_cycle\$CurrentCycle.mtz << \\\"\$script\\\"\""
    puts "catch \{eval exec \$command\} output"
    puts "puts \$logfile \$output"
    puts "close \$logfile"
    puts "set logfile \[open \"sigmaa.log\" r]"
    puts ""
    puts "set data \[read \$logfile\]"
    puts "set data \[split \$data \"\\n\"\]"
    puts "foreach line \$data \{"
    puts "   if \{ \[regexp \{ Overall MEAN FOM is \} \$line \] \} \{"
    puts "      set lastfomline \$line"
    puts "   \} elseif \{ \[regexp \{ Overall MEAN W is \} \$line \] \} \{"
    puts "      set lastfomline \$line"
    puts "   \}"
    puts "   if \{ \[regexp \{ Overall Correlation on \} \$line \] \} \{"
    puts "      set lastcorrline \$line"    
    puts "   \}"
    puts "\}"
    puts ""
    puts "puts \$solomonlogfile \$lastfomline"
    puts "puts \$solomonlogfile \$lastcorrline"
    puts "close \$logfile"

    puts "puts \$solomonlogfile \" \""
    
    puts ""
    puts "\#"
    puts "\# 4 Recalculate map."
    puts "\#"
    puts ""

    puts -nonewline "set script \"start -mtzin solomon_cycle\$CurrentCycle.mtz "
    puts            "-colin-fc /*/*/\\\\\\\[FWT,PHFWT\\\\\\\] -mapout solomon_cycle\$CurrentCycle.map\""

    puts "set logfile \[open \"cfft.log\" a+\]"
    puts "set command \"cfft <<\$script\""
    puts "catch \{eval exec \$command\} output"
    puts "puts \$logfile \$output"
    puts "close \$logfile"
    puts ""
    puts "incr LastCycle\n"
    puts "\} \n"

    puts "close \$solomonlogfile"
    puts ""
    puts "if \{ \[file exists junk.map \] \} \{"
    puts "    file delete junk.map"
    puts "\}"
    puts "if \{ \[file exists junk.mtz \] \} \{"
    puts "    file delete junk.mtz"
    puts "\}"
}

set inputxml [lindex $argv 0]
set crankbin [lindex $argv 1] 
set cycles   [lindex $argv 2]
set solv     [lindex $argv 3]
set enant    [lindex $argv 4]
set nomtz    [lindex $argv 5]
set nomap    [lindex $argv 6]
set flatten  [lindex $argv 7]

if { [file exists $inputxml] == 0 } {
    crank_error "xml2solomontcl.tcl::inputxml file does not exist"
}

source [file join $crankbin crankutils.tcl]

XMLParsefile $inputxml 

OutputSolomontclfile $cycles $solv $enant $nomtz $nomap $flatten
