#
#     Copyright (C) 2004  Raj Pannu, Steven Ness
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#CCP4i_cvs_Id $Id: bp3.tcl,v 1.4 2007/10/09 11:18:48 ccb Exp $

#
# bp3.tcl
#

#
# Tcl part of the CCP4i interface for BP3
#
# Steven Ness 2003
#

#source [SearchPath TOP utils xml_utils.tcl]

# Debug - print out full XML database
proc debug_print_xml_database { } {
	upvar XMLParse xml
	
	set allvars [array get xml]
	for { set i 0 } { $i < [llength $allvars] } { incr i } {
		#puts -nonewline "[lindex $allvars $i]\t"
		incr i
		#puts "[lindex $allvars $i] "
	}
}

#
# Check to see if $atomname is present in the file
# $CLIBD/atomsf.lib
#
proc GoodAtomName { atomname } {
	global env
	
	set atomsf "$env(CLIBD)/atomsf.lib"
#	#puts "atomsf=($atomsf)"

	set file_id [open $atomsf r]

	set done 0
	while { !$done } {
		gets $file_id line
		if { ! [regexp {AD.*} $line]} {
			set done 1
		}
	}
	
	lappend atomlist $line
	gets $file_id line
	gets $file_id line
	gets $file_id line
	gets $file_id line

	while { [gets $file_id line] >= 0 } {
		lappend atomlist $line
		gets $file_id line
		gets $file_id line
		gets $file_id line
		gets $file_id line
	}

	set match 0
	for { set i 0 } { $i < [llength $atomlist] } { incr i } {
		if { [regexp {([A-Za-z+0-9-]*) } [lindex $atomlist $i] full atom] } {
			if { [string compare $atomname $atom] == 0 } {
				set match 1
				break
			}
		}
	}
	
	if { $match == 1 } {
		##puts "Match found ($atomname) ($atom)"
		return 1
	} else {
		##puts "Match not found for ($atomname)"
		return 0
	}

}

#---------------------------------------------------------------------
proc XMLParsefile { xmlfile } {
#---------------------------------------------------------------------
#d_sum Extract information from CCP4 XML files
#d_desc Parse a CCP4 XML file and load the data into
# the global array xmldata.
#d_desc This is a generic version which will read any XML file.
#d_arg xmlfile Full path name of the XML file to be parsed.
	global xmldata

  # Initialise and configure the XML parser
  set parser [ xml::parser ]
  $parser configure -elementstartcommand XMLElementStartHandler
  $parser configure -elementendcommand XMLElementEndHandler
  $parser configure -characterdatacommand XMLCharacterHandler
  $parser configure -ignorewhitespace true

  # Read and parse the specified file
  if { [ReadFile $xmlfile xmltext] } {
     $parser parse $xmltext
     set xmldata(XML_FILE) $xmlfile
     set xmldata(XML_LOADED) 1
  } else {
     # Couldn't load the data
     set xmldata(XML_LOADED) 0
  }
}

proc XMLCharacterHandler {data args} {
	global XMLParseSection
	global XMLParseData
	
	if { $XMLParseSection(coordinate_data) } {
		if { $XMLParseSection(trial) } {
			if { $XMLParseSection(atom) } {
				if { $XMLParseSection(atom_id) } {
					set trim_data [string trim $data]
					if { !($trim_data == "") } {
						lappend XMLParseData(atom_id) $trim_data
					}
				}
				if { $XMLParseSection(element_name) } {
					set trim_data [string trim $data]
					if { !($trim_data == "") } {
						lappend XMLParseData(element_name) $trim_data
					}
				}
				if { $XMLParseSection(x) } {
					set trim_data [string trim $data]
					if { !($trim_data == "") } {
						lappend XMLParseData(x) $trim_data
					}
				}
				if { $XMLParseSection(y) } {
					set trim_data [string trim $data]
					if { !($trim_data == "") } {
						lappend XMLParseData(y) $trim_data
					}
				}
				if { $XMLParseSection(z) } {
					set trim_data [string trim $data]
					#puts "parsing z ($trim_data)"
					if { !($trim_data == "") } {
						lappend XMLParseData(z) $trim_data
					}
				}
				if { $XMLParseSection(occupancy) } {
					set trim_data [string trim $data]
					if { !($trim_data == "") } {
						#puts "parsing occu ($trim_data)"
						lappend XMLParseData(occupancy) $trim_data
						if { $XMLParseData(maxoccupancy) < $trim_data } {
							set XMLParseData(maxoccupancy) $trim_data
						}
					}
				}
			}
		}
	}
}

#---------------------------------------------------------------------
proc XMLElementStartHandler { name attlist } {
#---------------------------------------------------------------------
#d_sum Internal parsing procedure for processing CCP4 XML files
#d_desc See also HandleEndCCP4Xml
#d_arg name Name of the XML element
#d_arg attlist Attribute list for that element
  global xmldata
  global XMLParseSection
  global XMLParseData

  # Each arm of the switch statement deals with a different
  # element in the XML file
  switch $name {
      coordinate_data {
          # Element <coordinate_data>
		  #puts "Setting coordinate_data to 1"
		  set XMLParseSection(coordinate_data) 1
      }
	  trial {
          # Element <trial>
		  if { $XMLParseSection(coordinate_data) } {
			  #puts "Setting trial to 1"
			  set XMLParseSection(trial) 1
		  }
      }
	  atom {
          # Element <general_parameters>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  #puts "Setting atom to 1"
					  set XMLParseSection(atom) 1
					  #puts "XMLParseData(addatoms) == $XMLParseData(addatoms)"
					  if { $XMLParseData(addatoms) == 1 } {
						  set XMLParseData(natoms) [expr $XMLParseData(natoms) + 1]
					  }
				  }
		  }
	  }
	  atom_id {
          # Element <atom_id>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting atom_id to 1"
						  set XMLParseSection(atom_id) 1
					  }
			  }
		  }
	  }
	  element_name {
          # Element <element_name>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting element_name to 1"
						  set XMLParseSection(element_name) 1
					  }
			  }
		  }
	  }
	  x {
          # Element <x>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting x to 1"
						  set XMLParseSection(x) 1
					  }
				  }
		  }
	  }
	  y {
          # Element <y>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting y to 1"
						  set XMLParseSection(y) 1
					  }
				  }
		  }
	  }
	  z {
          # Element <z>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting z to 1"
						  set XMLParseSection(z) 1
					  }
				  }
		  }
	  }
	  occupancy {
          # Element <occupancy>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting occupancy to 1"
						  set XMLParseSection(occupancy) 1
					  }
				  }
		  }
	  }

      *  {
		  # Catch-all generic element - do nothing
		  return 0
      }
  }
  return 1
}

#---------------------------------------------------------------------
proc XMLElementEndHandler { name } {
#---------------------------------------------------------------------
#d_sum Internal parsing procedure for processing CCP4 XML files.
#d_desc See also HandleStartCCP4Xml
#d_arg name Name of the XML element
  global xmldata
  global XMLParseSection
  global XMLParseData

  switch $name {
      coordinate_data {
          # Element <coordinate_data>
		  #puts "Setting coordinate_data to 0"
		  set XMLParseSection(coordinate_data) 0
      }
	  trial {
          # Element <trial>
		  if { $XMLParseSection(coordinate_data) } {
			  #puts "Setting trial to 0"
			  set XMLParseSection(trial) 0
		  }
      }
	  atom {
          # Element <atom>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  #puts "Setting atom to 0"
					  set XMLParseSection(atom) 0
				  }
		  }
	  }

	  atom_id {
          # Element <atom_id>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting atom_id to 0"
						  set XMLParseSection(atom_id) 0
					  }
			  }
		  }
	  }
	  element_name {
          # Element <element_name>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting element_name to 0"
						  set XMLParseSection(element_name) 0
					  }
			  }
		  }
	  }
	  x {
          # Element <x>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting x to 0"
						  set XMLParseSection(x) 0
					  }
				  }
		  }
	  }
	  y {
          # Element <y>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting y to 0"
						  set XMLParseSection(y) 0
					  }
			  }
		  }
	  }
	  z {
          # Element <z>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting z to 0"
						  set XMLParseSection(z) 0
					  }
				  }
		  }
	  }
	  occupancy {
          # Element <occupancy>
		  if { $XMLParseSection(coordinate_data) } {
			  if { $XMLParseSection(trial) } {
					  if { $XMLParseSection(atom) } {
						  #puts "Setting occupancy to 0"
						  set XMLParseSection(occupancy) 0
					  }
				  }
		  }
	  }
       * {
	   # Unrecognised closing element
	   return 0
       }
  }
  return 1
}


#---------------------------------------------------------------------
proc XMLParsefile { xmlfile } {
	# Global used by the XML parser
 	global xmldata
	
	# Data structure to hold actual data
	global XMLParse 
	
	# Working data structure to hold temporary work data
	global XMLWork

	set XMLWork(section_depth) 0
	set XMLWork(section_id) ""
	set XMLWork(section_id_depth) 0

 	# Initialise and configure the XML parser
 	set parser 0
 	set parser [ xml::parser ]
 	$parser configure -elementstartcommand XMLElementStartHandler
 	$parser configure -elementendcommand XMLElementEndHandler
 	$parser configure -characterdatacommand XMLCharacterHandler
 	$parser configure -ignorewhitespace true

 	# Read and parse the specified file
 	if { [ReadFile $xmlfile xmltext] } {
 		$parser parse $xmltext
 		set xmldata(XML_FILE) $xmlfile
 		set xmldata(XML_LOADED) 1
 	} else {
 		# Couldn't load the data
 		set xmldata(XML_LOADED) 0
 	}
}



#-----------------------------------------------------------------------
proc GetXMLDefaults { arrayname } {
#-----------------------------------------------------------------------
# Get the default values of some parameters from XML files, unless
# already set e.g. by re-running a job
   upvar #0 $arrayname array
   global xmldata
   global XMLParseSection
   global XMLParseData

   # XML data input to BP3
   set XMLParseSection(heavy_atom_refinement_and_phasing) 0

   # XML data trial by CRUNCH2
   set XMLParseSection(coordinate_data) 0
   set XMLParseSection(trial) 0
   set XMLParseSection(atom) 0
   set XMLParseSection(atom_id) 0
   set XMLParseSection(element_name) 0
   set XMLParseSection(x) 0
   set XMLParseSection(y) 0
   set XMLParseSection(z) 0
   set XMLParseSection(occupancy) 0

   set XMLParseData(maxoccupancy) 0
   set XMLParseData(addatoms) 1
   set XMLParseData(natoms) 0
   set XMLParseData(ntrials) 0
   set XMLParseData(occupancy) {}
   set XMLParseData(x) {}
   set XMLParseData(y) {}
   set XMLParseData(z) {}
   set XMLParseData(atom) {}
   set XMLParseData(atom_id) {}
   set XMLParseData(occupancy) {}

   # Location of the XML file
   set xmlfile [FileJoin [GetDefaultDirPath ] $array(CRUNCH2_XMLIN)]
   #puts "xmlfile = $xmlfile"

   XmlDataReset

   # Check whether the data from this file has been loaded already
#   if {![XmlDataLoaded $xmlfile]} {
     XMLParsefile $xmlfile
#   }

   #puts "here2"
   #puts "natoms = $XMLParseData(natoms)"
   #puts "maxoccupancy = $XMLParseData(maxoccupancy)"

   #
   # Set interface from data read in from XML file
   #
   # Title
   set array(N_CRYSTALS) 1
   set array(N_ATOMS,1) $XMLParseData(natoms)

#    #puts "Hmmmm..."
#    #puts "x = $XMLParseData(x)"

   #puts "here3"

   for { set i 0 } { $i < [llength $XMLParseData(occupancy)] } { incr i } {
	   #puts "occupancy([lindex $XMLParseData(occupancy) $i])"
   }

   set i 0
   set j 1
   while { $j <= $XMLParseData(natoms) } {
	   set array(ATOM_ID,1_$j) [lindex $XMLParseData(element_name) $i]
 	   set array(ATOM_X,1_$j) [lindex $XMLParseData(x) $i]
 	   set array(ATOM_Y,1_$j) [lindex $XMLParseData(y) $i]
 	   set array(ATOM_Z,1_$j) [lindex $XMLParseData(z) $i]
 	   set occ [lindex $XMLParseData(occupancy) $i]
 	   set normalized_occ [expr $occ / $XMLParseData(maxoccupancy)]
       set rounded_occ [ expr round($normalized_occ * 1000) / 1000.0 ]
 	   set array(ATOM_OCCU,1_$j) $rounded_occ
	   #puts "occ ($occ) ($normalized_occ)"
 	   incr i
 	   incr j
   }

   #puts "here5"

   set i 1
   set array(HEAVY_ATOMS_LIST) ""
   set array(HEAVY_ATOMS_ALIAS) ""
   while { $i <= $XMLParseData(ntrials) } {
 	   lappend array(HEAVY_ATOMS_LIST) $i
 	   lappend array(HEAVY_ATOMS_ALIAS) $i
 	   incr i
   }

    # This is the toggle for the menu to appear
    set array(SOLUTIONS_READY) 1

    set array(HEAVY_ATOMS) 1

    # Redraw the task window
    set w $array(WINDOW)
    RerenderTask $w $arrayname bp3

}

proc DisplaySolution { arrayname } {

	upvar #0 $arrayname array
	global XMLParseData

	set s $array(HEAVY_ATOMS)

	#puts "Switching to $array(HEAVY_ATOMS)"

	set i 0
 	set j 1
	set offset [expr ($s - 1) * $XMLParseData(natoms)]

 	while { $i < $XMLParseData(natoms) } {
		set n [expr $offset + $i]
 		set array(ATOM_ID,1_$j) [lindex $XMLParseData(element_name) $n]
 		set array(ATOM_X,1_$j) [lindex $XMLParseData(x) $n]
 		set array(ATOM_Y,1_$j) [lindex $XMLParseData(y) $n]
 		set array(ATOM_Z,1_$j) [lindex $XMLParseData(z) $n]
 		set occ [lindex $XMLParseData(occupancy) $n]
 		set normalized_occ [expr $occ / $XMLParseData(maxoccupancy)]
 		set rounded_occ [ expr round($normalized_occ * 1000) / 1000.0 ]
 		set array(ATOM_OCCU,1_$j) $rounded_occ
 		incr i
 		incr j
     }

}


proc ChangeHand { arrayname } {
	#
	# Changes the hand of the current solution
	# by multiplying by -1 and then adding 1
	# XYZ are in fractional coordinates
	#

	upvar #0 $arrayname array
	global XMLParseData
	
	# Iterate over all current atoms
	# Multiply by -1
	# Add 1

	set s $array(HEAVY_ATOMS)

	set i 0
 	set j 1
	set offset [expr ($s - 1) * $XMLParseData(natoms)]

 	while { $i < $XMLParseData(natoms) } {
		set n [expr $offset + $i]

 		set array(ATOM_X,1_$j) [expr $array(ATOM_X,1_$j) * -1 + 1]
 		set array(ATOM_Y,1_$j) [expr $array(ATOM_Y,1_$j) * -1 + 1]
 		set array(ATOM_Z,1_$j) [expr $array(ATOM_Z,1_$j) * -1 + 1]

 		incr i
 		incr j
     }


	return 1

}



proc data_proc { arrayname c2 counter } {

	upvar #0 $arrayname array
	global configure

	CreateLine line \
		message "Dataset : $c2" \
		label "Dataset : $c2" \
		message "Dataset identifier" \
		label "ID" \
		widget DATA_ID

	CreateLine line \
		message "Set resolution (default: Obtain from MTZ file)" \
		label "Set resolution" \
		widget SET_RESOLUTION \
		message "Atomic form factors are default to CuKalpha wavelength" \
		label "Change atomic f' and f''" \
		widget CUKALPHA \
		message "Use anomalous data" \
		label "Use anomalous data" \
		widget ANOMALOUS \
	
	CreateLine line \
		label "Atom ID" \
		message "Give atom name" \
		widget FORM_ID \
		label "f'" \
		message "Give atomic f'" \
		widget FORM_FP \
		label "f''" \
		message "Give atomic f''" \
		widget FORM_FPP \
		toggle_display [Indxv CUKALPHA $counter $c2] hide 0
    
	CreateLine line \
		label "Resolution" \
		message "Specify resolution limits for the given crystal and dataset" \
		label "High" \
		message "High resolution limit" \
		widget HIGH_RES \
		label "Low" \
		message "Low resolution limit" \
		widget LOW_RES \
		toggle_display [Indxv SET_RESOLUTION $counter $c2] hide 0
	
	CreateLabinLine line \
		"Choose amplitude (FP) and essential sigma (SIGFP)" \
		HKLIN "FP"  FP   {} \
		-sigma "SIGFP" SIGFP  {} \
		-toggle_display [Indxv ANOMALOUS $counter $c2] hide 1

	CreateLabinLine line \
		"FP Plus" \
		HKLIN "FP+"  FP_PLUS  {} \
		-sigma "SIGFP+" SIGFP_PLUS  {} \
		-toggle_display [Indxv ANOMALOUS $counter $c2] hide 0

	CreateLabinLine line \
		"FP Minus" \
		HKLIN "FP-"  FP_MINUS  {} \
		-sigma "SIGFP-" SIGFP_MINUS  {} \
		-toggle_display [Indxv ANOMALOUS $counter $c2] hide 0
}

proc atom_proc { arrayname c1 counter } {

	upvar #0 $arrayname array
	global configure
	
	CreateLine line \
		label "Atom # $c1 : " \
		-italic \
		label "Atom ID" \
		message "Atom ID" \
		widget ATOM_ID \
		label "Isotropic B factors" \
		message "Isotropic B factors " \
		widget ISOTROPIC_B 

	CreateLine line \
		label "Site" \
		message "Site corresponding to Atom" \
		widget ATOM_SITE \
		toggle_display SAME_SITE hide 0
	
	CreateLine line \
		message "Fractional Coordinates : " \
		label "Frac Coords : " \
		message "Fractional X Coordinate" \
		label "X" \
		widget ATOM_X \
		message "Do not refine fractional x coordinate" \
		label "NOREF" \
		widget ATOM_X_NOREF \
		message "Fractional Y Coordinate" \
		label "Y" \
		widget ATOM_Y \
		message "Do not refine fractional y coordinate" \
		label "NOREF" \
		widget ATOM_Y_NOREF \
		message "Fractional Z Coordinate" \
		label "Z" \
		widget ATOM_Z \
		message "Do not refine fractional z coordinate" \
		label "NOREF" \
		widget ATOM_Z_NOREF \
		toggle_display SAME_SITE hide 1

	CreateLine line \
		message "Atomic occupancy" \
		label "Occupancy" \
		widget ATOM_OCCU \
		message "Do not refine Atomic occupancy" \
		label "NOREF" \
		widget ATOM_OCCU_NOREF \

	CreateLine line \
		message "Atomic isotropic B factor" \
		label "Biso" \
		widget ATOM_BISO \
		message "Do not refine Atomic isotropic B factor" \
		label "NOREF" \
		widget ATOM_BISO_NOREF \
		toggle_display [Indxv ISOTROPIC_B $counter $c1] hide 0

	CreateLine line \
		message "Atomic anisotropic U factor" \
		label "UANO" \
		message "Atomic anisotropic U factor" \
		label "NOREF" \
		widget ATOM_UANO_NOREF \
		message "Atomic anisotropic U factor <U11>" \
		label "U11" \
		widget ATOM_U11 \
		message "Atomic anisotropic U factor <U12>" \
		label "U12" \
		widget ATOM_U12 \
		message "Atomic anisotropic U factor <U13>" \
		label "U13" \
		widget ATOM_U13 \
		message "Atomic anisotropic U factor <U22>" \
		label "U22" \
		widget ATOM_U22 \
		message "Atomic anisotropic U factor <U23>" \
		label "U23" \
		widget ATOM_U23 \
		message "Atomic anisotropic U factor <U33>" \
		label "U33" \
		widget ATOM_U33 \
		toggle_display [Indxv ISOTROPIC_B $counter $c1] hide 1

}


proc crystal_proc { arrayname counter } {

	upvar #0 $arrayname array
	global configure

	CreateLine line \
		message "Crystal ID" \
		label "Crystal ID" \
		widget CRYSTAL_ID \
		message "Change crystallographic cell parameters" \
		label "Change cell parameters" \
		widget CHANGE_CELL
	
	OpenSubFrame frame -toggle_display CHANGE_CELL,$counter hide 0

	CreateLine line \
		label "Crystallographic Cell parameters " \
		message "Cell Parameters: " \
		message "Crystallographic Cell : A" \
		label "A" \
		widget CELL_1 \
		message "Crystallographic Cell : B" \
		label "B" \
		widget CELL_2 \
		message "Crystallographic Cell : C" \
		label "C" \
		widget CELL_3 \
		message "Crystallographic Cell : Alpha" \
		label "Alpha" \
		widget CELL_4 \
		message "Crystallographic Cell : Beta" \
		label "Beta" \
		widget CELL_5 \
		message "Crystallographic Cell : Gamma" \
		label "Gamma" \
		widget CELL_6 

	CloseSubFrame

	CreateExtendingFrame N_ATOMS atom_proc \
        "Define Atom" \
        "Add Atom" \
		[list  ATOM_ID ISOTROPIC_B] \
		-counter $counter 

	CreateExtendingFrame N_DATA data_proc \
        "Define Data" \
        "Add Data" \
		[list  DATA_ID FP SIGFP FP_PLUS SIGFP_PLUS FP_MINUS SIGFP_MINUS ANOMALOUS ERROR_PARAMS SET_RESOLUTION CUKALPHA \
			 NBINS \
			 LUZZ_ISOE1 LUZZ_ISOE2 LUZZ_ISOE3 LUZZ_ISOE4 LUZZ_ISOE5 \
			 LUZZ_ISOE6 LUZZ_ISOE7 LUZZ_ISOE8 LUZZ_ISOE9 LUZZ_ISOE10 \
			 LUZZ_ANOE1 LUZZ_ANOE2 LUZZ_ANOE3 LUZZ_ANOE4 LUZZ_ANOE5 \
			 LUZZ_ANOE6 LUZZ_ANOE7 LUZZ_ANOE8 LUZZ_ANOE9 LUZZ_ANOE10 \
			 LUZZ_SDLU1 LUZZ_SDLU2 LUZZ_SDLU3 LUZZ_SDLU4 LUZZ_SDLU5 \
			 LUZZ_SDLU6 LUZZ_SDLU7 LUZZ_SDLU8 LUZZ_SDLU9 LUZZ_SDLU10 ] \
		-counter $counter 


}


proc frac_sites { arrayname counter } {

	upvar #0 $arrayname array
	global configure

	CreateLine line \
		label "Site # $counter : " \
		-italic \
		message "Xfrac - X fractional coordinate" \
		label "Xfrac" \
		widget SITE_X \
		message "Yfrac - Y fractional coordinate" \
		label "Yfrac" \
		widget SITE_Y \
		message "Zfrac - Z fractional coordinate" \
		label "Zfrac" \
		widget SITE_Z \
		message "Noref Xfrac - X fractional coordinate" \
		label "Xfrac Noref" \
		widget SITE_X_NOREF \
		message "Noref Yfrac - Y fractional coordinate" \
		label "Yfrac Noref" \
		widget SITE_Y_NOREF \
		message "Noref Zfrac - Z fractional coordinate" \
		label "Zfrac Noref" \
		widget SITE_Z_NOREF

}


proc bp3_setup { typedefVar arrayname } {

# want to use crank interface instead
    WarningMessage "To run substructure phasing with Bp3, use the \"Crank - automated package\" CCP4i gui, and \"Start the pipeline\" with Substructure phasing."
    return 0    


	upvar #0 $typedefVar typedef

	set typedef(_bp3_mode)		{ menu { "Refine"
		"Phase"
		"Diff" }
		{ REFINE PHASE DIFF } }

	set typedef(_nbins) { menu { 15 14 13 12 11 10 9 8 7 6 5 } }

	set typedef(_heavy_atoms) { varmenu HEAVY_ATOMS_LIST HEAVY_ATOMS_ALIAS 8 }

}


#-----------------------------------------------------------------------
proc bp3_task_window { arrayname } {
	#-----------------------------------------------------------------------
	upvar #0 $arrayname array

	if { ![CreateTaskWindow $arrayname  \
			   "BP3 - Multivariate likelihood substructure refinement and phasing" BP3 \
			   [list "Site" \
					"Crystal" \
					"Other parameters" \
				   ] ] }  { return 0 }

	#
	# Retrive defaults from XML file if available & requested
	#
	source [SearchPath TOP utils xml_utils.tcl]

	#
	# Populate the task window
	#
	OpenFolder protocol

	CreateLine line \
		message "Enter a title for this job (TITLE)" \
		label "Title" \
		widget TITLE -width 80

	CreateInputFileLine line \
		"Enter input MTZ file name (HKLIN)" \
		"MTZ in" \
		HKLIN DIR_HKLIN \
		-fileout HKLOUT DIR_HKLOUT _bp3_ \
		-setfileparam cell_1 CELL_1 \
		-setfileparam cell_2 CELL_2 \
		-setfileparam cell_3 CELL_3 \
		-setfileparam cell_4 CELL_4 \
		-setfileparam cell_5 CELL_5 \
		-setfileparam cell_6 CELL_6 

	CreateOutputFileLine line \
		"Enter output MTZ file name (HKLOUT)" \
		"MTZ out" \
		HKLOUT DIR_HKLOUT
	
	CreateLine line \
		message "Same site in more than one crystal" \
		label "Same site in more than one crystal" \
		widget SAME_SITE \

 	CreateLine line \
 		message "Read in CRUNCH2 XML Output" \
 		label "Read in CRUNCH2 XML Output" \
 		widget READ_CRUNCH2_XML \
		button "Load XML Now" \
		-command "GetXMLDefaults $arrayname"

	CreateInputFileLine line \
 		"Enter input CRUNCH2 XML file" \
		"XML in" \
 		CRUNCH2_XMLIN CRUNCH2_DIR_XMLIN \
 		-toggle_display READ_CRUNCH2_XML open 1 \
 		-command "GetXMLDefaults $arrayname"

	CreateLine line \
		message "Change hand to other enantiomorph" \
		button "Change hand" \
		-command "ChangeHand $arrayname" \
		toggle_display SOLUTIONS_READY open 1
# 		message "solution" \
# 		label "solution" \
# 		widget HEAVY_ATOMS \
# 		-command "DisplaySolution $arrayname" \

	OpenFolder 1 SAME_SITE open 1 hide
	
	CreateExtendingFrame N_SITES frac_sites \
        "Define sites" \
        "Add Site" \
		[list  SITE_X \
			 SITE_Y \
			 SITE_Z \
			 SITE_X_NOREF \
			 SITE_X_NOREF \
			 SITE_Z_NOREF \
			] 
        
	OpenFolder 2
	
	CreateToggleFrame N_CRYSTALS crystal_proc \
        "Define Crystal" \
		"Crystal #" \
        "Add Crystal" \
		[list  CRYSTAL_ID \
			 CELL_1 \
			 CELL_2 \
			 CELL_3 \
			 CELL_4 \
			 CELL_5 \
			 CELL_6 \
			 N_ATOMS \
			 N_DATA \
			 FP \
			 SIGFP \
			 FP_PLUS \
			 SIGFP_PLUS \
			 FP_MINUS \
			 SIGFP_MINUS \
			 ANOMALOUS \
			 ERROR_PARAMS \
			 SET_RESOLUTION \
			 CUKALPHA \
			 NBINS \
			 LUZZ_ISOE1 \
			 LUZZ_ISOE2 \
			 LUZZ_ISOE3 \
			 LUZZ_ISOE4 \
			 LUZZ_ISOE5 \
			 LUZZ_ISOE6 \
			 LUZZ_ISOE7 \
			 LUZZ_ISOE8 \
			 LUZZ_ISOE9 \
			 LUZZ_ISOE10 \
			 LUZZ_ANOE1 \
			 LUZZ_ANOE2 \
			 LUZZ_ANOE3 \
			 LUZZ_ANOE4 \
			 LUZZ_ANOE5 \
			 LUZZ_ANOE6 \
			 LUZZ_ANOE7 \
			 LUZZ_ANOE8 \
			 LUZZ_ANOE9 \
			 LUZZ_ANOE10 \
			 LUZZ_SDLU1 \
			 LUZZ_SDLU2 \
			 LUZZ_SDLU3 \
			 LUZZ_SDLU4 \
			 LUZZ_SDLU5 \
			 LUZZ_SDLU6 \
			 LUZZ_SDLU7 \
			 LUZZ_SDLU8 \
			 LUZZ_SDLU9 \
			 LUZZ_SDLU10 \
			] \
		-child atom_proc \
		-child data_proc \

	OpenFolder 3 closed

	CreateLine line \
		message "The number of cycles of refinement (CYCLE)" \
		label "Refinement cycles" \
		widget CYCLES \

	CreateLine line \
		message "Number of nodes of the CENTric integral and the acentric PHASe and AMPLitude integral." \
		label "Number of nodes for : Centric integral"\
		widget CENTRIC_NODES \
		label "Acentric Phase"\
		widget ACENTRIC_NODES \
		label "Amplitude integral"\
		widget AMPLITUDE_NODES

	CreateLine line \
		message "Fast phasing - recommended for MAD" \
		label "Fast phasing" \
		widget PHASE  \

	CreateLine line \
		message "Only one cycle of minimization" \
		label "Only one cycle of minimization" \
		widget REFINE_ALL \
		message "Print all statistics at each step of refinement" \
		label "Print all statistics" \
		widget STATS \
		label "Verbose" \
		message "Verbose program output" \
		widget VERBOSE
	
	CreateLine line \
		label "Two dimensional integration" \
		message "Two dimensional integration" \
		widget TWO_D_INTEGRATION \
		label "Allin" \
		message "Allin Keyword" \
		widget ALLINFLAG

}
