
# phaser_EP.tcl --
#
# Maximum Likelihood Experimental Phasing 
#
# CCP4Interface
#
# =======================================================================

#------------------------------------------------------------------------
 proc phaser_EP_composition { arrayname counter_component } {
#------------------------------------------------------------------------
# The contents of the "Composition" folder, extending frame 
  upvar #0 $arrayname array

  CreateLine line \
      message "Select for protein or DNA/RNA as a component of the asymmetric unit" \
      label "Component #$counter_component" \
      widget PROTDNA \
      message "Choose how to define the size of this component" \
      label "" \
      widget COMP_OPTION -oblig \
      message "Enter the molecular weight of this component" \
      widget MW -oblig \
      message "Enter the number of copies of component #$counter_component in the asymmetric unit" \
      label "Number in asymmetric unit" \
      widget ASYM -oblig \
     toggle_display COMP_OPTION,$counter_component open MW hide

  CreateLine line \
      message "Select for protein or DNA/RNA as a component of the asymmetric unit" \
      label "Component #$counter_component" \
      widget PROTDNA \
      message "Choose how to define the size of this component" \
      label "" \
      widget COMP_OPTION -oblig \
      message "Enter the number of copies of component #$counter_component in the asymmetric unit" \
      label "                Number in asymmetric unit" \
      widget ASYM -oblig \
     toggle_display COMP_OPTION,$counter_component open FASTA hide

  CreateInputFileLine line \
      "Select the file containing the sequence in single letter code or FASTA format" "SEQ file" COMP_FILE DIR_COMP_FILE \
     -toggle_display COMP_OPTION,$counter_component open FASTA hide

  }

#------------------------------------------------------------------------
  proc phaser_EP_setup { typedefVar arrayname } {
#------------------------------------------------------------------------
  upvar #0 $typedefVar typedef
  upvar #0 $arrayname array

  set typedef(_phaser_onoff_option) \
    { menu { "on" "off" } { ON OFF } }

  set typedef(_phaser_comp_option) \
    { menu  { "molecular weight"  "sequence file" } { "MW"  "FASTA" } }

  set typedef(_phaser_resolution_option) \
    { menu  { "dmin only" "dmin and dmax" } { "DMINONLY" "DMINMAX" } }

  set typedef(_phaser_sadmacrocycle_option) \
    { menu  { "default protocol"  "phasing only"  "macrocycles entered below"} { "DEFAULT" "PHASING" "ENTERED"} }

  set typedef(_phaser_sadmacrocycle_part_option) \
    { menu  { "default protocol"  "phasing only"  "macrocycles entered below"} { "DEFAULT" "PHASING" "ENTERED"} }

  DefineMenu _phaser_mode_ep [list "Single-wavelength anomalous dispersion (SAD)" \
                                   "SAD with molecular replacement partial structure"] \
                                   [list "SAD" "MRSAD"]

  DefineMenu _phaser_protnucleic \
    [list "protein" "nucleic acid" ] [list "PROTEIN" "NUCLEIC" ]

  DefineMenu _phaser_clash_option \
    [list "by optical resolution" "by fixed distance" ] [list "DEFAULT" "DISTANCE" ]

  DefineMenu _phaser_coord_frac \
    [list "fractional" "orthogonal" ] [list "FRAC" "ORTH" ]

  DefineMenu _phaser_scat_option \
    [list "at CuK-alpha wavelength" "at other wavelength" "by fluorescence scan" "explicitly by atomtype"]  [list "CUKA" "LAMBDA" "MEASURED" "ATOMTYPE"]

  DefineMenu _phaser_ha_sites_option_part \
    [list "none" "entered below" "in SOL file" "in HA file" "in PDB file" ] \
    [list "NONE" "MANUAL" "SOL" "HA" "PDB" ]

  DefineMenu _phaser_ha_sites_option_sad \
    [list "entered below" "in SOL file" "in HA file" "in PDB file" ] \
    [list "MANUAL" "SOL" "HA" "PDB" ]

  DefineMenu _phaser_bfactor_option \
    [list "Wilson B" "Entered" ] \
    [list "WILSON" "VALUE" ]

  DefineMenu _phaser_rms_option \
   [list "rms difference" "sequence identity" ] [list "RMS" "IDENT" ]

  DefineMenu _phaser_hand_option \
   [list "Original enantiomorph" "Change enantiomorphic" "Both enantiomorphs"] [list "OFF" "ON" "BOTH" ]

  DefineMenu _phaser_scattering_option \
    [ list "average solvent content for protein crystal" "solvent content of protein crystal" "number of residues in asymmetric unit" "components in asymmetric unit"] \
    [ list "AVERAGE" "SOLVENT" "NRES" "COMPONENT"]

  return 1
  }

#------------------------------------------------------------------------
proc phaser_EP_run { arrayname } {
#------------------------------------------------------------------------

  upvar #0 $arrayname array

  if { $array(HKLIN) == "" } {
     WarningMessage "The mtz reflection data file has not been set"
     return 0
  }


#I have no idea why COMP_OPTION is either "sequence file" or "molecular weight"
  if { ([GetValue $arrayname SCATTERING] == "COMPONENT") } {
  for { set n 1 } { $n <= $array(N_COMPONENT) } { incr n } {
    if { $array(COMP_OPTION,$n) == "sequence file" &&  $array(COMP_FILE,$n) == "" } {
       WarningMessage "The sequence file for component # $n is not set"
       return 0
     }
    if { $array(COMP_OPTION,$n) == "molecular weight" &&  $array(MW,$n) == "" } {
       WarningMessage "The molecular weight for component # $n is not set"
       return 0
     }
   }
   }

  if { ([GetValue $arrayname SCATTERING] == "NRES")} { 
    if { ([GetValue $arrayname SCATTERING_NRES] == "")} { 
     WarningMessage "The number of residues in the composition has not been set"
     return 0
  }
  }

  if { ([GetValue $arrayname SCATTERING] == "SOLVENT")} { 
    if { ([GetValue $arrayname SCATTERING_SOLVENT] == "")} { 
     WarningMessage "The solvent content in the composition has not been set"
     return 0
  }
  }

  if { ([GetValue $arrayname SCATTERING] == "SOLVENT")} { 
    if { ([GetValue $arrayname SCATTERING_SOLVENT] == "")} { 
     WarningMessage "The solvent content in the composition has not been set"
     return 0
  }
  }

  if { ([GetValue $arrayname EP_MODE] == "SAD") } {
    if { ([GetValue $arrayname LLGC_COMPLETE_SAD] == "ON") } {
  for { set n 1 } { $n <= $array(N_LLGC_ATOMTYPE_SAD) } { incr n } {
    if { ($array(LLGC_ATOMTYPE_SAD,$n) == "") } {
       WarningMessage "Atom type $n for structure completion not set"
       return 0
     }
   }
   }
   }

  if { ([GetValue $arrayname EP_MODE] == "MRSAD") } {
    if { ([GetValue $arrayname PARTIAL_RMSID] == "")} {
     WarningMessage "The sequence identity or rms difference for the model has not been set"
     return 0
  }
  }

  if { ![SetHarvestParams $arrayname HKLIN  -run] } { return 0 }

# All output files with the user specified root will be saved to the project dir
  return 1
  }

#------------------------------------------------------------------------
 proc phaser_EP_composition { arrayname counter_component } {
#------------------------------------------------------------------------
# The contents of the "Composition" folder, extending frame 
  upvar #0 $arrayname array

  CreateLine line \
      message "Select for protein or DNA/RNA as a component of the asymmetric unit" \
      label "Component #$counter_component" \
      widget PROTDNA \
      message "Choose how to define the size of this component" \
      label "" \
      widget COMP_OPTION -oblig \
      message "Enter the molecular weight of this component" \
      widget MW -oblig \
      message "Enter the number of copies of component #$counter_component in the asymmetric unit" \
      label "Number in asymmetric unit" \
      widget ASYM -oblig \
     toggle_display COMP_OPTION,$counter_component open MW hide

  CreateLine line \
      message "Select for protein or DNA/RNA as a component of the asymmetric unit" \
      label "Component #$counter_component" \
      widget PROTDNA \
      message "Choose how to define the size of this component" \
      label "" \
      widget COMP_OPTION -oblig \
      message "Enter the number of copies of component #$counter_component in the asymmetric unit" \
      label "                Number in asymmetric unit" \
      widget ASYM -oblig \
     toggle_display COMP_OPTION,$counter_component open FASTA hide

  CreateInputFileLine line \
      "Select the file containing the sequence in single letter code or FASTA format" "SEQ file" COMP_FILE DIR_COMP_FILE \
     -toggle_display COMP_OPTION,$counter_component open FASTA hide

  }

#-----------------------------------------------------------------------
proc phaser_EP_tog_bfactor_off_sad { arrayname } {
#-----------------------------------------------------------------------
  upvar #0 $arrayname array

  set array(TOG_BFACTOR_SAD) OFF
}

#------------------------------------------------------------------------
 proc phaser_EP_updatevarlabel { arrayname FILE_SPACEGROUP } {
#------------------------------------------------------------------------
# procedure to update the FILE_SPACEGROUP varlabel
  upvar #0 $arrayname array
  set field_list [GetWidget $arrayname FILE_SPACEGROUP]
  foreach field $field_list {
    if { [winfo exists $field] } { $field configure -text $array(FILE_SPACEGROUP) } }
  }

#------------------------------------------------------------------------
  proc phaser_EP_set_reso {arrayname} {
#------------------------------------------------------------------------
  upvar #0 $arrayname array
  set array(RESO_DMIN) [GetValue $arrayname RESOLUTION_DMIN]
  set array(RESO_DMAX) [GetValue $arrayname RESOLUTION_DMAX]
}

#------------------------------------------------------------------------
  proc phaser_EP_tog_activate {arrayname} {
#------------------------------------------------------------------------

  }

#------------------------------------------------------------------------
 proc phaser_EP_ha_sad {  arrayname counter_a } {
#------------------------------------------------------------------------
# Procedure to define a anomalous atom site manually
  upvar #0 $arrayname array
  CreateLine line \
      message "Add new anomalous atom site manually" \
      widget HA_ATOM_TYPE_SAD -oblig \
      widget HA_ATOM_FRAC_SAD \
      widget HA_ATOM_X_SAD -oblig \
      widget HA_ATOM_Y_SAD -oblig \
      widget HA_ATOM_Z_SAD -oblig \
      widget HA_ATOM_O_SAD -oblig \
      widget HA_ATOM_B_SAD \
      format template HA
  }
#------------------------------------------------------------------------
 proc phaser_EP_ha_part {  arrayname counter_a } {
#------------------------------------------------------------------------
# Procedure to define a anomalous atom site manually
  upvar #0 $arrayname array
  CreateLine line \
      message "Add new anomalous atom site manually" \
      widget HA_ATOM_TYPE_PART -oblig \
      widget HA_ATOM_FRAC_PART \
      widget HA_ATOM_X_PART -oblig \
      widget HA_ATOM_Y_PART -oblig \
      widget HA_ATOM_Z_PART -oblig \
      widget HA_ATOM_O_PART -oblig \
      widget HA_ATOM_B_PART \
      format template HA
  }
#------------------------------------------------------------------------
 proc phaser_EP_scat_atomtype_sad {  arrayname counter_f } {
#------------------------------------------------------------------------
# Procedure to define a scattering for  atom manually
  upvar #0 $arrayname array
  CreateLine line \
      message "Add new atomtype Fp and Fdp manually" \
      label "Atom type " \
      widget SCAT_ATOMTYPE_SAD -oblig \
      label "FP = " \
      widget SCAT_ATOMTYPE_FP_SAD -oblig \
      label "FDP = " \
      widget SCAT_ATOMTYPE_FDP_SAD -oblig 
  }
#------------------------------------------------------------------------
 proc phaser_EP_llgc_atomtype_sad {  arrayname counter_f } {
#------------------------------------------------------------------------
# Procedure to define a scattering for  atom manually
  upvar #0 $arrayname array
  CreateLine line \
      message "Atomtypes for LLG-maps: if no completion, LLG-maps will still be calculated" \
      label "LLG-map calculation atom type" \
      widget LLGC_ATOMTYPE_SAD -oblig 
  }
#------------------------------------------------------------------------
  proc phaser_EP_sadmacro { arrayname counter_s } {
#------------------------------------------------------------------------
  global configure
  upvar #0 $arrayname array
  CreateLine line \
      message "Overall Occupancy Scale" \
      widget TOG_MACSAD_K \
      message "Overall B-factor" \
      widget TOG_MACSAD_B \
      message "Overall Sigma Scale" \
      widget TOG_MACSAD_SIGMA \
      message "Anomalous atom positions" \
      widget TOG_MACSAD_XYZ \
      message "Anomalous atom occupancy" \
      widget TOG_MACSAD_OCC \
      message "Anomalous atom B-factor" \
      widget TOG_MACSAD_BFAC \
      message "Anomalous atom F double prime" \
      widget TOG_MACSAD_FDP \
      message "Variance Real part of D-phi" \
      widget TOG_MACSAD_SA \
      message "Variance Imaginary part of D-phi" \
      widget TOG_MACSAD_SB \
      message "Variance Sigma-Plus" \
      widget TOG_MACSAD_SP \
      message "Variance Sigma-Delta" \
      widget TOG_MACSAD_SD \
      message "Number of Cycles" \
      widget MACSAD_NCYCL \
      format template SAD_MACROCYCLE_LABEL
  }
#------------------------------------------------------------------------
  proc phaser_EP_sadmacro_part { arrayname counter_s } {
#------------------------------------------------------------------------
  global configure
  upvar #0 $arrayname array
  CreateLine line \
      message "Overall Occupancy Scale" \
      widget TOG_MACS_PART_K \
      message "Overall B-factor" \
      widget TOG_MACS_PART_B \
      message "Overall Sigma Scale" \
      widget TOG_MACS_PART_SIGMA \
      message "Anomalous atom positions" \
      widget TOG_MACS_PART_XYZ \
      message "Anomalous atom occupancy" \
      widget TOG_MACS_PART_OCC \
      message "Anomalous atom B-factor" \
      widget TOG_MACS_PART_BFAC \
      message "Anomalous atom F double prime" \
      widget TOG_MACS_PART_FDP \
      message "Variance Real part of D-phi" \
      widget TOG_MACS_PART_SA \
      message "Variance Imaginary part of D-phi" \
      widget TOG_MACS_PART_SB \
      message "Variance Sigma-Plus" \
      widget TOG_MACS_PART_SP \
      message "Variance Sigma-Delta" \
      widget TOG_MACS_PART_SD \
      message "Partial Structure Scale" \
      widget TOG_MACS_PART_PK \
      message "Partial Structure B-factor" \
      widget TOG_MACS_PART_PB \
      message "Number of Cycles" \
      widget MACS_PART_NCYCL \
      format template SAD_MACROCYCLE_PART_LABEL
  }
#------------------------------------------------------------------------
  proc phaser_EP_task_window {arrayname} {
#------------------------------------------------------------------------
  upvar #0 $arrayname array
  global configure

  if { [CreateTaskWindow $arrayname \
        "Maximum Likelihood Experimental Phasing"\
        "Phaser" \
        [ list "Define data" \
               "Define atoms" \
               "Composition of the asymmetric unit" \
               "Define refinement parameters" \
               "Additional parameters" ] \
       ] == 0 } return

  SetHarvestParams $arrayname HKLIN -init

# SetProgramHelpFile phaser

#=PROTOCOL===============================================================

  OpenFolder protocol

  CreateTitleLine line TITLE

  CreateLine line \
      label "Mode for experimental phasing" widget EP_MODE 


#=FILE===================================================================

# OpenFolder file

#========================================================================
# The "Define data" folder

  OpenFolder 1 

  CreateInputFileLine line \
      "Enter MTZ file name (HKLIN)" \
      "MTZ in     " \
      HKLIN DIR_HKLIN \
        -setfileparam resolution_min RESOLUTION_DMAX \
        -setfileparam resolution_max RESOLUTION_DMIN \
        -setfileparam space_group_name FILE_SPACEGROUP \
        -command "phaser_EP_set_reso $arrayname" \
        -command "UpdateHarvestMTZ $arrayname HKLIN"

  CreateLine line \
     help pname \
     message "Enter unique identifier for crystal " \
     label "Crystal Name" widget HARVEST_PNAME  -oblig  -command "util_fix_harvestnames $arrayname"\
     help dname \
      message "Enter unique identifier for wavelength" \
      label "Wavelength Name" widget HARVEST_DNAME -oblig -command "util_fix_harvestnames $arrayname"\

  CreateLabinLine line \
      "Select anomalous amplitude F+ and anomalous sigma SIGF+" \
      HKLIN "F(+)" Fp_SAD [list F+ "F*+*" ] \
      -sigma "SIGF(+)" SIGFp_SAD [list SIGF+ "SIG*+*"] 
  CreateLabinLine line \
      "Select anomalous amplitude F- and anomalous sigma SIGF-" \
      HKLIN "F(-)" Fm_SAD [list F- "F*-*"] \
      -sigma "SIGF(-)" SIGFm_SAD [list SIGF- "SIG*-*"] 


  CreateLine line \
      message "Default is to take all data" \
      widget TOG_RESOLUTION -toggleon \
      label "Resolution" widget RESOLUTION_DMAX label "A to"  widget RESOLUTION_DMIN label "A" 

  CreateLine line \
      message "Space group read from mtz file." \
      label "Space group read from mtz file" \
      varlabel FILE_SPACEGROUP 

  CreateLine line \
      message "Keep or change enantiomorph of input sites" \
      widget TOG_HAND_OPTION -toggleon label "Enantiomorph choice"  widget HAND_ONOFF \
      toggle_display EP_MODE open SAD hide MRSAD

  CreateLine line \
      message "Choose how to determine f' and f''" \
      label "Scattering"  widget SCAT_OPTION_SAD \
      label ""  \
      message "Wavelength for data" \
      widget SCAT_LAMBDA_SAD -oblig label "A    "\
      message "Fix starting f''?" \
      widget TOG_FIX_FDP_SAD  label "fix FDP" \
      toggle_display SCAT_OPTION_SAD open LAMBDA

  CreateLine line \
      message "Choose how to determine f' and f''" \
      label "Scattering"  widget SCAT_OPTION_SAD \
      message "Fix starting f''?" \
      widget TOG_FIX_FDP_SAD  label "fix FDP" \
      toggle_display SCAT_OPTION_SAD open CUKA

  CreateLine line \
      message "Choose how to determine f' and f''" \
      label "Scattering"  widget SCAT_OPTION_SAD \
      message "Measured values" \
      label "FP ="  widget SCAT_FP_SAD  -oblig label "FDP =" widget SCAT_FDP_SAD  -oblig \
      message "Fix starting f''?" \
      widget TOG_FIX_FDP_SAD  label "fix FDP" \
      toggle_display SCAT_OPTION_SAD open MEASURED

  CreateLine line \
      message "Choose how to determine f' and f''" \
      label "Scattering"  widget SCAT_OPTION_SAD \
      message "Fix starting f''?" \
      widget TOG_FIX_FDP_SAD  label "fix FDP" \
      toggle_display SCAT_OPTION_SAD open ATOMTYPE

  OpenSubFrame frame \
      -toggle_display SCAT_OPTION_SAD open ATOMTYPE hide 
  CreateExtendingFrame N_SCAT_ATOMTYPE_SAD phaser_EP_scat_atomtype_sad \
      "Add another atomtype" "Add another atomtype" \
      [list SCAT_ATOMTYPE_SAD SCAT_ATOMTYPE_FP_SAD SCAT_ATOMTYPE_FDP_SAD ] 

  CloseSubFrame

  CreateLine line \
      message "Maps will be calculated but no completion done if OFF" \
      widget TOG_LLGC_SAD -toggleon \
      label "LLG-map completion" widget LLGC_COMPLETE_SAD \
      toggle_display LLGC_COMPLETE_SAD open OFF

  CreateLine line \
      message "Maps will be calculated but no completion done if OFF" \
      widget TOG_LLGC_SAD -toggleon \
      label "LLG-map completion" widget LLGC_COMPLETE_SAD \
      message "Maximum number of cycles of LLG-map calculation and new atom addition" \
      label "Maximum number of cycles of completion" widget LLGC_NCYC \
      toggle_display LLGC_COMPLETE_SAD open ON

  CreateLine line \
      message "Default sigma cut-off = 6" \
      widget TOG_LLGC_SIGMA_SAD -toggleon \
      label "LLG-map sigma cut-off for adding new atom sites" widget LLGC_SIGMA_SAD 

  CreateLine line \
      message "Default clash distance is optical resolution" \
      widget TOG_LLGC_CLASH_SAD -toggleon \
      label "LLG-map atomic separation distance cut-off" widget LLGC_CLASH_OPTION \
      toggle_display LLGC_CLASH_OPTION open DEFAULT

  CreateLine line \
      message "Default clash distance is optical resolution" \
      widget TOG_LLGC_CLASH_SAD -toggleon \
      label "LLG-map atomic separation distance cut-off" widget LLGC_CLASH_OPTION widget LLGC_CLASH_DISTANCE \
      toggle_display LLGC_CLASH_OPTION open DISTANCE

  OpenSubFrame frame 
     
  CreateExtendingFrame N_LLGC_ATOMTYPE_SAD phaser_EP_llgc_atomtype_sad \
      "Add another atomtype" "Add another atomtype" \
      [list LLGC_ATOMTYPE_SAD ] 

  CloseSubFrame


#======================================================================== 
# The "Define atoms " folder

  OpenFolder 2 

  OpenSubFrame frame \
      -toggle_display EP_MODE open MRSAD hide
  CreateInputFileLine line \
      "Select a PDB file for starting partial structure" \
      "Partial structure " \
      PARTPDB DIR_PARTPDB 
  CloseSubFrame

  CreateLine line \
      message "Specify RMS error in Angstrom, or \
               via the sequence identity (percentage or a fraction)" \
      label "Similarity of PDB of partial structure to the target structure" \
      widget PARTIAL_RMS_OPTION \
      widget PARTIAL_RMSID -oblig \
      toggle_display EP_MODE open MRSAD hide

  OpenSubFrame frame \
      -toggle_display EP_MODE open SAD hide

  CreateLine line \
     message "Select how you wish to define the anomalous atom sites" \
     label "Anomalous atom sites" widget HA_SITES_OPTION_SAD \
        -command "phaser_EP_tog_bfactor_off_sad $arrayname" \
     message "Change input atomic B-factors to Wilson or input value" \
     widget TOG_BFACTOR_SAD -toggleon \
     label "Set B-factors to"   widget BFACTOR_OPTION_SAD \
     label "" widget BFACTOR_VALUE_SAD -oblig \
     toggle_display BFACTOR_OPTION_SAD open VALUE

  CreateLine line \
     message "Select how you wish to define the anomalous atom sites" \
     label "Anomalous atom sites" widget HA_SITES_OPTION_SAD \
        -command "phaser_EP_tog_bfactor_off_sad $arrayname" \
     message "Change input atomic B-factors to Wilson or input value" \
     widget TOG_BFACTOR_SAD -toggleon \
     label "Set B-factors to"   widget BFACTOR_OPTION_SAD \
     toggle_display BFACTOR_OPTION_SAD open WILSON

  OpenSubFrame frame \
      -toggle_display HA_SITES_OPTION_SAD open MANUAL hide 
  CreateLineTemplate HA { 0.0 0.15 0.35 0.45 0.55 0.65 0.75 }
  CreateLine line \
      label "Atom Type" label "Frac/Orth" label "X" label "Y" label "Z" label "O" label "B" \
      format template HA
  CreateExtendingFrame N_HA_SAD phaser_EP_ha_sad \
      "Add another anomalous atom site" "Add another anomalous atom site" \
      [list HA_ATOM_TYPE_SAD HA_ATOM_FRAC_SAD HA_ATOM_X_SAD HA_ATOM_Y_SAD HA_ATOM_Z_SAD HA_ATOM_O_SAD HA_ATOM_B_SAD ] 

  CloseSubFrame

  OpenSubFrame frame \
      -toggle_display HA_SITES_OPTION_SAD open PDB hide 
  CreateInputFileLine line \
      "Select the pdb file" "PDB file" HA_FILE_PDB_SAD DIR_HA_FILE_PDB_SAD
  CloseSubFrame

  OpenSubFrame frame \
     -toggle_display HA_SITES_OPTION_SAD open HA hide 
  CreateInputFileLine line \
     "Select the heavy atom file" "HA   file"  HA_FILE_HA_SAD DIR_HA_FILE_HA_SAD
  CloseSubFrame

  OpenSubFrame frame \
      -toggle_display HA_SITES_OPTION_SAD open SOL hide 
  CreateInputFileLine line \
      "Select the sol file" "SOL file" HA_FILE_SOL_SAD  DIR_HA_FILE_SOL_SAD
  CloseSubFrame

  CloseSubFrame

  CloseSubFrame

  OpenSubFrame frame \
      -toggle_display EP_MODE open MRSAD hide

  CreateLine line \
     message "Select how you wish to define the anomalous atom sites" \
     label "Anomalous atom sites" widget HA_SITES_OPTION_PART \
        -command "phaser_EP_tog_bfactor_off_sad $arrayname" \
     message "Change input atomic B-factors to Wilson or input value" \
     widget TOG_BFACTOR_SAD -toggleon \
     label "Set B-factors to"   widget BFACTOR_OPTION_SAD \
     label "" widget BFACTOR_VALUE_SAD -oblig \
     toggle_display BFACTOR_OPTION_SAD open VALUE

  CreateLine line \
     message "Select how you wish to define the anomalous atom sites" \
     label "Anomalous atom sites" widget HA_SITES_OPTION_PART \
        -command "phaser_EP_tog_bfactor_off_sad $arrayname" \
     message "Change input atomic B-factors to Wilson or input value" \
     widget TOG_BFACTOR_SAD -toggleon \
     label "Set B-factors to"   widget BFACTOR_OPTION_SAD \
     toggle_display BFACTOR_OPTION_SAD open WILSON

  OpenSubFrame frame \
      -toggle_display HA_SITES_OPTION_PART open MANUAL hide 
  CreateLineTemplate HA { 0.0 0.15 0.35 0.45 0.55 0.65 0.75 }
  CreateLine line \
      label "Atom Type" label "Frac/Orth" label "X" label "Y" label "Z" label "O" label "B" \
      format template HA
  CreateExtendingFrame N_HA_PART phaser_EP_ha_part \
      "Add another anomalous atom site" "Add another anomalous atom site" \
      [list HA_ATOM_TYPE_PART HA_ATOM_FRAC_PART HA_ATOM_X_PART HA_ATOM_Y_PART HA_ATOM_Z_PART HA_ATOM_O_PART HA_ATOM_B_PART ] 

  CloseSubFrame

  OpenSubFrame frame \
      -toggle_display HA_SITES_OPTION_PART open PDB hide 
  CreateInputFileLine line \
      "Select the pdb file" "PDB file" HA_FILE_PDB_PART DIR_HA_FILE_PDB_PART
  CloseSubFrame

  OpenSubFrame frame \
     -toggle_display HA_SITES_OPTION_PART open HA hide 
  CreateInputFileLine line \
     "Select the heavy atom file" "HA   file"  HA_FILE_HA_PART DIR_HA_FILE_HA_PART
  CloseSubFrame

  OpenSubFrame frame \
      -toggle_display HA_SITES_OPTION_PART open SOL hide 
  CreateInputFileLine line \
      "Select the sol file" "SOL file" HA_FILE_SOL_PART  DIR_HA_FILE_SOL_PART
  CloseSubFrame

  CloseSubFrame

  CloseSubFrame

#========================================================================
# The "Composition of the asymmetric unit" folder

  OpenFolder 3 

  CreateLine line \
      message "Choose how to define composition of asymmetric unit" \
      label "Total scattering determined by "\
      widget SCATTERING \
      toggle_display SCATTERING hide [list SOLVENT NRES ] 

  CreateLine line \
      message "Choose how to define composition of asymmetric unit" \
      label "Total scattering determined by "\
      widget SCATTERING \
      message "Enter fractional solvent content" \
      widget SCATTERING_SOLVENT  -oblig \
      toggle_display SCATTERING open SOLVENT 

  CreateLine line \
      message "Choose how to define composition of asymmetric unit" \
      label "Total scattering determined by "\
      widget SCATTERING \
      message "Enter number of amino acid residues" \
      widget SCATTERING_NRES -oblig \
      toggle_display SCATTERING open NRES 

  OpenSubFrame frame -toggle_display SCATTERING open COMPONENT hide

  CreateExtendingFrame N_COMPONENT phaser_EP_composition \
      "Define another protein or nucleic acid component" "Define another component" \
      [list PROTDNA  MW  ASYM COMP_OPTION COMP_FILE DIR_COMP_FILE ]

  CloseSubFrame

#======================================================================== 
# The "Macrocycles" folder

  OpenFolder 4 closed 

  CreateLineTemplate SAD_MACROCYCLE_LABEL \
    { 0 0.067 0.134 0.201 0.268 0.335 0.402 0.469 0.536 0.603 0.670 0.737 0.804 0.871 0.92 }

  CreateLineTemplate SAD_MACROCYCLE_TOG \
    { 0.0 0.05 0.1 0.15 0.2 0.25 0.3 0.35 0.4 0.45 0.5 0.55 0.6 0.65 0.7 } 

  CreateLineTemplate SAD_MACROCYCLE_PART_LABEL \
    { 0 0.059 0.118 0.176 0.235 0.294 0.353 0.412 0.471 0.529 0.588 0.647 0.706 0.765 0.824 0.882 0.941}

  CreateLineTemplate SAD_MACROCYCLE_PART_TOG \
    { 0.0 0.05 0.1 0.15 0.2 0.25 0.3 0.35 0.4 0.45 0.5 0.55 0.6 0.65 0.7 0.75 0.8} 

  OpenSubFrame frame \
    -toggle_display EP_MODE open SAD

    CreateLine line \
      message "SAD refinement macrocyles" \
      widget TOG_MACSAD -toggleon label "SAD refinement" \
      widget MACSAD_OPTION 

    OpenSubFrame frame \
      -toggle_display MACSAD_OPTION open ENTERED hide
      CreateLine line \
        label "K" label "B" label "SIG" label "XYZ" label "OCC" label "BFAC" label "FDP" \
        label "SA" label "SB" label "SP" label "SD" label "NCYC" \
        format template SAD_MACROCYCLE_LABEL
     CreateExtendingFrame N_MACSAD phaser_EP_sadmacro \
        "Customize the SAD refinement" \
        "Add another SAD macrocycle" \
        [list TOG_MACSAD_K TOG_MACSAD_B TOG_MACSAD_XYZ TOG_MACSAD_OCC TOG_MACSAD_BFAC TOG_MACSAD_FDP \
              TOG_MACSAD_SA TOG_MACSAD_SB TOG_MACSAD_SP TOG_MACSAD_SD MACSAD_NCYCL ] 
    CloseSubFrame 

  CloseSubFrame 
  
  OpenSubFrame frame \
    -toggle_display EP_MODE open MRSAD

    CreateLine line \
      message "SAD refinement macrocyles" \
      widget TOG_MACS_PART -toggleon label "SAD refinement" \
      widget MACS_PART_OPTION 

    OpenSubFrame frame \
      -toggle_display MACS_PART_OPTION open ENTERED hide
      CreateLine line \
        label "K" label "B" label "SIG" label "XYZ" label "OCC" label "BFAC" label "FDP" \
        label "SA" label "SB" label "SP" label "SD" label "PK" label "PB" label "NCYC" \
        format template SAD_MACROCYCLE_PART_LABEL
     CreateExtendingFrame N_MACS_PART phaser_EP_sadmacro_part \
        "Customize the SAD refinement" \
        "Add another SAD macrocycle" \
        [list TOG_MACS_PART_K TOG_MACS_PART_B TOG_MACS_PART_XYZ TOG_MACS_PART_OCC TOG_MACS_PART_BFAC TOG_MACS_PART_FDP \
              TOG_MACS_PART_SA TOG_MACS_PART_SB TOG_MACS_PART_SP TOG_MACS_PART_SD TOG_MACS_PART_PK TOG_MACS_PART_PB MACS_PART_NCYCL ] 
    CloseSubFrame 

  CloseSubFrame 
#======================================================================== 
# The "Additional parameters" folder

  OpenFolder 5 closed 

  CreateLine line \
      message "Default is to apply an anisotropy correction" \
      message "Anisotropy correction" \
      widget TOG_ANISO -toggleon label "Anisotropy correction"  widget ANISO_ONOFF

  CreateLine line \
      message "Controls output of PDB files" \
      widget TOG_XYZOUT -toggleon label "Coordinate (PDB) file output"  widget XYZOUT_ONOFF

  CreateLine line \
      message "Controls output of MTZ files" \
      widget TOG_HKLOUT -toggleon label "Reflection (MTZ) file output" widget HKLOUT_ONOFF

  CreateLine line \
      message "Controls output of SOL files" \
      widget TOG_SCRIPT -toggleon label "Solution (SOL) file output" widget SCRIPT_ONOFF

  CreateLine line \
      message "Reject unlikely E-values. Default is ON with probability 0.00001" \
      widget TOG_OUTLIER -toggleon label "Outlier rejection" widget OUTLIER_OPTION \
      label "outlier probability" \
      message "Outliers with a probability less than the cutoff are rejected" \
      widget OUTLIER_PROB \
      toggle_display OUTLIER_OPTION open ON

  CreateLine line \
      message "Reject unlikely E-values. Default is ON with probability 0.00001" \
      widget TOG_OUTLIER -toggleon label "Outlier rejection" widget OUTLIER_OPTION \
      toggle_display OUTLIER_OPTION open OFF

  CreateLine line \
      message "Restrain B-factors to Wilson B-factor" \
      widget TOG_WILSON -toggleon label "B-factor Wilson Restraint" widget WILSON_OPTION \
      label "Sigma of restraint " \
      widget WILSON_SIGMA \
      toggle_display WILSON_OPTION open ON

  CreateLine line \
      message "Restrain B-factors to Wilson B-factor" \
      widget TOG_WILSON -toggleon label "B-factor Wilson Restraint" widget WILSON_OPTION \
      toggle_display WILSON_OPTION open OFF

  CreateLine line \
      message "Restrain Anisotropic B-factors to be Isotropic" \
      widget TOG_SPHERICITY -toggleon label "B-factor Sphericity Restraint" widget SPHERICITY_OPTION \
      label "Sigma of restraint " \
      widget SPHERICITY_SIGMA \
      toggle_display SPHERICITY_OPTION open ON

  CreateLine line \
      message "Restrain Anisotropic B-factors to be Isotropic" \
      widget TOG_SPHERICITY -toggleon label "B-factor Sphericity Restraint" widget SPHERICITY_OPTION \
      toggle_display SPHERICITY_OPTION open OFF

  CreateLine line \
      message "VERBOSE: more information included in log file than by default" \
      widget TOG_VERBOSE \
      label "Verbose output to log file" \
      message "Verbose extra option: even more information included in log file" \
      widget TOG_VERBOSE_EXTRA \
      label "extra"

 }
